@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-08-10
+ *  Updated   : 2005-07-16
  *  Notes     :
  *
- * Copyright (c) 1997-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,6 +45,9 @@
 #ifndef RFC2046_H
 #	include <rfc2046.h>
 #endif /* !RFC2046_H */
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
 
 /*
  * The prototypes bracketed by DECL_xxxx ifdef's are used to get moderately
@@ -75,6 +78,9 @@
 #ifdef DECL_CALLOC
 	extern void *calloc(size_t, size_t);
 #endif /* DECL_CALLOC */
+#ifdef DECL_FCHMOD
+	extern int fchmod(int, mode_t);
+#endif /* DECL_FCHMOD */
 #ifdef DECL_FCLOSE
 	extern int fclose(FILE *);
 #endif /* DECL_FCLOSE */
@@ -235,9 +241,6 @@
 #ifdef DECL_SYSTEM
 	extern int system(const char *);
 #endif /* DECL_SYSTEM */
-#ifdef DECL_TEMPNAM
-	extern char *tempnam(const char *, const char *);
-#endif /* DECL_TEMPNAM */
 #ifdef DECL_TGETENT
 	extern int tgetent(char *, char *);
 #endif /* DECL_TGETENT */
@@ -328,10 +331,9 @@ extern char **news_headers_to_display_array;
 extern char **news_headers_to_not_display_array;
 extern char *OPT_CHAR_list[];
 extern char *OPT_STRING_list[];
-extern char *ch_post_process;
 extern char *nntp_server;
-extern char *xover_cmd;
 extern char active_times_file[PATH_LEN];
+extern char article_name[PATH_LEN];
 extern char bug_addr[LEN];
 extern char bug_nntpserver1[PATH_LEN];
 extern char bug_nntpserver2[PATH_LEN];
@@ -348,7 +350,6 @@ extern char index_maildir[PATH_LEN];
 extern char index_newsdir[PATH_LEN];
 extern char index_savedir[PATH_LEN];
 extern char inewsdir[PATH_LEN];
-extern char last_put[];
 extern char libdir[PATH_LEN];
 extern char local_attributes_file[PATH_LEN];
 extern char local_config_file[PATH_LEN];
@@ -357,14 +358,13 @@ extern char local_input_history_file[PATH_LEN];
 extern char local_newsgroups_file[PATH_LEN];
 extern char local_newsrctable_file[PATH_LEN];
 extern char lock_file[PATH_LEN];
-extern char mail_active_file[PATH_LEN];
 extern char mail_news_user[LEN];
 extern char mailbox[PATH_LEN];
 extern char mailer[PATH_LEN];
 #ifdef HAVE_MH_MAIL_HANDLING
+	extern char mail_active_file[PATH_LEN];
 	extern char mailgroups_file[PATH_LEN];
 #endif /* HAVE_MH_MAIL_HANDLING */
-extern char mesg[LEN];
 extern char newnewsrc[PATH_LEN];
 extern char news_active_file[PATH_LEN];
 extern char newsgroups_file[PATH_LEN];
@@ -385,16 +385,18 @@ extern char txt_help_bug_report[LEN];
 extern char userid[PATH_LEN];
 
 extern char domain_name[];
-extern char host_name[];
 
 extern const char base64_alphabet[64];
 
 extern constext *content_encodings[];
 extern constext *content_types[];
 extern constext *txt_attrs[];
-extern constext *txt_colors[];
+#ifdef HAVE_COLOR
+	extern constext *txt_colors[];
+#endif /* HAVE_COLOR */
 extern constext *txt_confirm_choices[];
 extern constext *txt_hide_uue_type[];
+extern constext *txt_interactive_mailers[];
 extern constext *txt_kill_level_type[];
 #ifdef CHARSET_CONVERSION
 	extern constext *txt_mime_charsets[];	/* supported charsets */
@@ -403,6 +405,9 @@ extern constext *txt_mime_7bit_charsets[]; /* 7bit charsets */
 extern constext *txt_mailbox_formats[];
 extern constext *txt_marks[];
 extern constext *txt_mime_encodings[];
+#ifdef HAVE_UNICODE_NORMALIZATION
+	extern constext *txt_normalization_forms[];
+#endif /* HAVE_UNICODE_NORMALIZATION */
 extern constext *txt_onoff[];
 extern constext *txt_post_process_type[];
 extern constext *txt_quote_style_type[];
@@ -476,9 +481,9 @@ extern constext txt_cannot_create[];
 extern constext txt_cannot_create_uniq_name[];
 extern constext txt_cannot_find_base_art[];
 extern constext txt_cannot_get_nntp_server_name[];
-#if defined(M_UNIX) && !defined(USE_TERMINFO)
+#if !defined(USE_CURSES) && defined(M_UNIX) && !defined(USE_TERMINFO)
 	extern constext txt_cannot_get_term_entry[];
-#endif /* M_UNIX && !USE_TERMINFO */
+#endif /* !USE_CURSES && M_UNIX && !USE_TERMINFO */
 extern constext txt_cannot_open[];
 extern constext txt_cannot_open_for_saving[];
 extern constext txt_cannot_post[];
@@ -494,12 +499,14 @@ extern constext txt_caughtup[];
 extern constext txt_check_article[];
 extern constext txt_checking_for_news[];
 extern constext txt_checking_new_groups[];
-#ifndef HAVE_LIBUU
+#if !defined(HAVE_LIBUU) && defined(M_UNIX) && defined(HAVE_SUM) && !defined(DONT_HAVE_PIPING)
 	extern constext txt_checksum_of_file[];
-#endif /* !HAVE_LIBUU */
+#endif /* !HAVE_LIBUU && M_UNIX && HAVE SUM && !DONT_HAVE_PIPING */
 extern constext txt_choose_post_process_type[];
-extern constext txt_color_off[];
-extern constext txt_color_on[];
+#ifdef HAVE_COLOR
+	extern constext txt_color_off[];
+	extern constext txt_color_on[];
+#endif /* HAVE_COLOR */
 extern constext txt_command_failed[];
 extern constext txt_confirm_select_on_exit[];
 #ifdef NNTP_ABLE
@@ -625,6 +632,8 @@ extern constext txt_filter_score_help[];
 extern constext txt_filter_text_type[];
 extern constext txt_followup_newsgroups[];
 extern constext txt_followup_poster[];
+extern constext txt_forwarded[];
+extern constext txt_forwarded_end[];
 extern constext txt_from_line_only[];
 extern constext txt_from_line_only_case[];
 extern constext txt_full[];
@@ -635,8 +644,10 @@ extern constext txt_gethostbyname[];
 extern constext txt_global[];
 extern constext txt_group[];
 extern constext txt_group_aliased[];
+extern constext txt_group_bogus[];
 extern constext txt_group_is_moderated[];
 extern constext txt_group_plural[];
+extern constext txt_group_rereading[];
 extern constext txt_group_select_com[];
 extern constext txt_group_selection[];
 extern constext txt_group_singular[];
@@ -666,7 +677,6 @@ extern constext txt_help_article_prev_unread[];
 extern constext txt_help_article_quick_kill[];
 extern constext txt_help_article_quick_select[];
 extern constext txt_help_article_quit_to_select_level[];
-extern constext txt_help_article_read_next_unread[];
 extern constext txt_help_article_reply[];
 extern constext txt_help_article_reply_no_quote[];
 extern constext txt_help_article_reply_with_header[];
@@ -692,6 +702,7 @@ extern constext txt_help_filter_text[];
 extern constext txt_help_filter_text_type[];
 extern constext txt_help_filter_time[];
 extern constext txt_help_global_article_range[];
+extern constext txt_help_global_edit_filter[];
 extern constext txt_help_global_esc[];
 extern constext txt_help_global_help[];
 extern constext txt_help_global_last_art[];
@@ -716,6 +727,8 @@ extern constext txt_help_global_quit_tin[];
 extern constext txt_help_global_redraw_screen[];
 extern constext txt_help_global_save[];
 extern constext txt_help_global_auto_save[];
+extern constext txt_help_global_scroll_down[];
+extern constext txt_help_global_scroll_up[];
 extern constext txt_help_global_search_auth_backwards[];
 extern constext txt_help_global_search_auth_forwards[];
 extern constext txt_help_global_search_body[];
@@ -733,11 +746,11 @@ extern constext txt_help_global_tag[];
 extern constext txt_help_global_toggle_info_line[];
 extern constext txt_help_global_toggle_inverse_video[];
 extern constext txt_help_global_toggle_mini_help[];
+extern constext txt_help_global_toggle_subj_display[];
 extern constext txt_help_global_version[];
 extern constext txt_help_group_catchup[];
 extern constext txt_help_group_catchup_next[];
 extern constext txt_help_group_first_thread[];
-extern constext txt_help_group_goto_group[];
 extern constext txt_help_group_last_thread[];
 extern constext txt_help_group_list_thread[];
 extern constext txt_help_group_mark_article_unread[];
@@ -745,12 +758,7 @@ extern constext txt_help_group_mark_thread_read[];
 extern constext txt_help_group_mark_thread_unread[];
 extern constext txt_help_group_mark_unsel_art_read[];
 extern constext txt_help_group_next[];
-extern constext txt_help_group_next_unread_art[];
-extern constext txt_help_group_next_unread_article[];
 extern constext txt_help_group_prev[];
-extern constext txt_help_group_prev_unread_art[];
-extern constext txt_help_group_read_article[];
-extern constext txt_help_group_repost[];
 extern constext txt_help_group_reverse_thread_selection[];
 extern constext txt_help_group_select_all[];
 extern constext txt_help_group_select_thread[];
@@ -760,7 +768,6 @@ extern constext txt_help_group_tag_parts[];
 extern constext txt_help_group_thread_by_num[];
 extern constext txt_help_group_toggle_getart_limit[];
 extern constext txt_help_group_toggle_read_articles[];
-extern constext txt_help_group_toggle_subj_display[];
 extern constext txt_help_group_toggle_thread_selection[];
 extern constext txt_help_group_toggle_threading[];
 extern constext txt_help_group_undo_thread_selection[];
@@ -800,7 +807,6 @@ extern constext txt_help_thread_first_article[];
 extern constext txt_help_thread_last_article[];
 extern constext txt_help_thread_mark_article_read[];
 extern constext txt_help_thread_read_article[];
-extern constext txt_help_thread_toggle_subj_display[];
 extern constext txt_help_title_disp[];
 extern constext txt_help_title_misc[];
 extern constext txt_help_title_navi[];
@@ -829,10 +835,7 @@ extern constext txt_is_tex_encoded[];
 extern constext txt_keymap_missing_key[];
 extern constext txt_keymap_invalid_key[];
 extern constext txt_keymap_invalid_name[];
-#ifdef DEBUG
-	extern constext txt_keymap_redef[];
-#endif /* DEBUG */
-extern constext txt_keymap_conflict[];
+extern constext txt_keymap_upgraded[];
 extern constext txt_kill_from[];
 extern constext txt_kill_lines[];
 extern constext txt_kill_menu[];
@@ -904,16 +907,17 @@ extern constext txt_no_newsgroups[];
 extern constext txt_no_next_unread_art[];
 extern constext txt_no_prev_group[];
 extern constext txt_no_prev_unread_art[];
-extern constext txt_no_resp[];
 extern constext txt_no_responses[];
 extern constext txt_no_resps_in_thread[];
 extern constext txt_no_search_string[];
 extern constext txt_no_subject[];
-extern constext txt_no_term_clear_eol[];
-extern constext txt_no_term_clear_eos[];
-extern constext txt_no_term_clearscreen[];
-extern constext txt_no_term_cursor_motion[];
-extern constext txt_no_term_set[];
+#ifndef USE_CURSES
+	extern constext txt_no_term_clear_eol[];
+	extern constext txt_no_term_clear_eos[];
+	extern constext txt_no_term_clearscreen[];
+	extern constext txt_no_term_cursor_motion[];
+	extern constext txt_no_term_set[];
+#endif /* !USE_CURSES */
 extern constext txt_no_viewer_found[];
 extern constext txt_no_xover_support[];
 extern constext txt_not_exist[];
@@ -980,6 +984,9 @@ extern constext txt_reading_newsrc[];
 #ifdef NNTP_ABLE
 	extern constext txt_reconnect_to_news_server[];
 #endif /* NNTP_ABLE */
+#ifdef HAVE_GETTIMEOFDAY
+	extern constext txt_remaining[];
+#endif /* HAVE_GETTIMEOFDAY */
 extern constext txt_remove_bogus[];
 extern constext txt_rename_error[];
 extern constext txt_reply_to_author[];
@@ -1004,7 +1011,9 @@ extern constext txt_saved_to[];
 extern constext txt_saved_to_range[];
 extern constext txt_saving[];
 extern constext txt_screen_init_failed[];
-extern constext txt_screen_too_small[];
+#ifndef USE_CURSES
+	extern constext txt_screen_too_small[];
+#endif /* !USE_CURSES */
 extern constext txt_screen_too_small_exiting[];
 extern constext txt_search_backwards[];
 extern constext txt_search_body[];
@@ -1046,12 +1055,15 @@ extern constext txt_supersede_group[];
 extern constext txt_superseding_art[];
 extern constext txt_suspended_message[];
 extern constext txt_tab[];
+extern constext txt_tex[];
 extern constext txt_there_is_no_news[];
 extern constext txt_thread_upper[];
 extern constext txt_thread_com[];
 extern constext txt_thread_marked_as_deselected[];
 extern constext txt_thread_marked_as_selected[];
-extern constext txt_thread_plural[];
+#if 0 /* unused */
+	extern constext txt_thread_plural[];
+#endif /* 0 */
 extern constext txt_thread_range[];
 extern constext txt_thread_singular[];
 extern constext txt_thread_x_of_n[];
@@ -1198,16 +1210,13 @@ extern constext txt_uu_success[];
 #endif /* !NO_SHELL_ESCAPE */
 
 extern int *my_group;
-extern int MORE_POS;
 extern int NOTESLINES;
-extern int RIGHT_POS;
 extern int _hp_glitch;
 extern int cCOLS;
 extern int cLINES;
 extern int curr_line;
 extern int debug;
-extern int groupname_len;
-extern int i_key_search_last;
+extern int input_context;
 extern int iso2asc_supported;
 extern int last_resp;
 extern int max_active;
@@ -1233,11 +1242,9 @@ extern int xmouse;
 extern int xrow;
 
 extern long *base;
-extern long head_next;
 
 extern signed long int read_newsrc_lines;
 
-extern mode_t real_umask;
 extern pid_t process_id;
 
 extern struct regex_cache strip_re_regex;
@@ -1267,7 +1274,9 @@ extern struct t_group *curr_group;
 extern struct t_newnews *newnews;
 extern struct t_option option_table[];
 extern struct t_save *save;
+extern struct t_capabilities nntp_caps;
 
+extern t_bool *OPT_ON_OFF_list[];
 extern t_bool can_post;
 extern t_bool check_for_new_newsgroups;
 extern t_bool cmd_line;
@@ -1285,7 +1294,6 @@ extern t_bool newsrc_active;
 extern t_bool no_write;
 extern t_bool post_article_and_exit;
 extern t_bool post_postponed_and_exit;
-extern t_bool quitting;
 extern t_bool read_local_newsgroups_file;
 extern t_bool read_news_via_nntp;
 extern t_bool read_saved_news;
@@ -1297,6 +1305,8 @@ extern t_bool batch_mode;
 extern t_bool verbose;
 extern t_bool xref_supported;
 
+extern t_function last_search;
+
 extern t_menu selmenu;
 extern t_menu grpmenu;
 extern t_menu *currmenu;
@@ -1431,12 +1441,6 @@ extern constext txt_processing_saved_arts[];
 	extern constext txt_reading_mailgroups_file[];
 #endif /* HAVE_MH_MAIL_HANDLING */
 
-#ifdef M_AMIGA
-	extern constext txt_env_var_not_found[];
-	extern constext txt_usage_bbs_mode[];
-	extern t_bool tin_bbs_mode;
-#endif /* M_AMIGA */
-
 #ifndef NO_ETIQUETTE
 	extern constext txt_warn_posting_etiquette[];
 #endif /* NO_ETIQUETTE */
@@ -1473,8 +1477,11 @@ extern struct opttxt txt_batch_save;
 extern struct opttxt txt_beginner_level;
 extern struct opttxt txt_cache_overview_files;
 extern struct opttxt txt_catchup_read_groups;
-extern struct opttxt txt_color_options;
+#ifdef HAVE_COLOR
+	extern struct opttxt txt_color_options;
+#endif /* HAVE_COLOR */
 extern struct opttxt txt_confirm_choice;
+extern struct opttxt txt_date_format;
 extern struct opttxt txt_display_options;
 extern struct opttxt txt_draw_arrow;
 extern struct opttxt txt_editor_format;
@@ -1488,6 +1495,7 @@ extern struct opttxt txt_group_catchup_on_exit;
 extern struct opttxt txt_groupname_max_length;
 extern struct opttxt txt_hide_uue;
 extern struct opttxt txt_inews_prog;
+extern struct opttxt txt_interactive_mailer;
 extern struct opttxt txt_inverse_okay;
 extern struct opttxt txt_keep_dead_articles;
 extern struct opttxt txt_kill_level;
@@ -1504,7 +1512,9 @@ extern struct opttxt txt_mono_markstar;
 extern struct opttxt txt_mono_markdash;
 extern struct opttxt txt_mono_markslash;
 extern struct opttxt txt_mono_markstroke;
-extern struct opttxt txt_mm_charset;
+#ifndef CHARSET_CONVERSION
+	extern struct opttxt txt_mm_charset;
+#endif /* CHARSET_CONVERSION */
 extern struct opttxt txt_metamail_prog;
 extern struct opttxt txt_news_headers_to_display;
 extern struct opttxt txt_news_headers_to_not_display;
@@ -1517,8 +1527,10 @@ extern struct opttxt txt_post_process;
 extern struct opttxt txt_post_process_view;
 extern struct opttxt txt_posted_articles_file;
 extern struct opttxt txt_posting_options;
-extern struct opttxt txt_print_header;
-extern struct opttxt txt_printer;
+#ifndef DISABLE_PRINTING
+	extern struct opttxt txt_print_header;
+	extern struct opttxt txt_printer;
+#endif /* !DISABLE_PRINTING */
 extern struct opttxt txt_process_only_unread;
 extern struct opttxt txt_prompt_followupto;
 extern struct opttxt txt_quote_chars;
@@ -1557,12 +1569,13 @@ extern struct opttxt txt_strokes_regex;
 extern struct opttxt txt_tab_goto_next_unread;
 extern struct opttxt txt_tex2iso_conv;
 extern struct opttxt txt_thread_articles;
+extern struct opttxt txt_thread_perc;
 extern struct opttxt txt_thread_catchup_on_exit;
 extern struct opttxt txt_thread_score;
 extern struct opttxt txt_underscores_regex;
 extern struct opttxt txt_unlink_article;
 extern struct opttxt txt_url_handler;
-extern struct opttxt txt_use_mailreader_i;
+extern struct opttxt txt_url_highlight;
 extern struct opttxt txt_use_mouse;
 extern struct opttxt txt_wildcard;
 extern struct opttxt txt_word_highlight;
@@ -1596,6 +1609,7 @@ extern struct opttxt txt_xpost_quote_format;
 	extern struct opttxt txt_col_from;
 	extern struct opttxt txt_col_title;
 	extern struct opttxt txt_col_signature;
+	extern struct opttxt txt_col_urls;
 	extern struct opttxt txt_col_markstar;
 	extern struct opttxt txt_col_markdash;
 	extern struct opttxt txt_col_markslash;
@@ -1610,4 +1624,10 @@ extern struct opttxt txt_xpost_quote_format;
 #ifdef XFACE_ABLE
 	extern struct opttxt txt_use_slrnface;
 #endif /* XFACE_ABLE */
+#ifdef HAVE_UNICODE_NORMALIZATION
+	extern struct opttxt txt_normalization_form;
+#endif /* HAVE_UNICODE_NORMALIZATION */
+#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
+	extern struct opttxt txt_render_bidi;
+#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
 #endif /* !EXTERN_H */
