@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2003-08-26
+ *  Updated   : 2005-06-21
  *  Notes     :
  *
- * Copyright (c) 1997-2003 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2005 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,13 @@
 #ifndef PROTO_H
 #	define PROTO_H 1
 
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
+#ifndef OPTIONS_MENU_H
+#	include "options_menu.h"
+#endif /* !OPTIONS_MENU_H */
+
 /* This fixes ambiguities on platforms that don't distinguish extern case */
 #ifdef CASE_PROBLEM
 #	define Raw tin_raw
@@ -47,7 +54,6 @@
 /* active.c */
 extern char group_flag(char ch);
 extern int find_newnews_index(const char *cur_newnews_host);
-extern int get_active_num(void);
 extern t_bool match_group_list(const char *group, const char *group_list);
 extern t_bool parse_active_line(char *line, long *max, long *min, char *moderated);
 extern t_bool process_bogus(char *name);
@@ -66,7 +72,7 @@ extern void make_threads(struct t_group *group, t_bool rethread);
 extern void set_article(struct t_article *art);
 extern void show_art_msg(const char *group);
 extern void sort_arts(unsigned /* int */ sort_art_type);
-extern void write_nov_file(struct t_group *group);
+extern void write_overview(struct t_group *group);
 
 /* attrib.c */
 extern void read_attributes_files(void);
@@ -79,8 +85,8 @@ extern void write_attributes_file(const char *file);
 
 /* charset.c */
 extern char *convert_to_printable(char *buf);
+extern char *convert_body2printable(char* buf);
 extern t_bool is_art_tex_encoded(FILE *fp);
-extern void convert_body2printable(char* buf);
 extern void convert_iso2asc(char *iso, char **asc_buffer, int *max_line_len, int t);
 extern void convert_tex2iso(char *from, char *to);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -88,19 +94,16 @@ extern void convert_tex2iso(char *from, char *to);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* color.c */
+extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
 #ifdef HAVE_COLOR
 	extern void bcol(int color);
 	extern void fcol(int color);
 #endif /* HAVE_COLOR */
-extern void draw_pager_line(const char *str, int flags);
 
 /* config.c */
 extern char **ulBuildArgv(char *cmd, int *new_argc);
-extern char *fmt_option_prompt(char *dst, int len, t_bool editing, int option);
 extern char *quote_space_to_dash(char *str);
 extern const char *print_boolean(t_bool value);
-extern int change_config_file(struct t_group *group);
-extern int option_row(int option);
 extern t_bool match_boolean(char *line, const char *pat, t_bool *dst);
 extern t_bool match_integer(char *line, const char *pat, int *dst, int maxval);
 extern t_bool match_list(char *line, constext *pat, constext *const *table, size_t tablelen, int *dst);
@@ -109,8 +112,6 @@ extern t_bool match_string(char *line, const char *pat, char *dst, size_t dstlen
 extern t_bool read_config_file(char *file, t_bool global_file);
 extern void quote_dash_to_space(char *str);
 extern void read_server_config(void);
-extern void refresh_config_page(int act_option);
-extern void show_menu_help(const char *help_message);
 extern void write_config_file(char *file);
 
 /* cook.c */
@@ -123,22 +124,11 @@ extern OUTC_RETTYPE outchar(OUTC_ARGS);
 extern int InitScreen(void);
 extern int RawState(void);
 extern int ReadCh(void);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && !defined(USE_CURSES)
-	extern wint_t ReadWch(void);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE && !USE_CURSES */
 extern int get_arrow_key(int prech);
 extern int get_termcaps(void);
-extern void ClearScreen(void);
-extern void CleartoEOLN(void);
-extern void CleartoEOS(void);
-extern void ScrollScreen(int lines_to_scroll);
-extern void SetScrollRegion(int topline, int bottomline);
 extern void EndInverse(void);
 extern void EndWin(void);
 extern void InitWin(void);
-#ifndef USE_CURSES
-	extern void MoveCursor(int row, int col);
-#endif /* USE_CURSES */
 extern void Raw(int state);
 extern void StartInverse(void);
 extern void cursoroff(void);
@@ -150,6 +140,17 @@ extern void set_xclick_off(void);
 extern void set_xclick_on(void);
 extern void setup_screen(void);
 extern void word_highlight_string(int row, int col, int size, int color);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && !defined(USE_CURSES)
+	extern wint_t ReadWch(void);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE && !USE_CURSES */
+#ifndef USE_CURSES
+	extern void ClearScreen(void);
+	extern void CleartoEOLN(void);
+	extern void CleartoEOS(void);
+	extern void MoveCursor(int row, int col);
+	extern void ScrollScreen(int lines_to_scroll);
+	extern void SetScrollRegion(int topline, int bottomline);
+#endif /* USE_CURSES */
 #if 0
 	extern void ToggleInverse(void);
 #endif /* 0 */
@@ -157,12 +158,15 @@ extern void word_highlight_string(int row, int col, int size, int color);
 /* debug.c */
 #ifdef DEBUG
 	extern void debug_delete_files(void);
-	extern void debug_nntp(const char *func, const char *line);
+	extern void debug_nntp(const char *func, const char *fmt, ...);
 	extern void debug_print_active(void);
 	extern void debug_print_arts(void);
 	extern void debug_print_filters(void);
 	extern void debug_print_header(struct t_article *s);
-	extern void vDbgPrintMalloc(int iIsMalloc, const char *pcFile, int iLine, size_t iSize);
+	extern void debug_print_malloc(int is_malloc, const char *xfile, int line, size_t size);
+#	ifdef NNTP_ABLE
+		extern void debug_print_nntp_extensions(void);
+#	endif /* NNTP_ABLE */
 #endif /* DEBUG */
 #ifdef DEBUG_NEWSRC
 	extern void debug_print_newsrc(struct t_newsrc *NewSrc, FILE *fp);
@@ -180,8 +184,8 @@ extern void feed_articles(int function, int level, struct t_group *group, int re
 
 /* filter.c */
 extern t_bool filter_articles(struct t_group *group);
-extern t_bool filter_menu(int type, struct t_group *group, struct t_article *art);
-extern t_bool quick_filter(int type, struct t_group *group, struct t_article *art);
+extern t_bool filter_menu(t_function type, struct t_group *group, struct t_article *art);
+extern t_bool quick_filter(t_function type, struct t_group *group, struct t_article *art);
 extern t_bool quick_filter_select_posted_art(struct t_group *group, const char *subj, const char *a_message_id);
 extern t_bool read_filter_file(const char *file);
 extern void free_filter_array(struct t_filters *ptr);
@@ -191,6 +195,7 @@ extern void write_filter_file(const char *filename);
 
 /* getline.c */
 extern char *tin_getline(const char *prompt, int number_only, const char *str, int max_chars, t_bool passwd, int which_hist);
+extern void gl_redraw(void);
 
 /* global.c */
 extern void bug_report(void);
@@ -202,17 +207,17 @@ extern void top_of_list(void);
 extern void end_of_list(void);
 extern void move_to_item(int n);
 extern void prompt_item_num(int ch, const char *prompt);
+extern void scroll_down(void);
+extern void scroll_up(void);
 extern void set_first_screen_item(void);
 
 /* group.c */
 extern int find_new_pos(int old_top, long old_artnum, int cur_pos);
 extern int group_page(struct t_group *group);
 extern void clear_note_area(void);
-extern void mark_screen(int level, int screen_row, int screen_col, const char *value);
+extern void mark_screen(int screen_row, int screen_col, const char *value);
 extern void pos_first_unread_thread(void);
-extern void set_subj_from_size(int num_cols);
 extern void show_group_page(void);
-extern void toggle_subject_from(void);
 
 /* hashstr.c */
 extern char *hash_str(const char *s);
@@ -238,7 +243,6 @@ extern void get_from_name(char *from_name, struct t_group *thisgrp);
 extern void get_user_info(char *user_name, char *full_name);
 
 /* init.c */
-extern t_bool(*wildcard_func)(const char *str, char *patt, t_bool icase);		/* Wildcard matching function */
 extern void init_selfinfo(void);
 extern void postinit_regexp(void);
 #ifdef HAVE_COLOR
@@ -251,11 +255,13 @@ extern void joinpath(char *result, const char *dir, const char *file);
 /* keymap.c */
 extern char *printascii(char *buf, int ch);
 extern t_bool read_keymap_file(void);
-extern void build_keymaps(void);
 extern void free_keymaps(void);
+extern void setup_default_keys(void);
 
 /* langinfo.c */
-extern char *tin_nl_langinfo(nl_item item);
+#ifndef NO_LOCALE
+	extern const char *tin_nl_langinfo(nl_item item);
+#endif /* !NO_LOCALE */
 
 /* list.c */
 extern char *random_organization(char *in_org);
@@ -288,10 +294,6 @@ extern void grp_del_mail_art(struct t_article *article);
 	extern void write_mail_active_file(void);
 #endif /* HAVE_MH_MAIL_HANDLING */
 
-/* mimetypes.c */
-extern void lookup_mimetype(const char *ext, t_part *part);
-extern t_bool lookup_extension(char *extension, size_t ext_len, const char *major, const char *minor);
-
 /* main.c */
 extern int main(int argc, char *argv[]);
 extern int read_cmd_line_groups(void);
@@ -315,13 +317,19 @@ extern void *my_realloc1(const char *file, int line, void *p, size_t size);
 	extern void init_screen_array(t_bool allocate);
 #endif /* !USE_CURSES */
 
+/* mimetypes.c */
+extern void lookup_mimetype(const char *ext, t_part *part);
+extern t_bool lookup_extension(char *extension, size_t ext_len, const char *major, const char *minor);
+
 /* misc.c */
-extern const char *eat_re(char *s, t_bool eat_was);
+extern char *buffer_to_ascii(char *c);
 extern char *escape_shell_meta(const char *source, int quote_area);
 extern char *get_tmpfilename(const char *filename);
+extern char *idna_decode(char *in);
 extern char *quote_wild(char *str);
 extern char *quote_wild_whitespace(char *str);
 extern char *strip_line(char *line);
+extern const char *eat_re(char *s, t_bool eat_was);
 extern const char *get_val(const char *env, const char *def);
 extern const char *gnksa_strerror(int errcode);
 extern int gnksa_check_from(char *from);
@@ -335,6 +343,7 @@ extern int parse_from(const char *from, char *address, char *realname);
 extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
 extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group);
 extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, char *format);
+extern int tin_version_info(FILE *fp);
 extern long file_mtime(const char *file);
 extern long file_size(const char *file);
 extern t_bool backup_file(const char *filename, const char *backupname);
@@ -349,6 +358,7 @@ extern void cleanup_tmp_files(void);
 extern void copy_body(FILE *fp_ip, FILE *fp_op, char *prefix, char *initl, t_bool raw_data);
 extern void create_index_lock_file(char *the_lock_file);
 extern void dir_name(const char *fullpath, char *dir);
+extern void draw_mark_selected(int i);
 extern void draw_percent_mark(long cur_num, long max_num);
 extern void get_author(t_bool thread, struct t_article *art, char *str, size_t len);
 extern void get_cwd(char *buf);
@@ -361,6 +371,9 @@ extern void show_inverse_video_status(void);
 extern void strip_name(const char *from, char *address);
 extern void tin_done(int ret);
 extern void toggle_inverse_video(void);
+#if defined(CHARSET_CONVERSION) || (defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE))
+	extern char *utf8_valid(char *line);
+#endif /* CHARSET_CONVERSION || (MULTIBYTE_ABLE && !NO_LOCALE) */
 #ifdef CHARSET_CONVERSION
 	extern t_bool buffer_to_network(char *line, int mmnwcharset);
 #endif /* CHARSET_CONVERSION */
@@ -380,6 +393,7 @@ extern void toggle_inverse_video(void);
 #endif /* !NO_SHELL_ESCAPE */
 
 /* newsrc.c */
+extern int group_get_art_info(char *tin_spooldir, char *groupname, int grouptype, long *art_count, long *art_max, long *art_min);
 extern signed long int read_newsrc(char *newsrc_file, t_bool allgroups);
 extern signed long int write_newsrc(void);
 extern t_bool pos_group_in_newsrc(struct t_group *group, int pos);
@@ -396,10 +410,10 @@ extern void subscribe(struct t_group *group, int sub_state, t_bool get_info);
 extern void thd_mark_read(struct t_group *group, long thread);
 extern void thd_mark_unread(struct t_group *group, long thread);
 extern void set_default_bitmap(struct t_group *group);
-extern void art_mark_deleted(struct t_article *art);
-extern void art_mark_undeleted(struct t_article *art);
 #ifdef DEBUG_NEWSRC
-	extern void vNewsrcTestHarness(void);
+#	if 0
+	extern void newsrc_test_harness(void);
+#	endif /* 0 */
 #endif /* DEBUG_NEWSRC */
 
 /* nntplib.c */
@@ -407,40 +421,28 @@ extern FILE *get_nntp_fp(FILE *fp);
 extern FILE *get_nntp_wr_fp(FILE *fp);
 extern char *getserverbyfile(const char *file);
 extern char *get_server(char *string, int size);
-extern int server_init(char *machine, const char *service, int port, char *text, size_t mlen);
-extern void close_server(void);
+extern int get_respcode(char *, size_t);
+extern int get_only_respcode(char *, size_t);
+extern int nntp_open(void);
+extern void nntp_close(void);
 extern void put_server(const char *string);
 extern void u_put_server(const char *string);
-#ifdef DEBUG
-	extern const char *nntp_respcode(int respcode);
-#endif /* DEBUG */
+#ifdef NNTP_ABLE
+	extern FILE *nntp_command(const char *, int, char *, size_t);
+#endif /* NNTP_ABLE */
 
 /* nrctbl.c */
 extern int get_newsrcname(char *newsrc_name, const char *nntpserver_name);
 extern void get_nntpserver(char *nntpserver_name, char *nick_name);
 
-/* open.c */
-extern FILE *open_art_fp(struct t_group *group, long art);
-extern FILE *open_newgroups_fp(int the_index);
-extern FILE *open_news_active_fp(void);
-extern FILE *open_newsgroups_fp(void);
-extern FILE *open_overview_fmt_fp(void);
-extern FILE *open_subscription_fp(void);
-extern FILE *open_art_header(long art);
-extern int get_respcode(char *, size_t);
-extern int get_only_respcode(char *, size_t);
-extern int nntp_open(void);
-extern int group_get_art_info(char *tin_spooldir, char *groupname, int grouptype, long *art_count, long *art_max, long *art_min);
-extern long setup_hard_base(struct t_group *group);
-extern void nntp_close(void);
-extern void vGet1GrpArtInfo(struct t_group *grp);
-#ifdef HAVE_MH_MAIL_HANDLING
-	extern FILE *open_mail_active_fp(const char *mode);
-	extern FILE *open_mailgroups_fp(void);
-#endif /* HAVE_MH_MAIL_HANDLING */
-#ifdef NNTP_ABLE
-	extern FILE *nntp_command(const char *, int, char *, size_t);
-#endif /* NNTP_ABLE */
+/* options_menu.c */
+extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, enum option_enum option);
+extern int change_config_file(struct t_group *group);
+extern int option_row(enum option_enum option);
+extern t_bool option_is_visible(enum option_enum option);
+extern void check_score_defaults(void);
+extern void refresh_config_page(enum option_enum act_option);
+extern void show_menu_help(const char *help_message);
 
 /* page.c */
 extern int show_page(struct t_group *group, int respnum, int *threadnum);
@@ -470,41 +472,41 @@ extern time_t parsedate(char *p, TIMEINFO *now);
 #endif /* !HAVE_VSNPRINTF */
 
 /* post.c */
+extern char *checknadd_headers(const char *infile);
 extern int count_postponed_articles(void);
 extern int mail_to_author(const char *group, int respnum, t_bool copy_text, t_bool with_headers, t_bool raw_data);
 extern int mail_to_someone(const char *address, t_bool confirm_to_mail, t_openartinfo *artinfo, const struct t_group *group);
-extern int post_response(const char *group, int respnum, t_bool copy_text, t_bool with_headers, t_bool raw_data);
-extern int repost_article(const char *group, int respnum, t_bool supersede, t_openartinfo *artinfo);
+extern int post_response(const char *groupname, int respnum, t_bool copy_text, t_bool with_headers, t_bool raw_data);
+extern int repost_article(const char *groupname, int respnum, t_bool supersede, t_openartinfo *artinfo);
 extern t_bool cancel_article(struct t_group *group, struct t_article *art, int respnum);
 extern t_bool mail_bug_report(void);
 extern t_bool pickup_postponed_articles(t_bool ask, t_bool all);
-extern t_bool post_article(const char *group);
+extern t_bool post_article(const char *groupname);
 extern t_bool reread_active_after_posting(void);
 extern t_bool user_posted_messages(void);
-extern void checknadd_headers(const char *infile);
 extern void init_postinfo(void);
 extern void quick_post_article(t_bool postponed_only);
 #ifdef USE_CANLOCK
-	extern const char *build_cankey(const char *messageid, const char *secret);
 	extern const char *build_canlock(const char *messageid, const char *secret);
 	extern char *get_secret(void);
 #endif /* USE_CANLOCK */
 
 /* prompt.c */
 extern char *prompt_string_default(const char *prompt, char *def, const char *failtext, int history);
-extern char *sized_message(const char *format, const char *subject);
-extern int prompt_list(int row, int col, int var, constext *help_text, constext *prompt_text, constext *list[], int size);
+extern char *sized_message(char **result, const char *format, const char *subject);
 extern int prompt_num(int ch, const char *prompt);
-extern int prompt_yn(int line, const char *prompt, t_bool default_answer);
+extern int prompt_yn(const char *prompt, t_bool default_answer);
 extern int prompt_msgid(void);
 extern t_bool prompt_default_string(const char *prompt, char *buf, int buf_len, char *default_prompt, int which_hist);
 extern t_bool prompt_menu_string(int line, const char *prompt, char *var);
-extern t_bool prompt_option_char(int option);
-extern t_bool prompt_option_num(int option);
-extern t_bool prompt_option_string(int option);
+extern t_bool prompt_option_char(enum option_enum option);
+extern t_bool prompt_option_list(enum option_enum option);
+extern t_bool prompt_option_num(enum option_enum option);
+extern t_bool prompt_option_on_off(enum option_enum option);
+extern t_bool prompt_option_string(enum option_enum option);
 extern t_bool prompt_string(const char *prompt, char *buf, int which_hist);
 extern void prompt_continue(void);
-extern void prompt_on_off(int row, int col, t_bool *var, constext *help_text, constext *prompt_text);
+extern void prompt_slk_redraw(void);
 
 /* read.c */
 extern char *tin_fgets(FILE *fp, t_bool header);
@@ -523,7 +525,7 @@ extern void thread_by_reference(void);
 
 /* regex.c */
 extern t_bool compile_regex(const char *regex, struct regex_cache *cache, int options);
-extern t_bool match_regex(const char *string, char *pattern, t_bool icase);
+extern t_bool match_regex(const char *string, char *pattern, struct regex_cache *cache, t_bool icase);
 extern void highlight_regexes(int row, struct regex_cache *regex, int color);
 
 /* rfc1524.c */
@@ -531,12 +533,15 @@ extern t_mailcap *get_mailcap_entry(t_part *part, const char *path);
 extern void free_mailcap(t_mailcap *tmailcap);
 
 /* rfc2045.c */
+extern int read_decoded_base64_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read, char **rest);
+extern int read_decoded_qp_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read);
 extern void rfc1521_encode(char *line, FILE *f, int e);
 
 /* rfc2046.c */
+extern FILE *open_art_fp(struct t_group *group, long art);
 extern const char *get_param(t_param *list, const char *name);
 extern char *parse_header(char *buf, const char *pat, t_bool decode, t_bool structured);
-extern int art_open(t_bool wrap_lines, struct t_article *art, struct t_group *group, t_openartinfo *artinfo, t_bool show_progress_meter);
+extern int art_open(t_bool wrap_lines, struct t_article *art, struct t_group *group, t_openartinfo *artinfo, t_bool show_progress_meter, char *pmesg);
 extern int content_type(char *type);
 extern int parse_rfc822_headers(struct t_header *hdr, FILE *from, FILE *to);
 extern t_part *new_part(t_part *part);
@@ -551,17 +556,20 @@ extern char *rfc1522_decode(const char *s);
 extern char *rfc1522_encode(char *s, const char *charset, t_bool ismail);
 extern int mmdecode(const char *what, int encoding, int delimiter, char *where);
 extern void rfc15211522_encode(const char *filename, constext *mime_encoding, struct t_group *group, t_bool allow_8bit_header, t_bool ismail);
+extern void compose_mail_mime_forwarded(const char *filename, FILE *articlefp, t_bool include_text, struct t_group *group);
+extern void compose_mail_text_plain(const char *filename, struct t_group *group);
 
 /* save.c */
 extern int check_start_save_any_news(int function, t_bool catchup);
 extern t_bool create_path(const char *path);
 extern t_bool expand_save_filename(char *outpath, const char *path);
-extern t_bool post_process_files(int proc_type_ch, t_bool auto_delete);
+extern t_bool post_process_files(t_function proc_type_type, t_bool auto_delete);
 extern t_bool save_and_process_art(t_openartinfo *artinfo, struct t_article *artptr, t_bool is_mailbox, const char *inpath, int max, t_bool post_process);
 extern void decode_save_mime(t_openartinfo *art, t_bool postproc);
 extern void print_art_seperator_line(FILE *fp, t_bool is_mailbox);
 
 /* screen.c */
+extern char *fmt_message(const char *fmt, va_list ap);
 extern void center_line(int line, t_bool inverse, const char *str);
 extern void clear_message(void);
 extern void draw_arrow_mark(int line);
@@ -577,20 +585,19 @@ extern void stow_cursor(void);
 extern void wait_message(unsigned int sdelay, const char *fmt, ...);
 
 /* search.c */
+extern enum option_enum search_config(t_bool forward, t_bool repeat, enum option_enum current, enum option_enum last);
 extern int get_search_vectors(int *start, int *end);
-extern int search(int key, int current_art, t_bool forward, t_bool repeat);
+extern int search(t_function func, int current_art, t_bool repeat);
 extern int search_active(t_bool forward, t_bool repeat);
 extern int search_article(t_bool forward, t_bool repeat, int start_line, int lines, t_lineinfo *line, int reveal_ctrl_l_lines, FILE *fp);
-extern int search_config(t_bool forward, t_bool repeat, int current, int last);
 extern int search_body(struct t_group *group, int current_art, t_bool repeat);
+extern void reset_srch_offsets(void);
 
 /* select.c */
 extern int add_my_group(const char *group, t_bool add);
 extern int choose_new_group(void);
 extern int skip_newgroups(void);
-extern void draw_group_arrow(void);
 extern void selection_page(int start_groupnum, int num_cmd_line_groups);
-extern void set_groupname_len(t_bool all_groups);
 extern void show_selection_page(void);
 extern void toggle_my_groups(const char *group);
 
@@ -598,7 +605,7 @@ extern void toggle_my_groups(const char *group);
 extern void msg_write_signature(FILE *fp, t_bool flag, struct t_group *thisgroup);
 
 /* signal.c */
-extern RETSIGTYPE(*sigdisp (int sig, RETSIGTYPE (*func)(SIG_ARGS))) (SIG_ARGS);
+extern RETSIGTYPE(*sigdisp (int signum, RETSIGTYPE (*func)(SIG_ARGS))) (SIG_ARGS);
 extern t_bool set_win_size(int *num_lines, int *num_cols);
 extern void allow_resize(t_bool allow);
 extern void handle_resize(t_bool repaint);
@@ -611,16 +618,22 @@ extern size_t my_strftime(char *s, size_t maxsize, const char *format, struct tm
 
 /* string.c */
 extern char *eat_tab(char *s);
+extern char *fmt_string(const char *fmt, ...);
 extern char *my_strdup(const char *str);
 extern char *str_trim(char *string);
+extern char *strunc(const char *message, int len);
 extern char *tin_ltoa(long value, int digits);
-#if !defined(HAVE_STRCASESTR) || defined(DECL_STRCASESTR)
-	extern const char *strcasestr(const char *haystack, const char *needle);
-#endif /* !HAVE_STRCASESTR || DECL_STRCASESTR */
+extern char *tin_strtok(char *str, const char *delim);
 extern int sh_format(char *dst, size_t len, const char *fmt, ...);
 extern size_t mystrcat(char **t, const char *s);
 extern void my_strncpy(char *p, const char *q, size_t n);
 extern void str_lwr(char *str);
+#if !defined(HAVE_STRCASESTR) || defined(DECL_STRCASESTR)
+	extern const char *strcasestr(const char *haystack, const char *needle);
+#endif /* !HAVE_STRCASESTR || DECL_STRCASESTR */
+#if !defined(HAVE_STRSEP) || defined(DECL_SEP)
+	extern char *strsep(char **stringp, const char *delim);
+#endif /* !HAVE_STRSEP || DECL_STRSEP */
 #ifndef HAVE_STRPBRK
 	extern char *strpbrk(char *str1, char *str2);
 #endif /* !HAVE_STRPBRK */
@@ -651,8 +664,18 @@ extern void str_lwr(char *str);
 #	define strrstr(s,p)	my_strrstr(s,p)
 #endif /* !HAVE_STRRSTR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	extern void wcspart(wchar_t *to, const wchar_t *from, int columns, int size_to, t_bool pad);
+	extern char *wchar_t2char(const wchar_t *wstr);
+	extern wchar_t *char2wchar_t(const char *str);
+	extern wchar_t *wcspart(const wchar_t *wstr, int columns, t_bool pad);
+	extern wchar_t *wstrunc(const wchar_t *wmessage, int len);
+	extern wchar_t *my_wcsdup(const wchar_t *wstr);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
+	extern char *render_bidi(const char *str, t_bool *is_rtl);
+#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
+#ifdef HAVE_UNICODE_NORMALIZATION
+	extern char *normalize(const char *str);
+#endif /* HAVE_UNICODE_NORMALIZATION */
 
 /* tags.c */
 extern int line_is_tagged(int n);
@@ -690,7 +713,6 @@ extern int stat_thread(int n, struct t_art_stat *sbuf);
 extern int which_response(int n);
 extern int which_thread(int n);
 extern int thread_page(struct t_group *group, int respnum, int thread_depth, t_pagerinfo *page);
-extern void draw_line(int i, int magic);
 extern void fixup_thread(int respnum, t_bool redraw);
 
 /* version.c */
