@@ -1,12 +1,12 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : tinrc.h
- *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
+ *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1999-04-13
- *  Updated   : 2003-01-22
+ *  Updated   : 2004-06-20
  *  Notes     :
  *
- * Copyright (c) 1999-2003 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1999-2005 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Jason Faultless.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -94,6 +91,7 @@ struct t_config {
 	char maildir[PATH_LEN];				/* mailbox dir where = saves are stored */
 	int mailbox_format;					/* format of the mailbox (mboxo, mboxrd, mmdf, ...) */
 	char mail_address[HEADER_LEN];				/* user's mail address */
+	char metamail_prog[PATH_LEN];				/* name of MIME message viewer */
 #	ifndef CHARSET_CONVERSION
 		char mm_charset[LEN];				/* MIME charset */
 #	else
@@ -131,6 +129,7 @@ struct t_config {
 	int getart_limit;					/* number of article to get */
 	int recent_time;				/* Time limit when article is "fresh" */
 	int groupname_max_length;			/* max len of group names to display on screen */
+	int hide_uue;						/* treatment of uuencoded data in pager */
 	int kill_level;						/* Define how killed articles are shown */
 	int mail_mime_encoding;
 	int post_mime_encoding;
@@ -142,6 +141,7 @@ struct t_config {
 	int sort_threads_type;				/* method used to sort base[] */
 	int strip_bogus;
 	int thread_articles;				/* threading system for viewing articles */
+	int thread_perc;				/* how close the match needs to be for THREAD_PERC to recognize two articles as the same thread */
 	int thread_score;				/* how the score for threads is computed*/
 	int wildcard;						/* 0=wildmat, 1=regex */
 	int score_limit_kill;					/* score limit to kill articles */
@@ -168,12 +168,18 @@ struct t_config {
 		int col_quote3;						/* color of >=3 times quoted lines */
 		int col_response;					/* color of respone counter */
 		int col_signature;					/* color of signature */
+		int col_urls;						/* color of urls highlight */
 		int col_subject;					/* color of article subject */
 		int col_text;						/* color of textlines*/
 		int col_title;						/* color of Help/Mail-Sign */
-		int word_h_display_marks;			/* display * or _ when highlighting or space or nothing*/
 #	endif /* HAVE_COLOR */
+	int word_h_display_marks;			/* display * or _ when highlighting or space or nothing*/
+	int mono_markdash;				/* attribute for text highlighting with _underdashes_ */
+	int mono_markstar;				/* attribute for text highlighting with *stars* */
+	int mono_markslash;				/* attribute for text highlighting with /slashes/ */
+	int mono_markstroke;				/* attribute for text highlighting with -strokes- */
 	t_bool word_highlight;				/* like word_highlight but stored in tinrc */
+	t_bool url_highlight;				/* highlight urls in text bodies */
 	int wrap_column;				/* screen column to wrap of text messages */
 #	ifdef HAVE_COLOR
 		t_bool use_color;					/* like use_color but stored in tinrc */
@@ -197,9 +203,9 @@ struct t_config {
 	t_bool info_in_last_line;
 	t_bool inverse_okay;
 	t_bool keep_dead_articles;			/* keep all dead articles in dead.articles */
-	t_bool keep_posted_articles;		/* keep all posted articles in ~/Mail/posted */
-	char keep_posted_articles_file[LEN];		/* file, to keep posted articles */
+	char posted_articles_file[LEN];		/* if set, file in which to keep posted articles */
 	t_bool mail_8bit_header;			/* allow 8bit chars. in header of mail message */
+	t_bool mark_ignore_tags;			/* Ignore tags for GROUP_MARK_THREAD_READ/THREAD_MARK_ARTICLE_READ */
 	t_bool mark_saved_read;				/* mark saved article/thread as read */
 	t_bool pgdn_goto_next;
 	t_bool pos_first_unread;			/* position cursor at first/last unread article */
@@ -212,12 +218,10 @@ struct t_config {
 	t_bool prompt_followupto;			/* display empty Followup-To header in editor */
 	int quote_style;					/* quoting behaviour */
 	t_bool show_description;
-	t_bool show_lines;
+	int show_info;				/* show lines and/or score (or nothing) */
 	t_bool show_only_unread_arts;		/* show only new/unread arts or all arts */
 	t_bool show_only_unread_groups;		/* set TRUE to see only subscribed groups with new news */
-	t_bool show_score;
 	t_bool show_signatures;				/* show signatures when displaying articles */
-	t_bool hide_uue;					/* set TRUE to treat uuencoded data as an attachment */
 	t_bool sigdashes;					/* set TRUE to prepend every signature with dashes */
 	t_bool signature_repost;			/* set TRUE to add signature when reposting articles */
 	t_bool space_goto_next_unread;
@@ -229,21 +233,29 @@ struct t_config {
 	t_bool thread_catchup_on_exit;		/* catchup thread with left arrow key or not */
 	t_bool unlink_article;
 	char inews_prog[PATH_LEN];
-	t_bool use_mailreader_i;			/* invoke user's mailreader earlier to use more of its features (i = interactive) */
+	int interactive_mailer;			/* invoke user's mailreader */
 	t_bool use_mouse;					/* enables/disables mouse support under xterm */
 #	ifdef HAVE_KEYPAD
 		t_bool use_keypad;
 #	endif /* HAVE_KEYPAD */
-#	ifdef HAVE_METAMAIL
-		t_bool ask_for_metamail;			/* enables/disables the metamail query if a MIME message is going to be displayed */
-		t_bool use_metamail;				/* enables/disables metamail on MIME messages */
-#	endif /* HAVE_METAMAIL */
+	t_bool wrap_on_next_unread;		/* Wrap around threads when searching next unread article */
+	t_bool ask_for_metamail;			/* enables/disables the viewer query if a MIME message is going to be displayed */
 	t_bool default_filter_kill_case;
 	t_bool default_filter_kill_expire;
 	t_bool default_filter_kill_global;
 	t_bool default_filter_select_case;
 	t_bool default_filter_select_expire;
+#	ifdef XFACE_ABLE
+		t_bool use_slrnface;			/* Use the slrnface programme to display 'X-Face:'s */
+#	endif /* XFACE_ABLE */
 	t_bool default_filter_select_global;
+	char date_format[LEN];			/* format string for the date display in the page header */
+#	ifdef HAVE_UNICODE_NORMALIZATION
+	int normalization_form;
+#	endif /* HAVE_UNICODE_NORMALIZATION */
+#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
+	t_bool render_bidi;
+#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
 };
 
 #endif /* !TINRC_H */
