@@ -3,10 +3,10 @@
  *  Module    : version.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-06-29
+ *  Updated   : 2004-10-19
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,29 +40,22 @@
 
 #	define PRODUCT		"tin"
 #	ifndef TIN_AUTOCONF_H
-#		define VERSION		"1.6.2"
+#		define VERSION		"1.7.10"
 #	endif /* !TIN_AUTOCONF_H */
-#	define RELEASEDATE	"20030910"
-#	define RELEASENAME	"Pabbay"
+#	define RELEASEDATE	"20050929"
+#	define RELEASENAME	"Tahay"
 /* config-file versions - must by dotted triples */
-#	define TINRC_VERSION	"1.3.6"
-#	define ATTRIBUTES_VERSION	"1.0.0"
+#	define TINRC_VERSION	"1.3.7"
+#	define ATTRIBUTES_VERSION	"1.0.4"
 #	define FILTER_VERSION	"1.0.0"
+#	define KEYMAP_VERSION	"1.0.6"
 #	define SERVERCONFIG_VERSION	"1.0.0"
 
-#	ifdef M_AMIGA
-#		define OSNAME	"AMIGA"
-#		define AMIVER	VERSION
-#	endif /* M_AMIGA */
-
 #	ifdef VMS
 #		define OSNAME	"VMS"
 #	endif /* VMS */
 
 #	ifdef M_UNIX
-#		ifdef __amiga
-#			define OSNAME	"AMIGA"
-#		endif /* !__amiga */
 #		ifdef __BEOS__
 #			define OSNAME	"BeOS"
 #		endif /* __BEOS__ */
