@@ -3,10 +3,10 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2003-06-29
+ *  Updated   : 2004-06-30
  *  Notes     :
  *
- * Copyright (c) 1992-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,6 +62,8 @@ static time_t active_timestamp;	/* time active file read (local) */
 /*
  * Local prototypes
  */
+static FILE *open_newgroups_fp(int idx);
+static FILE *open_news_active_fp(void);
 static void active_add(struct t_group *ptr, long count, long max, long min, const char *moderated);
 static void append_group_line(char *active_file, char *group_path, long art_max, long art_min, char *base_dir);
 static void check_for_any_new_groups(void);
@@ -72,25 +74,6 @@ static void read_newsrc_active_file(void);
 static void subscribe_new_group(char *group, char *autosubscribe, char *autounsubscribe);
 
 
-/*
- * Get default array size for active[] from environment (AmigaDOS)
- * or just return the standard default.
- */
-int
-get_active_num(
-	void)
-{
-#ifdef ENV_VAR_GROUPS /* M_AMIGA only */
-	char *ptr;
-	int num;
-
-	if ((ptr = getenv(ENV_VAR_GROUPS)) != NULL)
-		return ((num = atoi(ptr)) ? num : DEFAULT_ACTIVE_NUM);
-#endif /* ENV_VAR_GROUPS */
-	return DEFAULT_ACTIVE_NUM;
-}
-
-
 t_bool
 need_reread_active_file(
 	void)
@@ -138,7 +121,6 @@ resync_active_file(
 		toggle_my_groups(old_group);
 
 	FreeAndNull(old_group);
-	set_groupname_len(FALSE);
 	show_selection_page();
 
 	return TRUE;
@@ -159,8 +141,8 @@ active_add(
 	const char *moderated)
 {
 	/* name - pre-initialised when group is made */
-	ptr->aliasedto = ((moderated[0] == '=') ? my_strdup(moderated + 1) : (char *) 0);
-	ptr->description = (char *) 0;
+	ptr->aliasedto = ((moderated[0] == '=') ? my_strdup(moderated + 1) : NULL);
+	ptr->description = NULL;
 	/* spool - see below */
 	ptr->moderated = moderated[0];
 	ptr->count = count;
@@ -317,7 +299,7 @@ read_newsrc_active_file(
 			char line[NNTP_STRLEN];
 			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr) {
 				ngnames[index_i] = my_strdup(ptr);
-				sprintf(buf, "GROUP %s", ngnames[index_i]);
+				snprintf(buf, sizeof(buf), "GROUP %s", ngnames[index_i]);
 #	ifdef DEBUG
 				debug_nntp("read_newsrc_active_file", buf);
 #	endif /* DEBUG */
@@ -338,7 +320,7 @@ read_newsrc_active_file(
 					int i;
 					int j = index_o;
 					for (i = 0; i < window - 1; i++) {
-						sprintf(buf, "GROUP %s", ngnames[j]);
+						snprintf(buf, sizeof(buf), "GROUP %s", ngnames[j]);
 #	ifdef DEBUG
 						debug_nntp("read_newsrc_active_file", buf);
 #	endif /* DEBUG */
@@ -359,8 +341,7 @@ read_newsrc_active_file(
 						{
 							char fmt[20];
 
-							sprintf(fmt, "%%ld %%ld %%ld %%%ds", NNTP_STRLEN);
-
+							snprintf(fmt, sizeof(fmt), "%%ld %%ld %%ld %%%ds", NNTP_STRLEN);
 							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4)
 								error_message(_(txt_error_invalid_response_to_group), line);
 							if (strcmp(ngname, ngnames[index_o]) != 0)
@@ -456,6 +437,22 @@ read_newsrc_active_file(
 }
 
 
+/*
+ * Open the news active file locally or send the LIST command
+ */
+static FILE *
+open_news_active_fp(
+	void)
+{
+#ifdef NNTP_ABLE
+	if (read_news_via_nntp && !read_saved_news)
+		return (nntp_command("LIST", OK_GROUPS, NULL, 0));
+	else
+#endif /* NNTP_ABLE */
+		return (fopen(news_active_file, "r"));
+}
+
+
 /*
  * Load the active file into active[]
  */
@@ -463,7 +460,7 @@ static void
 read_active_file(
 	void)
 {
-	FILE *fp = (FILE *) 0;
+	FILE *fp;
 	char *ptr;
 	char moderated[PATH_LEN];
 	long count = -1L, min = 1L, max = 0L;
@@ -483,7 +480,7 @@ read_active_file(
 #	ifndef NNTP_ONLY
 		else
 			error_message(_(txt_cannot_open_active_file), news_active_file, tin_progname);
-#	endif /* NNTP_ONLY */
+#	endif /* !NNTP_ONLY */
 #else
 		error_message(_(txt_cannot_open), news_active_file);
 #endif /* NNTP_ABLE */
@@ -549,7 +546,7 @@ void
 read_news_active_file(
 	void)
 {
-	FILE *fp = 0;
+	FILE *fp;
 
 	/*
 	 * Ignore -n if no .newsrc can be found or .newsrc is empty
@@ -591,6 +588,41 @@ read_news_active_file(
 }
 
 
+/*
+ * Open the active.times file locally or send the NEWGROUPS command
+ *
+ * NEWGROUPS yymmdd hhmmss
+ */
+static FILE *
+open_newgroups_fp(
+	int idx)
+{
+#ifdef NNTP_ABLE
+	char line[NNTP_STRLEN];
+	struct tm *ngtm;
+
+	if (read_news_via_nntp && !read_saved_news) {
+		if (idx == -1)
+			return (FILE *) 0;
+
+		ngtm = localtime(&newnews[idx].time);
+		/*
+		 * in the current draft, NEWGROUPS is allowed to take a 4 digit year
+		 * component - but even with a 2 digit year component it is y2k
+		 * compliant... we should switch over to ngtm->tm_year + 1900
+		 * when most servers can handle the new format
+		 */
+		snprintf(line, sizeof(line), "NEWGROUPS %02d%02d%02d %02d%02d%02d",
+			ngtm->tm_year % 100, ngtm->tm_mon + 1, ngtm->tm_mday,
+			ngtm->tm_hour, ngtm->tm_min, ngtm->tm_sec);
+
+		return (nntp_command(line, OK_NEWGROUPS, NULL, 0));
+	} else
+#endif /* NNTP_ABLE */
+		return (fopen(active_times_file, "r"));
+}
+
+
 /*
  * Check for any newly created newsgroups.
  *
@@ -670,7 +702,7 @@ check_for_any_new_groups(
 	if (newnews_index >= 0)
 		newnews[newnews_index].time = new_newnews_time;
 	else {
-		sprintf(buf, "%s %lu", nntp_server, (unsigned long int) new_newnews_time);
+		snprintf(buf, sizeof(buf), "%s %lu", nntp_server, (unsigned long int) new_newnews_time);
 		load_newnews_info(buf);
 	}
 }
@@ -753,9 +785,8 @@ match_group_list(
 	char *separator;
 	char pattern[HEADER_LEN];
 	size_t group_len, list_len;
-	t_bool accept, negate;
+	t_bool negate, accept = FALSE;
 
-	accept = FALSE;
 	list_len = strlen(group_list);
 	/*
 	 * walk through comma-separated entries in list
@@ -765,7 +796,7 @@ match_group_list(
 		 * find end/length of this entry
 		 */
 		separator = strchr(group_list, ',');
-		group_len = ((separator == NULL) ? list_len : (size_t) (separator - group_list));
+		group_len = MIN(((separator == NULL) ? list_len : (size_t) (separator - group_list)), sizeof(pattern) - 1);
 
 		if ((negate = ('!' == *group_list))) {
 			/*
@@ -820,7 +851,7 @@ load_newnews_info(
 	 */
 	if (!num_newnews) {
 		for (i = 0; i < max_newnews; i++) {
-			newnews[i].host = (char *) 0;
+			newnews[i].host = NULL;
 			newnews[i].time = (time_t) 0;
 		}
 	}
@@ -957,9 +988,7 @@ make_group_list(
 				make_group_list(active_file, base_dir, group_path);
 				find_art_max_min(group_path, &art_max, &art_min);
 				append_group_line(active_file, group_path, art_max, art_min, base_dir);
-
-				ptr = strrchr(group_path, '/'); /* TODO: Unix'ism */
-				if (ptr != NULL)
+				if ((ptr = strrchr(group_path, '/')) != NULL) /* TODO: Unix'ism */
 					*ptr = '\0';
 			}
 		}
