@@ -3,10 +3,10 @@
  *  Module    : attrib.c
  *  Author    : I. Lea
  *  Created   : 1993-12-01
- *  Updated   : 2003-05-10
+ *  Updated   : 2005-06-30
  *  Notes     : Group attribute routines
  *
- * Copyright (c) 1993-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,9 @@ enum {
 	ATTRIB_DELETE_TMP_FILES,
 	ATTRIB_SHOW_ONLY_UNREAD,
 	ATTRIB_THREAD_ARTS,
+	ATTRIB_THREAD_PERC,
 	ATTRIB_SHOW_AUTHOR,
+	ATTRIB_SHOW_INFO,
 	ATTRIB_SORT_ART_TYPE,
 	ATTRIB_POST_PROC_TYPE,
 	ATTRIB_QUICK_KILL_HEADER,
@@ -79,11 +81,13 @@ enum {
 	ATTRIB_MAILING_LIST,
 	ATTRIB_X_HEADERS,
 	ATTRIB_X_BODY,
-	ATTRIB_AUTO_SAVE_MSG,
+	ATTRIB_AUTO_SAVE_MSG, /* TODO: what is this? dublicate of ATTRIB_AUTO_SAVE? */
 	ATTRIB_X_COMMENT_TO,
+	ATTRIB_FCC,
 	ATTRIB_NEWS_QUOTE,
 	ATTRIB_QUOTE_CHARS,
 	ATTRIB_MIME_TYPES_TO_SAVE,
+	ATTRIB_MIME_FORWARD,
 #ifdef HAVE_ISPELL
 		ATTRIB_ISPELL,
 #endif /* HAVE_ISPELL */
@@ -125,32 +129,34 @@ set_default_attributes(
 	attributes->global = FALSE;	/* global/group specific */
 	attributes->maildir = tinrc.maildir;
 	attributes->savedir = tinrc.savedir;
-	attributes->savefile = (char *) 0;
+	attributes->savefile = NULL;
 	attributes->sigfile = tinrc.sigfile;
-	attributes->organization = (*default_organization ? default_organization : (char *) 0);
-	attributes->followup_to = (char *) 0;
-	attributes->mailing_list = (char *) 0;
-	attributes->x_headers = (char *) 0;
-	attributes->x_body = (char *) 0;
+	attributes->organization = (*default_organization ? default_organization : NULL);
+	attributes->followup_to = NULL;
+	attributes->mailing_list = NULL;
+	attributes->x_headers = NULL;
+	attributes->x_body = NULL;
 	attributes->from = tinrc.mail_address;
 	attributes->news_quote_format = tinrc.news_quote_format;
 	attributes->quote_chars = tinrc.quote_chars;
 	attributes->mime_types_to_save = my_strdup("*/*");
 #ifdef HAVE_ISPELL
-	attributes->ispell = (char *) 0;
+	attributes->ispell = NULL;
 #endif /* HAVE_ISPELL */
-	attributes->quick_kill_scope = (tinrc.default_filter_kill_global ? my_strdup("*") : (char *) 0);
+	attributes->quick_kill_scope = (tinrc.default_filter_kill_global ? my_strdup("*") : NULL);
 	attributes->quick_kill_header = tinrc.default_filter_kill_header;
 	attributes->quick_kill_case = tinrc.default_filter_kill_case;
 	attributes->quick_kill_expire = tinrc.default_filter_kill_expire;
-	attributes->quick_select_scope = (tinrc.default_filter_select_global ? my_strdup("*") : (char *) 0);
+	attributes->quick_select_scope = (tinrc.default_filter_select_global ? my_strdup("*") : NULL);
 	attributes->quick_select_header = tinrc.default_filter_select_header;
 	attributes->quick_select_case = tinrc.default_filter_select_case;
 	attributes->quick_select_expire = tinrc.default_filter_select_expire;
 	attributes->show_only_unread = tinrc.show_only_unread_arts;
 	attributes->thread_arts = tinrc.thread_articles;
+	attributes->thread_perc = tinrc.thread_perc;
 	attributes->sort_art_type = tinrc.sort_article_type;
 	attributes->sort_threads_type = tinrc.sort_threads_type;
+	attributes->show_info = tinrc.show_info;
 	attributes->show_author = tinrc.show_author;
 	attributes->auto_save = tinrc.auto_save;
 	attributes->auto_select = FALSE;
@@ -159,17 +165,15 @@ set_default_attributes(
 	attributes->post_proc_type = tinrc.post_process;
 	attributes->x_comment_to = FALSE;
 	attributes->tex2iso_conv = tinrc.tex2iso_conv;
+	attributes->mime_forward = FALSE;
+	attributes->fcc = NULL;
 #ifdef CHARSET_CONVERSION
 	attributes->mm_network_charset = tinrc.mm_network_charset;
-	attributes->undeclared_charset = (char *) 0;
+	attributes->undeclared_charset = NULL;
 #endif /* CHARSET_CONVERSION */
 }
 
 
-/*
- * Load global & local attributes into active[].attribute
- */
-
 #define MATCH_BOOLEAN(pattern, type) \
 	if (match_boolean(line, pattern, &flag)) { \
 		num = (flag != FALSE); \
@@ -184,7 +188,7 @@ set_default_attributes(
 		break; \
 	}
 #define MATCH_STRING(pattern, type) \
-	if (match_string(line, pattern, buf, sizeof(buf))) { \
+	if (match_string(line, pattern, buf, sizeof(buf) - strlen(pattern))) { \
 		set_attrib(type, scope, buf); \
 		found = TRUE; \
 		break; \
@@ -197,6 +201,14 @@ set_default_attributes(
 			break; \
 		}
 #endif /* CHARSET_CONVERSION */
+#if !defined(CHARSET_CONVERSION) || !defined(HAVE_ISPELL)
+#	define SKIP_ITEM(pattern) \
+		if (!strncmp(line, pattern, strlen(pattern))) { \
+			found = TRUE; \
+			break; \
+		}
+#endif /* !CHARSET_CONVERSION || !HAVE_ISPELL */
+
 
 /*
  * (re)read global/local attributes file
@@ -273,6 +285,7 @@ read_attributes_file(
 					break;
 
 				case 'f':
+					MATCH_STRING("fcc=", ATTRIB_FCC);
 					MATCH_STRING("followup_to=", ATTRIB_FOLLOWUP_TO);
 					MATCH_STRING("from=", ATTRIB_FROM);
 					break;
@@ -280,15 +293,20 @@ read_attributes_file(
 				case 'i':
 #ifdef HAVE_ISPELL
 					MATCH_STRING("ispell=", ATTRIB_ISPELL);
+#else
+					SKIP_ITEM("ispell=");
 #endif /* HAVE_ISPELL */
 					break;
 
 				case 'm':
 					MATCH_STRING("maildir=", ATTRIB_MAILDIR);
 					MATCH_STRING("mailing_list=", ATTRIB_MAILING_LIST);
+					MATCH_BOOLEAN("mime_forward=", ATTRIB_MIME_FORWARD);
 					MATCH_STRING("mime_types_to_save=", ATTRIB_MIME_TYPES_TO_SAVE);
 #ifdef CHARSET_CONVERSION
-					MATCH_LIST("mm_network_charset=", ATTRIB_MM_NETWORK_CHARSET,txt_mime_charsets, NUM_MIME_CHARSETS);
+					MATCH_LIST("mm_network_charset=", ATTRIB_MM_NETWORK_CHARSET, txt_mime_charsets, NUM_MIME_CHARSETS);
+#else
+					SKIP_ITEM("mm_network_charset=");
 #endif /* CHARSET_CONVERSION */
 					break;
 
@@ -329,6 +347,7 @@ read_attributes_file(
 						break;
 					}
 					MATCH_INTEGER("show_author=", ATTRIB_SHOW_AUTHOR, SHOW_FROM_BOTH);
+					MATCH_INTEGER("show_info=", ATTRIB_SHOW_INFO, SHOW_INFO_BOTH);
 					MATCH_BOOLEAN("show_only_unread=", ATTRIB_SHOW_ONLY_UNREAD);
 					MATCH_STRING("sigfile=", ATTRIB_SIGFILE);
 					MATCH_INTEGER("sort_art_type=", ATTRIB_SORT_ART_TYPE, SORT_ARTICLES_BY_LINES_ASCEND);
@@ -338,11 +357,14 @@ read_attributes_file(
 				case 't':
 					MATCH_BOOLEAN("tex2iso_conv=", ATTRIB_TEX2ISO_CONV);
 					MATCH_INTEGER("thread_arts=", ATTRIB_THREAD_ARTS, THREAD_MAX);
+					MATCH_INTEGER("thread_perc=", ATTRIB_THREAD_PERC, 100);
 					break;
 
 				case 'u':
 #ifdef CHARSET_CONVERSION
 					MATCH_STRING("undeclared_charset=", ATTRIB_UNDECLARED_CHARSET);
+#else
+					SKIP_ITEM("undeclared_charset=");
 #endif /* CHARSET_CONVERSION */
 					break;
 
@@ -391,8 +413,11 @@ set_attrib(
 {
 	struct t_group *group;
 
-	if (scope == NULL || *scope == '\0')	/* No active scope set yet */
+	if (scope == NULL || *scope == '\0') {	/* No active scope set yet */
+		/* TODO: include full line in error-message */
+		error_message("attribute with no scope: %s", data);
 		return;
+	}
 #if 0
 	fprintf(stderr, "set_attrib #%d %s %s(%d)\n", type, scope, data, (int) *data);
 #endif /* 0 */
@@ -405,6 +430,7 @@ set_attrib(
 			do_set_attrib(group, type, data);
 	} else {
 		int i;
+
 		/* TODO: Can we get out of doing this per group for .global case */
 		for_each_group(i) {
 			group = &active[i];
@@ -443,96 +469,145 @@ do_set_attrib(
 		case ATTRIB_MAILDIR:
 			free_if_not_default(&group->attribute->maildir, tinrc.maildir);
 			SET_STRING(maildir);
+
 		case ATTRIB_SAVEDIR:
 			free_if_not_default(&group->attribute->savedir, tinrc.savedir);
 			SET_STRING(savedir);
+
 		case ATTRIB_SAVEFILE:
 			FreeIfNeeded(group->attribute->savefile);
 			SET_STRING(savefile);
+
 		case ATTRIB_ORGANIZATION:
 			free_if_not_default(&group->attribute->organization, default_organization);
 			SET_STRING(organization);
+
 		case ATTRIB_FROM:
 			free_if_not_default(&group->attribute->from, tinrc.mail_address);
 			SET_STRING(from);
+
 		case ATTRIB_SIGFILE:
 			free_if_not_default(&group->attribute->sigfile, tinrc.sigfile);
 			SET_STRING(sigfile);
+
 		case ATTRIB_FOLLOWUP_TO:
 			FreeIfNeeded(group->attribute->followup_to);
 			SET_STRING(followup_to);
+
 		case ATTRIB_AUTO_SELECT:
 			SET_INTEGER(auto_select);
+
 		case ATTRIB_AUTO_SAVE:
 			SET_INTEGER(auto_save);
+
 		case ATTRIB_BATCH_SAVE:
 			SET_INTEGER(batch_save);
+
 		case ATTRIB_DELETE_TMP_FILES:
 			SET_INTEGER(delete_tmp_files);
+
 		case ATTRIB_SHOW_ONLY_UNREAD:
 			SET_INTEGER(show_only_unread);
+
 		case ATTRIB_THREAD_ARTS:
 			SET_INTEGER(thread_arts);
+
+		case ATTRIB_THREAD_PERC:
+			SET_INTEGER(thread_perc);
+
 		case ATTRIB_SHOW_AUTHOR:
 			SET_INTEGER(show_author);
+
+		case ATTRIB_SHOW_INFO:
+			SET_INTEGER(show_info);
+
 		case ATTRIB_SORT_ART_TYPE:
 			SET_INTEGER(sort_art_type);
+
 		case ATTRIB_SORT_THREADS_TYPE:
 			SET_INTEGER(sort_threads_type);
+
 		case ATTRIB_POST_PROC_TYPE:
 			SET_INTEGER(post_proc_type);
+
 		case ATTRIB_QUICK_KILL_HEADER:
 			SET_INTEGER(quick_kill_header);
+
 		case ATTRIB_QUICK_KILL_SCOPE:
 			FreeIfNeeded(group->attribute->quick_kill_scope);
 			SET_STRING(quick_kill_scope);
+
 		case ATTRIB_QUICK_KILL_EXPIRE:
 			SET_INTEGER(quick_kill_expire);
+
 		case ATTRIB_QUICK_KILL_CASE:
 			SET_INTEGER(quick_kill_case);
+
 		case ATTRIB_QUICK_SELECT_HEADER:
 			SET_INTEGER(quick_select_header);
+
 		case ATTRIB_QUICK_SELECT_SCOPE:
 			FreeIfNeeded(group->attribute->quick_select_scope);
 			SET_STRING(quick_select_scope);
+
 		case ATTRIB_QUICK_SELECT_EXPIRE:
 			SET_INTEGER(quick_select_expire);
+
 		case ATTRIB_QUICK_SELECT_CASE:
 			SET_INTEGER(quick_select_case);
+
 		case ATTRIB_MAILING_LIST:
 			FreeIfNeeded(group->attribute->mailing_list);
 			SET_STRING(mailing_list);
+
 #ifdef CHARSET_CONVERSION
 		case ATTRIB_MM_NETWORK_CHARSET:
 			SET_INTEGER(mm_network_charset);
+
 		case ATTRIB_UNDECLARED_CHARSET:
 			FreeIfNeeded(group->attribute->undeclared_charset);
 			SET_STRING(undeclared_charset);
 #endif /* CHARSET_CONVERSION */
+
 		case ATTRIB_X_HEADERS:
 			FreeIfNeeded(group->attribute->x_headers);
 			SET_STRING(x_headers);
+
 		case ATTRIB_X_BODY:
 			FreeIfNeeded(group->attribute->x_body);
 			SET_STRING(x_body);
+
 		case ATTRIB_X_COMMENT_TO:
 			SET_INTEGER(x_comment_to);
+
+		case ATTRIB_FCC:
+			FreeIfNeeded(group->attribute->fcc);
+			SET_STRING(fcc);
+
 		case ATTRIB_NEWS_QUOTE:
 			free_if_not_default(&group->attribute->news_quote_format, tinrc.news_quote_format);
 			SET_STRING(news_quote_format);
+
 		case ATTRIB_QUOTE_CHARS:
 			free_if_not_default(&group->attribute->quote_chars, tinrc.quote_chars);
 			SET_STRING(quote_chars);
+
 		case ATTRIB_MIME_TYPES_TO_SAVE:
 			FreeIfNeeded(group->attribute->mime_types_to_save);
 			SET_STRING(mime_types_to_save);
+
+		case ATTRIB_MIME_FORWARD:
+			SET_INTEGER(mime_forward);
+
 #ifdef HAVE_ISPELL
 		case ATTRIB_ISPELL:
 			FreeIfNeeded(group->attribute->ispell);
 			SET_STRING(ispell);
 #endif /* HAVE_ISPELL */
+
 		case ATTRIB_TEX2ISO_CONV:
 			SET_INTEGER(tex2iso_conv);
+
 		default:
 			break;
 	}
@@ -601,12 +676,19 @@ write_attributes_file(
 		fprintf(fp, "%d=%s, ", i, _(txt_threading[i]));
 	}
 	fprintf(fp, "\n");
+	fprintf(fp, _("#  thread_perc=NUM\n"));
 	fprintf(fp, _("#  show_author=NUM\n"));
 	fprintf(fp, "#    %d=%s, %d=%s, %d=%s, %d=%s\n",
 		SHOW_FROM_NONE, _(txt_show_from[SHOW_FROM_NONE]),
 		SHOW_FROM_ADDR, _(txt_show_from[SHOW_FROM_ADDR]),
 		SHOW_FROM_NAME, _(txt_show_from[SHOW_FROM_NAME]),
 		SHOW_FROM_BOTH, _(txt_show_from[SHOW_FROM_BOTH]));
+	fprintf(fp, _("#  show_info=NUM\n"));
+	fprintf(fp, "#    %d=%s, %d=%s, %d=%s, %d=%s\n",
+		SHOW_INFO_NOTHING, _(txt_show_info_type[SHOW_INFO_NOTHING]),
+		SHOW_INFO_LINES, _(txt_show_info_type[SHOW_INFO_LINES]),
+		SHOW_INFO_SCORE, _(txt_show_info_type[SHOW_INFO_SCORE]),
+		SHOW_INFO_BOTH, _(txt_show_info_type[SHOW_INFO_BOTH]));
 	fprintf(fp, _("#  sort_art_type=NUM\n"));
 	fprintf(fp, "#    %d=%s,\n",
 		SORT_ARTICLES_BY_NOTHING, _(txt_sort_a_type[SORT_ARTICLES_BY_NOTHING]));
@@ -650,7 +732,9 @@ write_attributes_file(
 	fprintf(fp, _("#    2=from (case sensitive) 3=from (ignore case)\n"));
 	fprintf(fp, _("#    4=msgid 5=lines\n"));
 	fprintf(fp, _("#  x_comment_to=ON/OFF\n"));
+	fprintf(fp, _("#  fcc=STRING (eg. =mailbox)\n"));
 	fprintf(fp, _("#  tex2iso_conv=ON/OFF\n"));
+	fprintf(fp, _("#  mime_forward=ON/OFF\n"));
 #ifdef CHARSET_CONVERSION
 	fprintf(fp, _("#  mm_network_charset=supported_charset"));
 	for (i = 0; i < NUM_MIME_CHARSETS; i++) {
@@ -681,10 +765,11 @@ write_attributes_file(
 		fprintf(fp, "scope=*sources*\n");
 		fprintf(fp, "post_proc_type=%d\n\n", POST_PROC_SHAR);
 
-		fprintf(fp, _("# in *binaries* do full post processing, remove tmp files\n"));
-		fprintf(fp, _("# and set Followup-To: poster\n"));
+		fprintf(fp, _("# in *binaries* do full post processing but no TeX2ISO conversion,\n"));
+		fprintf(fp, _("# remove tmp files and set Followup-To: poster\n"));
 		fprintf(fp, "scope=*binaries*\n");
 		fprintf(fp, "post_proc_type=%d\n", POST_PROC_YES);
+		fprintf(fp, "tex2iso_conv=OFF\n");
 		fprintf(fp, "delete_tmp_files=ON\n");
 		fprintf(fp, "followup_to=poster\n\n");
 	} else {
@@ -710,7 +795,8 @@ write_attributes_file(
 		error_message(_(txt_filesystem_full), ATTRIBUTES_FILE);
 	else if (copy_ok)
 		rename_file(new_file, file);
-	else unlink (new_file);
+	else
+		unlink (new_file);
 
 	free(new_file);
 	return;
@@ -739,6 +825,7 @@ write_attributes_file(
 #	endif /* HAVE_ISPELL */
 		fprintf(fp, "show_only_unread=%s\n", print_boolean(attr->show_only_unread));
 		fprintf(fp, "thread_arts=%d\n", attr->thread_arts);
+		fprintf(fp, "thread_perc=%d\n", attr->thread_perc);
 		fprintf(fp, "auto_select=%s\n", print_boolean(attr->auto_select));
 		fprintf(fp, "auto_save=%s\n", print_boolean(attr->auto_save));
 		fprintf(fp, "batch_save=%s\n", print_boolean(attr->batch_save));
@@ -746,6 +833,7 @@ write_attributes_file(
 		fprintf(fp, "sort_art_type=%d\n", attr->sort_art_type);
 		fprintf(fp, "sort_threads_type=%d\n", attr->sort_threads_type);
 		fprintf(fp, "show_author=%d\n", attr->show_author);
+		fprintf(fp, "show_info=%d\n", attr->show_info);
 		fprintf(fp, "post_proc_type=%d\n", attr->post_proc_type);
 		fprintf(fp, "quick_kill_scope=%s\n", attr->quick_kill_scope);
 		fprintf(fp, "quick_kill_case=%s\n", print_boolean(attr->quick_kill_case));
@@ -756,7 +844,9 @@ write_attributes_file(
 		fprintf(fp, "quick_select_expire=%s\n", print_boolean(attr->quick_select_expire));
 		fprintf(fp, "quick_select_header=%d\n", attr->quick_select_header);
 		fprintf(fp, "x_comment_to=%s\n", print_boolean(attr->x_comment_to));
+		fprintf(fp, "fcc=%s\n", attr->fcc);
 		fprintf(fp, "tex2iso_conv=%s\n", print_boolean(attr->tex2iso_conv));
+		fprintf(fp, "mime_forward=%s\n", print_boolean(attr->mime_forward));
 #	ifdef CHARSET_CONVERSION
 		fprintf(fp, "mm_network_charset=%s\n", txt_mime_charsets[attr->mm_network_charset]);
 		fprintf(fp, "undeclared_charset=%s\n", attr->undeclared_charset);
@@ -828,6 +918,7 @@ dump_attributes(
 #		endif /* HAVE_ISPELL */
 		fprintf(stderr, "\tshow_only_unread=%s\n", print_boolean(group->attribute->show_only_unread));
 		fprintf(stderr, "\tthread_arts=%d\n", group->attribute->thread_arts);
+		fprintf(stderr, "\tthread_perc=%d\n", group->attribute->thread_perc);
 		fprintf(stderr, "\tauto_select=%s\n", print_boolean(group->attribute->auto_select));
 		fprintf(stderr, "\tauto_save=%s\n", print_boolean(group->attribute->auto_save));
 		fprintf(stderr, "\tbatch_save=%s\n", print_boolean(group->attribute->batch_save));
@@ -835,6 +926,7 @@ dump_attributes(
 		fprintf(stderr, "\tsort_art_type=%d\n", group->attribute->sort_art_type);
 		fprintf(stderr, "\tsort_threads_type=%d\n", group->attribute->sort_threads_type);
 		fprintf(stderr, "\tshow_author=%d\n", group->attribute->show_author);
+		fprintf(stderr, "\tshow_info=%d\n", group->attribute->show_info);
 		fprintf(stderr, "\tpost_proc_type=%d\n", group->attribute->post_proc_type);
 		fprintf(stderr, "\tquick_kill_scope=%s\n", group->attribute->quick_kill_scope);
 		fprintf(stderr, "\tquick_kill_case=%s\n", print_boolean(group->attribute->quick_kill_case));
@@ -845,7 +937,9 @@ dump_attributes(
 		fprintf(stderr, "\tquick_select_expire=%s\n", print_boolean(group->attribute->quick_select_expire));
 		fprintf(stderr, "\tquick_select_header=%d\n", group->attribute->quick_select_header);
 		fprintf(stderr, "\tx_comment_to=%s\n", print_boolean(group->attribute->x_comment_to));
+		fprintf(stderr, "\tfcc=%s\n", group->attribute->fcc);
 		fprintf(stderr, "\ttex2iso_conv=%s\n", print_boolean(group->attribute->tex2iso_conv));
+		fprintf(stderr, "\tmime_forward=%s\n", print_boolean(group->attribute->mime_forward));
 #		ifdef CHARSET_CONVERSION
 		fprintf(stderr, "\tmm_network_charset=%s\n", txt_mime_charsets[group->attribute->mm_network_charset]);
 		fprintf(stderr, "\tundeclared_charset=%s\n", group->attribute->undeclared_charset);
