@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   : 2003-07-21
+ *  Updated   : 2005-07-20
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2003 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2005 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,9 +40,7 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef RFC2046_H
-#	include "rfc2046.h"
-#endif /* !RFC2046_H */
+
 
 /*
  * We malloc() this many t_lineinfo's at a time
@@ -57,14 +55,10 @@
 #define MATCH_REGEX(x,y,z)	(pcre_exec(x.re, x.extra, y, z, 0, 0, NULL, 0) >= 0)
 
 
-static int put_rest(char **rest, char **line, int *max_line_len, const int offset);
-static int read_decoded_base64_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read, char **rest);
-static int read_decoded_qp_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read);
 static t_bool header_wanted(const char *line);
 static t_part *new_uue(t_part **part, char *name);
 static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part, int hide_uue, int tabs);
 static void put_cooked(size_t buf_len, t_bool wrap_lines, int flags, const char *fmt, ...);
-static void set_rest(char **rest, const char *ptr);
 #ifdef DEBUG_ART
 	static void dump_cooked(void);
 #endif /* DEBUG_ART */
@@ -160,13 +154,13 @@ put_cooked(
 	char *p, *bufp, *buf;
 	int wrap_column;
 	int space;
-	static int overflow = 0;
+/*	static int overflow = 0; */ /* TODO: we don't use it (anymore?) */
 	static int saved_flags = 0;
+	va_list ap;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	int bytes;
 	wint_t *wp;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-	va_list ap;
 
 	buf = my_malloc(buf_len + 1);
 
@@ -209,7 +203,7 @@ put_cooked(
 		if (*p == '\n')
 			p++;
 		bufp = p;
-		overflow = 0;
+/*		overflow = 0; */
 
 		if (art->cooked_lines == 0) {
 			art->cookl = my_malloc(sizeof(t_lineinfo) * CHUNK);
@@ -246,7 +240,7 @@ put_cooked(
 	if (*bufp != '\0') {
 		fputs(bufp, art->cooked);
 		saved_flags = flags;
-		overflow += strlen(bufp);
+/*		overflow += strlen(bufp); */
 	}
 
 	va_end(ap);
@@ -254,312 +248,6 @@ put_cooked(
 }
 
 
-/*
- * FIXME: should go into rfc1521.c
- *
- * Set everything in ptr as the rest of a physical line to be processed
- * later.
- */
-static void
-set_rest(
-	char **rest,
-	const char *ptr)
-{
-	char *old_rest = *rest;
-
-	if (ptr == NULL || strlen(ptr) == 0) {
-		FreeAndNull(*rest);
-		return;
-	}
-	*rest = my_strdup(ptr);
-	FreeIfNeeded(old_rest);
-}
-
-
-/*
- * FIXME: should go into rfc1521.c
- *
- * Copy things that were left over from the last decoding into the new line.
- * If there's a newline in the rest, copy everything up to and including that
- * newline into the expected buffer, adjust rest and return. If there's no
- * newline in the rest, copy all of it to the expected buffer and return.
- *
- * Side effects: resizes line if necessary, adjusts max_line_len
- * accordingly.
- *
- * This function returns the number of character written to the line buffer.
- * The buffer is LF + NULL terminated if a complete line was written. The
- * buffer is NOT neccessarily NULL terminated if there was no newline in the
- * rest. Use the return value to find out how long the string is.
- */
-static int
-put_rest(
-	char **rest,
-	char **line,
-	int *max_line_len,
-	const int offset)
-{
-	char *my_rest = *rest;
-	char *ptr;
-	char c;
-	int put_chars = offset;
-
-	if ((ptr = my_rest) == NULL)
-		return 0;
-	if (strlen(my_rest) == 0) {
-		FreeAndNull(*rest);
-		return 0;
-	}
-
-	while ((c = *ptr++) && (c != '\n')) {
-		if ((c == '\r') && (*ptr == '\n'))
-			continue;	/* step over CRLF */
-		/*
-		 * Resize line if necessary. Keep in mind that we add LF and \0 later.
-		 */
-		if (put_chars >= *max_line_len - 2) {
-			if (*max_line_len == 0)
-				*max_line_len = LEN;
-			else
-				*max_line_len <<= 1;
-			*line = my_realloc(*line, *max_line_len);
-		}
-		(*line)[put_chars++] = c;
-	}
-	if (c == '\n') {
-		/*
-		 * FIXME: Adding a newline may be not correct. At least it may
-		 * be not what the author of that article intended.
-		 * Unfortunately, a newline is expected at the end of a line by
-		 * some other code in cook.c and even those functions invoking
-		 * this one rely on it.
-		 */
-		(*line)[put_chars++] = '\n';
-		(*line)[put_chars] = '\0';	/* don't count the termining NULL! */
-		set_rest(rest, ptr);
-	} else
-		/* rest is now empty */
-		FreeAndNull(*rest);
-
-	return put_chars;
-}
-
-
-/*
- * FIXME: should go into rfc1521.c
- *
- * Read a logical base64 encoded line into the specified line buffer.
- * Logical lines can be split over several physical base64 encoded lines and
- * a single physical base64 encoded line can contain serveral logical lines.
- * This function keeps track of all these cases and always copies only one
- * decoded line to the line buffer.
- *
- * Side effects: resizes line if necessary, adjusts max_line_len
- * accordingly.
- *
- * This function returns the number of physical lines read or a negative
- * value on error.
- */
-static int
-read_decoded_base64_line(
-	FILE *file,
-	char **line,
-	int *max_line_len,
-	const int max_lines_to_read,
-	char **rest)
-{
-	char *buf2;	/* holds the entire decoded line */
-	char *buf;	/* holds the entire encoded line*/
-	int count = 0;
-	int lines_read = 0;
-	int put_chars = 0;
-
-	/*
-	 * First of all, catch everything that is left over from the last decoding.
-	 * If there's a newline in that rest, copy everything up to and including
-	 * that newline in the expected buffer, adjust rest and return. If there's
-	 * no newline in the rest, copy all of it (modulo length of the buffer) to
-	 * the expected buffer and continue as if there was no rest.
-	 */
-	put_chars = put_rest(rest, line, max_line_len, 0);
-	if (put_chars && ((*line)[put_chars - 1] == '\n'))
-		return 0;	/* we didn't read any new lines but filled the line */
-
-	/*
-	 * At this point, either there was no rest or there was no newline in the
-	 * rest. In any case, we need to read further encoded lines and decode
-	 * them until we find a newline or there are no more (encoded or physical)
-	 * lines in this part of the posting. To be sure, now allocate memory for
-	 * the output if it wasn't already done.
-	 */
-	if (*max_line_len == 0) {
-		*max_line_len = LEN;
-		*line = my_malloc(*max_line_len);
-	}
-
-	/*
-	 * max_lines_to_read==0 occurs at end of an encoded part and if there was
-	 * no trailing newline in the encoded text. So we put one there and exit.
-	 * FIXME: Adding a newline may be not correct. At least it may be not
-	 * what the author of that article intended. Unfortunately, a newline is
-	 * expected at the end of a line by some other code in cook.c.
-	 */
-	if (max_lines_to_read <= 0) {
-		if (put_chars) {
-			(*line)[put_chars++] = '\n';
-			(*line)[put_chars] = '\0';
-		}
-		return max_lines_to_read;
-	}
-	/*
-	 * Ok, now read a new line from the original article.
-	 */
-	do {
-		if ((buf = tin_fgets(file, FALSE)) == NULL) {
-			/*
-			 * Premature end of file (or file error), leave loop. To prevent
-			 * re-invoking of this function, set the numbers of read lines to
-			 * the expected maximum that should be read at most.
-			 *
-			 * FIXME: Adding a newline may be not correct. At least it may be
-			 * not what the author of that article intended. Unfortunately, a
-			 * newline is expected at the end of a line by some other code in
-			 * cook.c.
-			 */
-			if (put_chars > *max_line_len - 2) {
-				*max_line_len <<= 1;
-				*line = my_realloc(*line, *max_line_len);
-			}
-			(*line)[put_chars++] = '\n';
-			(*line)[put_chars] = '\0';
-			return max_lines_to_read;
-		}
-		lines_read++;
-		buf2 = my_malloc(strlen(buf) + 1); /* decoded string is always shorter than encoded string, so this is safe */
-		count = mmdecode(buf, 'b', '\0', buf2);
-		buf2[count] = '\0';
-		FreeIfNeeded(*rest);
-		*rest = buf2;
-		put_chars = put_rest(rest, line, max_line_len, put_chars);
-		if (put_chars && ((*line)[put_chars - 1] == '\n')) /* end of logical line reached */
-			return lines_read;
-	} while (lines_read < max_lines_to_read);
-	return lines_read;
-}
-
-
-/*
- * FIXME: should go into rfc1521.c
- *
- * Read a logical quoted-printable encoded line into the specified line
- * buffer. Quoted-printable lines can be split over several physical lines,
- * so this function collects all affected lines, concatenates and decodes
- * them.
- *
- * Side effects: resizes line if necessary, adjusts max_line_len
- * accordingly.
- *
- * This function returns the number of physical lines read or a negative
- * value on error.
- */
-static int
-read_decoded_qp_line(
-	FILE *file,
-	char **line,					/* where to copy the decoded line */
-	int *max_line_len,				/* (maximum) line length */
-	const int max_lines_to_read)	/* don't read more physical lines than told here */
-{
-	char *buf, *buf2;
-	char *ptr;
-	char c;
-	int buflen = LEN;
-	size_t chars_to_add = 0;
-	int count = 0;
-	int lines_read = 0;
-
-	buf = my_malloc(buflen); /* initial internal line buffer */
-	*buf = '\0';
-	do {
-		if ((buf2 = tin_fgets(file, FALSE)) == NULL) {
-			/*
-			 * Premature end of file (or file error, leave loop. To prevent
-			 * re-invokation of this function, set the numbers of read lines
-			 * to the expected maximum that should be read at most.
-			 */
-			lines_read = max_lines_to_read;
-			break;
-		}
-		lines_read++;
-		if ((chars_to_add = strlen(buf2)) == 0) /* Empty line, leave loop. */
-			break;
-
-		/*
-		 * Strip trailing white space at the end of the line.
-		 * See RFC 2045, section 6.7, #3
-		 */
-		c = buf2[chars_to_add - 1];
-		while ((chars_to_add > 0) && ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r'))) {
-			--chars_to_add;
-			c = (chars_to_add > 0 ? buf2[chars_to_add - 1] : '\0');
-		}
-
-		/*
-		 * '=' at the end of a line indicates a soft break meaning
-		 * that the following physical line "belongs" to this one.
-		 * (See RFC 2045, section 6.7, #5)
-		 *
-		 * Skip that equal sign now; since c holds this char, the
-		 * loop is not left but the next line is read and concatenated
-		 * with this one while the '=' is overwritten.
-		 */
-		if (c == '=') /* c is 0 when chars_to_add is 0 so this is safe */
-			buf2[--chars_to_add] = '\0';
-
-		/*
-		 * Join physical lines to a logical one; keep in mind that a LF is
-		 * added afterwards.
-		 */
-		if (chars_to_add > buflen - strlen(buf) - 2) {
-			buflen <<= 1;
-			buf = my_realloc(buf, buflen);
-		}
-		strncat(buf, buf2, buflen);
-	} while ((c == '=') && (lines_read < max_lines_to_read));
-	/*
-	 * re-add newline and NULL termination at end of line
-	 * FIXME: Adding a newline may be not correct. At least it may be not
-	 * what the author of that article intended. Unfortunately, a newline is
-	 * expected at the end of a line by some other code in cook.c.
-	 */
-	strcat(buf, "\n");
-
-	/*
-	 * Now decode complete (logical) line from buf to buf2 and copy it to the
-	 * buffer where the invoking function expects it. Don't decode directly
-	 * to the buffer of the other function to prevent buffer overruns and to
-	 * decide if the encoding was ok.
-	 */
-	buf2 = my_malloc(strlen(buf) + 1); /* Don't use realloc here, tin_fgets relies on its internal state! */
-	count = mmdecode(buf, 'q', '\0', buf2);
-
-	if (count >= 0) {
-		buf2[count] = '\0';
-		ptr = buf2;
-	} else	/* error in encoding: copy raw line */
-		ptr = buf;
-
-	if (*max_line_len < (int) strlen(ptr) + 1) {
-		*max_line_len = strlen(ptr) + 1;
-		*line = my_realloc(*line, *max_line_len);
-	}
-	strncpy(*line, ptr, *max_line_len);
-	(*line)[*max_line_len - 1] = '\0'; /* be sure to terminate string */
-	free(buf);
-	free(buf2);
-	return lines_read;
-}
-
-
 /*
  * Add a new uuencode attachment description to the current part
  */
@@ -573,6 +261,7 @@ new_uue(
 	if (!(*part)->uue)			/* new_part() is simple and doesn't attach list heads */
 		(*part)->uue = ptr;
 
+	free_list(ptr->params);
 	/*
 	 * Load the name into the parameter list
 	 */
@@ -718,7 +407,12 @@ process_text_body_part(
 			break;	/* premature end of file, file error etc. */
 
 		/* convert network to local charset, tex2iso, iso2asc etc. */
-		process_charsets(&line, &max_line_len, get_param(part->params, "charset"), tinrc.mm_local_charset, CURR_GROUP.attribute->tex2iso_conv && art->tex2iso);
+		process_charsets(&line, &max_line_len, get_param(part->params, "charset"), tinrc.mm_local_charset, curr_group->attribute->tex2iso_conv && art->tex2iso);
+
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		if (IS_LOCAL_CHARSET("UTF-8"))
+			utf8_valid(line);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 		len = (int) strlen(line);
 
@@ -742,6 +436,9 @@ process_text_body_part(
 		/*
 		 * Detect and process uuencoded sections
 		 * Look for the start or the end of a uuencoded section
+		 *
+		 * TODO: look for a tailing size line after end (non standard
+		 *       extension)?
 		 */
 		if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
 			in_uue = TRUE;
@@ -808,7 +505,7 @@ process_text_body_part(
 		 * Skip output if we're hiding uue or the sig
 		 */
 		if (in_uue && hide_uue)
-			continue;						/* No further processing needed */
+			continue;	/* No further processing needed */
 
 		flags = in_sig ? C_SIG : C_BODY;
 
@@ -907,8 +604,8 @@ static void
 dump_cooked(
 	void)
 {
-	int i;
 	char *line;
+	int i;
 
 	for (i = 0; i < art->cooked_lines; i++) {
 		fseek(art->cooked, art->cookl[i].offset, SEEK_SET);
@@ -981,11 +678,13 @@ cook_article(
 		}
 
 		if (header_wanted(line)) {	/* Put cooked data */
+			int i = LEN;
+			char *l = my_strdup(convert_body2printable(rfc1522_decode(line)));	/* FIXME: don't decode addr-part of From:/Cc:/ etc.pp. */
+
 			header_put = TRUE;
-			/*
-			 * FIXME: don't decode addr-part of From:/Cc:/ etc.pp.
-			 */
-			put_cooked(LEN, wrap_lines, C_HEADER, "%s\n", rfc1522_decode(line));
+			expand_ctrl_chars(&l, &i, tabs);
+			put_cooked(LEN, wrap_lines, C_HEADER, "%s", l);
+			free(l);
 		}
 	}
 
