@@ -3,10 +3,10 @@
  *  Module    : debug.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-04-28
+ *  Updated   : 2005-05-04
  *  Notes     : debug routines
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,6 +40,9 @@
 #endif /* !TIN_H */
 
 #if defined(DEBUG) || defined(DEBUG_NEWSRC)
+#	ifndef NEWSRC_H
+#		include "newsrc.h"
+#	endif /* !NEWSRC_H */
 #	ifndef TCURSES_H
 #		include "tcurses.h"
 #	endif /* !TCURSES_H */
@@ -91,21 +94,29 @@ debug_delete_files(
 void
 debug_nntp(
 	const char *func,
-	const char *line)
+	const char *fmt,
+	...)
 {
-	char file[PATH_LEN];
 	FILE *fp;
+	char *buf;
+	char file[PATH_LEN];
+	va_list ap;
 
 	if (!debug)
 		return;
 
+	va_start(ap, fmt);
+	buf = fmt_message(fmt, ap);
+
 	joinpath(file, TMPDIR, "NNTP");
 
 	if ((fp = fopen(file, "a+")) != NULL) {
-		fprintf(fp,"%s: %s\n", func, line);
+		fprintf(fp,"%s: %s\n", func, buf);
 		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
 		fclose(fp);
 	}
+	free(buf);
+	va_end(ap);
 }
 
 
@@ -130,8 +141,8 @@ void
 debug_print_header(
 	struct t_article *s)
 {
-	char file[PATH_LEN];
 	FILE *fp;
+	char file[PATH_LEN];
 
 	if (debug != 2)
 		return;
@@ -158,7 +169,6 @@ debug_print_header(
 				fprintf(fp, "archive.ispart=[%s]\n", bool_unparse(s->archive->ispart));
 		}
 		fprintf(fp,"thread=[%d]  prev=[%d]  status=[%d]\n\n", s->thread, s->prev, s->status);
-/*		fprintf(fp,"thread=[%s]  prev=[%s]  status=[%s]\n", (s->thread == ART_NORMAL ? "ART_NORMAL" : "ART_EXPIRED"), s->prev, bool_unparse(s->status)); */
 		fflush(fp);
 		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
 		fclose(fp);
@@ -170,8 +180,8 @@ void
 debug_print_active(
 	void)
 {
-	char file[PATH_LEN];
 	FILE *fp;
+	char file[PATH_LEN];
 	int i;
 	struct t_group *group;
 
@@ -240,25 +250,25 @@ debug_print_attributes(
 
 
 void
-vDbgPrintMalloc(
-	int iIsMalloc,
-	const char *pcFile,
-	int iLine,
-	size_t iSize)
+debug_print_malloc(
+	int is_malloc,
+	const char *xfile,
+	int line,
+	size_t size)
 {
 	FILE *fp;
 	char file[PATH_LEN];
-	static int iTotal = 0;
+	static int total = 0;
 
 	if (debug == 4) {
 		joinpath(file, TMPDIR, "MALLOC");
 		if ((fp = fopen(file, "a+")) != NULL) {
-			iTotal += iSize;
+			total += size;
 			/* sometimes size_t is long */
-			if (iIsMalloc)
-				fprintf(fp, "%10s:%-4d Malloc  %6ld. Total %d\n", pcFile, iLine, (long) iSize, iTotal);
+			if (is_malloc)
+				fprintf(fp, "%10s:%-4d Malloc  %6ld. Total %d\n", xfile, line, (long) size, total);
 			else
-				fprintf(fp, "%10s:%-4d Realloc %6ld. Total %d\n", pcFile, iLine, (long) iSize, iTotal);
+				fprintf(fp, "%10s:%-4d Realloc %6ld. Total %d\n", xfile, line, (long) size, total);
 			fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
 			fclose(fp);
 		}
@@ -293,8 +303,8 @@ void
 debug_print_filters(
 	void)
 {
-	char file[PATH_LEN];
 	FILE *fp;
+	char file[PATH_LEN];
 	int i, num;
 	struct t_filter *the_filter;
 
@@ -320,15 +330,32 @@ debug_print_filters(
 		fclose(fp);
 	}
 }
+
+
+/* TODO: print out all fields of t_capabilities */
+#	ifdef NNTP_ABLE
+void
+debug_print_nntp_extensions(
+	void)
+{
+	if (debug < 2)
+		return;
+	debug_nntp("###", "NNTP EXTENSIONS/CAPABILITIES");
+	debug_nntp("###", "Type/Version : %d/%d", nntp_caps.type, nntp_caps.version);
+	debug_nntp("###", "Command-names: %s %s", BlankIfNull(nntp_caps.over_cmd), BlankIfNull(nntp_caps.hdr_cmd));
+	debug_nntp("###", "List         : %s", nntp_caps.list_motd ? "MOTD" : "");
+}
+#	endif /* NNTP_ABLE */
 #endif /* DEBUG */
 
+
 #ifdef DEBUG_NEWSRC
 void
 debug_print_comment(
 	const char *comment)
 {
-	char file[PATH_LEN];
 	FILE *fp;
+	char file[PATH_LEN];
 
 	if (debug < 2)
 		return;
@@ -348,8 +375,8 @@ debug_print_bitmap(
 	struct t_group *group,
 	struct t_article *art)
 {
-	char file[PATH_LEN];
 	FILE *fp;
+	char file[PATH_LEN];
 
 	if (debug != 3)
 		return;
@@ -367,9 +394,8 @@ debug_print_bitmap(
 				bool_unparse(art->killed),
 				bool_unparse(art->selected),
 				art->subject);
-			fprintf(fp, "thread=[%s]  prev=[%d]  status=[%s]\n",
-				(art->thread == ART_NORMAL ? "ART_NORMAL" : "ART_EXPIRED"),
-				art->prev,
+			fprintf(fp, "thread=[%d]  prev=[%d]  status=[%s]\n",
+				art->thread, art->prev,
 				(art->status == ART_READ ? "READ" : "UNREAD"));
 		}
 		debug_print_newsrc(&group->newsrc, fp);
@@ -381,22 +407,22 @@ debug_print_bitmap(
 
 void
 debug_print_newsrc(
-	struct t_newsrc *NewSrc,
+	struct t_newsrc *lnewsrc,
 	FILE *fp)
 {
 	int i, j;
 
 	fprintf(fp, "Newsrc: min=[%ld] max=[%ld] bitlen=[%ld] num_unread=[%ld] present=[%d]\n",
-		NewSrc->xmin, NewSrc->xmax, NewSrc->xbitlen,
-		NewSrc->num_unread, NewSrc->present);
+		lnewsrc->xmin, lnewsrc->xmax, lnewsrc->xbitlen,
+		lnewsrc->num_unread, lnewsrc->present);
 
 	fprintf(fp, "bitmap=[");
-	if (NewSrc->xbitlen && NewSrc->xbitmap) {
-		for (j = 0, i = NewSrc->xmin; i <= NewSrc->xmax; i++) {
+	if (lnewsrc->xbitlen && lnewsrc->xbitmap) {
+		for (j = 0, i = lnewsrc->xmin; i <= lnewsrc->xmax; i++) {
 			fprintf(fp, "%d",
-				(NTEST(NewSrc->xbitmap, i - NewSrc->xmin) == ART_READ ?
+				(NTEST(lnewsrc->xbitmap, i - lnewsrc->xmin) == ART_READ ?
 				ART_READ : ART_UNREAD));
-			if ((j++ % 8) == 7 && i < NewSrc->xmax)
+			if ((j++ % 8) == 7 && i < lnewsrc->xmax)
 				fprintf(fp, " ");
 		}
 	}
