@@ -3,9 +3,9 @@
  *  Module    : header.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
- *  Updated   : 1997-03-19
+ *  Updated   : 2005-07-09
  *
- * Copyright (c) 1997-2003 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2005 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,23 +55,18 @@ get_host_name(
 	hostname[0] = '\0';
 
 #ifdef HAVE_GETHOSTNAME
-	gethostname(hostname, sizeof(hostname));
-#else
-#	ifdef M_AMIGA
-	if ((ptr = getenv("NodeName")) != NULL)
-		strncpy(hostname, ptr, MAXHOSTNAMELEN);
-#	endif /* M_AMIGA */
+	gethostname(hostname, sizeof(hostname) - 1);
 #endif /* HAVE_GETHOSTNAME */
 #ifdef HAVE_SYS_UTSNAME_H
 	if (!*hostname)
-		strcpy(hostname, system_info.nodename);
+		strncpy(hostname, system_info.nodename, sizeof(hostname) - 1);
 #endif /* HAVE_SYS_UTSNAME_H */
 	if (!*hostname) {
 		if ((ptr = getenv("HOST")) != NULL)
-			strncpy(hostname, ptr, MAXHOSTNAMELEN);
+			my_strncpy(hostname, ptr, sizeof(hostname) - 1);
 		else {
 			if ((ptr = getenv("HOSTNAME")) != NULL)
-				strncpy(hostname, ptr, MAXHOSTNAMELEN);
+				my_strncpy(hostname, ptr, sizeof(hostname) - 1);
 			else
 				hostname[0] = '\0';
 		}
@@ -82,40 +77,28 @@ get_host_name(
 
 
 #ifdef DOMAIN_NAME
-/* find domainname - check DOMAIN_NAME */
-/* TODO: check /etc/defaultdomain as a last resort? */
+/*
+ * find domainname - check DOMAIN_NAME
+ * TODO: check /etc/defaultdomain as a last resort?
+ *       there is M_UNIX specific code in here, but no VMS
+ *       alternative
+ */
 const char *
 get_domain_name(
 	void)
 {
-	char *ptr;
-	char buff[MAXHOSTNAMELEN];
 	FILE *fp;
+	char *ptr;
+	char buff[MAXHOSTNAMELEN + 1];
 	static char domain[8192];
 
-#	ifdef M_AMIGA
-/*
- * Damn compiler bugs...
- * Without this hack, SASC 6.55 produces a TST.B d16(pc),
- * which is illegal on a 68000
- */
-static const char *domain_name_hack = DOMAIN_NAME;
-#		undef DOMAIN_NAME
-#		define DOMAIN_NAME domain_name_hack
-#	endif /* M_AMIGA */
-
 	domain[0] = '\0';
 
 	if (strlen(DOMAIN_NAME))
 		strcpy(domain, DOMAIN_NAME);
 
-#	ifdef M_AMIGA
-	if (strchr(domain, ':')) /* absolute AmigaOS paths contain one, RFC-hostnames don't */
-#	else
-	if (domain[0] == '/' && domain[1])
-#	endif /* M_AMIGA */
-	{
-		/* If 1st letter is '/' read domainname from specified file */
+	if (domain[0] == '/' && domain[1]) {
+		/* read domainname from specified file */
 		if ((fp = fopen(domain, "r")) != NULL) {
 			while (fgets(buff, (int) sizeof(buff), fp) != NULL) {
 				if (buff[0] == '#' || buff[0] == '\n')
@@ -126,14 +109,14 @@ static const char *domain_name_hack = DOMAIN_NAME;
 					strcpy(domain, buff);
 				}
 			}
-			if (domain[0] == '/') /* file was empty */
+			if (domain[0] == '/')	/* '/' is not allowed in domainames -> file was empty */
 				domain[0] = '\0';
 
 			fclose(fp);
 		} else
 			domain[0] = '\0';
 	}
-	domain[MAXHOSTNAMELEN] = '\0';
+	domain[MAXHOSTNAMELEN + 1] = '\0';
 	return domain;
 }
 #endif /* DOMAIN_NAME */
@@ -149,7 +132,7 @@ get_fqdn(
 {
 	char *domain;
 	char line[MAXLINELEN + 1];
-	char name[MAXHOSTNAMELEN + 2];
+	char name[MAXHOSTNAMELEN + 1];
 	static char fqdn[1024];
 	struct hostent *hp;
 	struct in_addr in;
@@ -161,18 +144,19 @@ get_fqdn(
 	if (host) {
 		if (strchr(host, '.'))
 			return host;
-		(void) strncpy(name, host, MAXHOSTNAMELEN);
+		strncpy(name, host, sizeof(name) - 1);
 	} else {
 #	ifdef HAVE_GETHOSTNAME
-		if (gethostname(name, MAXHOSTNAMELEN))
+		if (gethostname(name, sizeof(name) - 1))
 #	endif /* HAVE_GETHOSTNAME */
 			return NULL;
 	}
 
 #	ifdef HAVE_INET_ADDR
 	if ('0' <= *name && *name <= '9') {
-		in.s_addr = inet_addr(name);
-		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
+		in_addr_t addr = inet_addr(name);
+
+		if ((hp = gethostbyaddr((char *) &addr, 4, AF_INET)))
 			in.s_addr = (*hp->h_addr);
 		return (hp && strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in));
 	}
@@ -181,7 +165,7 @@ get_fqdn(
 		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
 			in.s_addr = (*hp->h_addr);
 
-	sprintf(fqdn, "%s", hp
+	snprintf(fqdn, sizeof(fqdn), "%s", hp
 		? strchr(hp->h_name, '.')
 			? hp->h_name : inet_ntoa(in)
 		: "");
@@ -211,7 +195,7 @@ get_fqdn(
 				}
 			}
 			if (domain)
-				sprintf(fqdn, "%s.%s", name, strip_line(domain));
+				snprintf(fqdn, sizeof(fqdn), "%s.%s", name, strip_line(domain));
 
 			fclose(inf);
 		}
@@ -245,28 +229,34 @@ static const char *
 get_user_name(
 	void)
 {
-#	if defined(M_AMIGA) || defined(VMS)
-	char *p;
-#	endif /* M_AMIGA || VMS */
 	static char username[128];
 	struct passwd *pw;
+#ifdef VMS
+	char *p;
+#endif /* VMS */
 
 	username[0] = '\0';
-#	if defined(M_AMIGA) || defined(VMS)
-	if ((p = getenv("USER"))) {
-		STRCPY(username, p);
-#		ifdef VMS
-		lower(username);
-#		endif /* VMS */
-	}
-#	else
+
+#ifndef VMS
 	if ((pw = getpwuid(getuid())) != NULL)
 		strcpy(username, pw->pw_name);
-	else {
+	else
+#else
+	if (((p = getlogin()) != NULL) && strlen(p)) {
+		if ((pw = getpwnam(p)) != NULL)
+			strcpy(username, pw->pw_name);
+	}
+#endif /* !VMS */
+
+	if (!*username) {
 		error_message(_(txt_error_passwd_missing));
 		tin_done(EXIT_FAILURE);
 	}
-#	endif /* M_AMIGA || VMS */
+
+#	ifdef VMS
+	lower(username);
+#	endif /* VMS */
+
 	return username;
 }
 
@@ -286,11 +276,11 @@ get_full_name(
 	fullname[0] = '\0';
 
 	if ((p = getenv("NAME")) != NULL) {
-		strncpy(fullname, p, sizeof(fullname));
+		my_strncpy(fullname, p, sizeof(fullname) - 1);
 		return fullname;
 	}
 	if ((p = getenv("REALNAME")) != NULL) {
-		strncpy(fullname, p, sizeof(fullname));
+		my_strncpy(fullname, p, sizeof(fullname) - 1);
 		return fullname;
 	}
 
@@ -307,7 +297,7 @@ get_full_name(
 			STRCPY(tmp, pw->pw_name);
 			if (*tmp && isalpha((int)(unsigned char) *tmp) && islower((int)(unsigned char) *tmp))
 				*tmp = toupper((int)(unsigned char) *tmp);
-			snprintf(fullname, sizeof(fullname) - 1, "%s%s%s", buf, tmp, p);
+			snprintf(fullname, sizeof(fullname), "%s%s%s", buf, tmp, p);
 		} else
 			STRCPY(fullname, buf);
 	}
@@ -368,7 +358,7 @@ build_sender(
 	sender[0] = '\0';
 
 	if ((ptr = get_full_name()))
-		snprintf(sender, sizeof(sender) - 1, ((strpbrk(ptr, "\".:;<>@[]()\\")) ? "\"%s\"" : "%s "), ptr);
+		snprintf(sender, sizeof(sender), ((strpbrk(ptr, "\".:;<>@[]()\\")) ? "\"%s\"" : "%s "), ptr);
 	if ((ptr = get_user_name())) {
 		strcat(sender, "<");
 		strcat(sender, ptr);
@@ -383,9 +373,9 @@ build_sender(
 			strcat(sender, ptr);
 			strcat(sender, ">");
 		} else
-			return (char *) 0;
+			return NULL;
 	} else
-		return (char *) 0;
+		return NULL;
 
 	return sender;
 }
