@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-08-10
+ *  Updated   : 2005-08-14
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,9 +44,6 @@
 #ifndef included_trace_h
 #	include "trace.h"
 #endif /* !included_trace_h */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
 #ifndef VERSION_H
 #	include "version.h"
 #endif /* !VERSION_H */
@@ -58,6 +55,9 @@
  * local prototypes
  */
 static int read_site_config(void);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	static t_bool utf8_pcre(void);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #ifdef HAVE_COLOR
 	static void preinit_colors(void);
 #endif /* HAVE_COLOR */
@@ -66,7 +66,7 @@ static int read_site_config(void);
 char **news_headers_to_display_array;	/* array of which headers to display */
 char **news_headers_to_not_display_array;	/* array of which headers to not display */
 char active_times_file[PATH_LEN];
-char article[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
+char article_name[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
 char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
 char bug_nntpserver2[PATH_LEN];		/* welcome message of NNTP server used */
 char cmdline_nntpserver[PATH_LEN];
@@ -79,7 +79,6 @@ char domain_name[MAXHOSTNAMELEN];
 char global_attributes_file[PATH_LEN];
 char global_config_file[PATH_LEN];
 char homedir[PATH_LEN];
-char host_name[MAXHOSTNAMELEN];
 char index_maildir[PATH_LEN];
 char index_newsdir[PATH_LEN];	/* directory for private overview data */
 char index_savedir[PATH_LEN];
@@ -92,7 +91,6 @@ char local_newsgroups_file[PATH_LEN];	/* local copy of NNTP newsgroups file */
 char local_newsrctable_file[PATH_LEN];
 char lock_file[PATH_LEN];		/* contains name of index lock file */
 char filter_file[PATH_LEN];
-char mail_active_file[PATH_LEN];
 char mail_news_user[LEN];		/* mail new news to this user address */
 char mailbox[PATH_LEN];			/* system mailbox for each user */
 char mailer[PATH_LEN];			/* mail program */
@@ -111,6 +109,7 @@ char tin_progname[PATH_LEN];		/* program name */
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
 #ifdef HAVE_MH_MAIL_HANDLING
+	char mail_active_file[PATH_LEN];
 	char mailgroups_file[PATH_LEN];
 #endif /* HAVE_MH_MAIL_HANDLING */
 #ifndef NNTP_ONLY
@@ -122,20 +121,17 @@ char userid[PATH_LEN];
 	char rcdir_asfile[PATH_LEN];	/* rcdir expressed as dev:[dir]tin.dir, for stat() */
 #endif /* VMS */
 
-int groupname_len = 0;			/* 'runtime' copy of groupname_max_len */
+t_function last_search;	/* for repeated search */
 int hist_last[HIST_MAXNUM + 1];
 int hist_pos[HIST_MAXNUM + 1];
-int i_key_search_last;			/* for repeated search */
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
 int num_headers_to_display;		/* num headers to display -- swp */
 int num_headers_to_not_display;		/* num headers to not display -- swp */
 int system_status;
 int xmouse, xrow, xcol;			/* xterm button pressing information */
 
-mode_t real_umask;
 pid_t process_id;			/* Useful to have around for .suffixes */
 
-t_bool (*wildcard_func) (const char *str, char *patt, t_bool icase);		/* Wildcard matching function */
 t_bool batch_mode;			/* update index files only mode */
 t_bool check_for_new_newsgroups;	/* don't check for new newsgroups */
 t_bool cmd_line;			/* batch / interactive mode */
@@ -276,7 +272,7 @@ struct t_config tinrc = {
 	FILTER_SUBJ_CASE_SENSITIVE,		/* default_filter_kill_header */
 	FILTER_SUBJ_CASE_SENSITIVE,		/* default_filter_select_header */
 	0,		/* default_move_group */
-	iKeySaveAppendFile,		/* default_save_mode */
+	'a',		/* default_save_mode */
 	0,		/* getart_limit */
 	2,		/* recent_time */
 	32,		/* groupname_max_length */
@@ -292,6 +288,7 @@ struct t_config tinrc = {
 	SORT_THREADS_BY_SCORE_DESCEND,		/* sort_threads_type */
 	BOGUS_SHOW,		/* strip_bogus */
 	THREAD_BOTH,		/* thread_articles */
+	THREAD_PERC_DEFAULT,	/* thread_perc */
 	THREAD_SCORE_MAX,	/* thread_score */
 	0,		/* Default to wildmat, not regex */
 	-50,		/* score_limit_kill */
@@ -318,6 +315,7 @@ struct t_config tinrc = {
 	0,		/* col_quote3 (initialised later) */
 	0,		/* col_response (initialised later) */
 	0,		/* col_signature (initialised later) */
+	0,		/* col_urls (initialised later) */
 	0,		/* col_subject (initialised later) */
 	0,		/* col_text (initialised later) */
 	0,		/* col_title (initialised later) */
@@ -328,6 +326,7 @@ struct t_config tinrc = {
 	5,		/* mono_markslash */
 	3,		/* mono_markstroke */
 	TRUE,		/* word_highlight */
+	TRUE,		/* url_highlight */
 	0,		/* wrap_column */
 #ifdef HAVE_COLOR
 	FALSE,		/* use_color */
@@ -374,7 +373,7 @@ struct t_config tinrc = {
 	FALSE,		/* prompt_followupto */
 	QUOTE_COMPRESS|QUOTE_EMPTY,	/* quote_style */
 	TRUE,		/* show_description */
-	1,		/* show_infos */
+	SHOW_INFO_LINES,		/* show_info */
 	TRUE,		/* show_only_unread_arts */
 	FALSE,		/* show_only_unread_groups */
 	TRUE,		/* show_signatures */
@@ -393,7 +392,7 @@ struct t_config tinrc = {
 	TRUE,		/* thread_catchup_on_exit */
 	TRUE,		/* unlink_article */
 	"",		/* inews_prog */
-	FALSE,		/* use_mailreader_i */
+	INTERACTIVE_NONE,		/* interactive_mailer */
 	FALSE,		/* use_mouse */
 #ifdef HAVE_KEYPAD
 	FALSE,		/* use_keypad */
@@ -408,9 +407,56 @@ struct t_config tinrc = {
 #ifdef XFACE_ABLE
 	FALSE,		/* use_slrnface */
 #endif /* XFACE_ABLE */
-	TRUE		/* default_filter_select_global */
+	TRUE,		/* default_filter_select_global */
+	DEFAULT_DATE_FORMAT,	/* date_format */
+#ifdef HAVE_UNICODE_NORMALIZATION
+	NORMALIZE_NFKC,		/* normalization form */
+#endif /* HAVE_UNICODE_NORMALIZATION */
+#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
+	FALSE		/* render_bidi */
+#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
 };
 
+struct t_capabilities nntp_caps = {
+	0, /* type (none, LIST EXTENSIONS, CAPABILITIES) */
+	0, /* CAPABILITIES version */
+	FALSE, /* MODE-READER: "MODE READER" */
+	FALSE, /* READER: "ARTICLE", "BODY" */
+	FALSE, /* POST */
+	FALSE, /* LIST: "LIST ACTIVE" */
+	FALSE, /* LIST: "LIST ACTIVE.TIMES" */
+	FALSE, /* LIST: "LIST DISTRIB.PATS" */
+	FALSE, /* LIST: "LIST HEADERS" */
+	FALSE, /* LIST: "LIST NEWSGROUPS" */
+	FALSE, /* LIST: "LIST OVERVIEW.FMT" */
+	FALSE, /* LIST: "LIST MOTD" */
+	FALSE, /* LIST: "LIST SUBSCRIPTIONS" */
+	FALSE, /* LIST: "LIST DISTRIBUTIONS" */
+	FALSE, /* LIST: "LIST MODERATORS" */
+	FALSE, /* XPAT */
+	FALSE, /* HDR: "HDR", "LIST HEADERS" */
+	NULL, /* [X]HDR */
+	FALSE, /* OVER: "OVER", "LIST OVERVIEW.FMT" */
+	FALSE, /* OVER: "OVER mid" */
+	NULL, /* [X]OVER */
+	FALSE, /* NEWNEWS */
+	NULL, /* IMPLEMENTATION */
+	FALSE, /* STARTTLS */
+	FALSE, /* AUTHINFO USER/PASS */
+	FALSE, /* AUTHINFO SASL */
+#if 0
+	FALSE, /* SASL CRAM-MD5 */
+	FALSE, /* SASL DIGEST-MD5 */
+	FALSE, /* SASL PLAIN */
+	FALSE, /* SASL GSSAPI */
+	FALSE, /* SASL EXTERNAL */
+	FALSE, /* STREAMING: "MODE STREAM", "CHECK", "TAKETHIS" */
+	FALSE /* IHAVE */
+#endif /* 0 */
+};
+
+static mode_t real_umask;
+
 #ifdef HAVE_COLOR
 
 #	define DFT_FORE -1
@@ -440,6 +486,7 @@ static const struct {
 	{ &tinrc.col_quote3,      4 },
 	{ &tinrc.col_response,    2 },
 	{ &tinrc.col_signature,   4 },
+	{ &tinrc.col_urls,       -1 },
 	{ &tinrc.col_subject,     6 },
 	{ &tinrc.col_text,       DFT_FORE },
 	{ &tinrc.col_title,       4 },
@@ -491,30 +538,27 @@ void
 init_selfinfo(
 	void)
 {
+	FILE *fp;
 	char *ptr;
 	const char *cptr;
-	FILE *fp;
 	struct stat sb;
-#ifndef M_AMIGA
 	struct passwd *myentry;
-#endif /* !M_AMIGA */
 
-	host_name[0] = '\0';
 	domain_name[0] = '\0';
 
-#ifndef M_AMIGA /* TODO: why do we exclude M_AMIGA here but not in main.c:read_cmd_line_options() */
-#	if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
-	if (uname(&system_info) < 0) {
+#ifdef HAVE_SYS_UTSNAME_H
+#	ifdef HAVE_UNAME
+	if (uname(&system_info) != -1)
+		;
+	else
+#	endif /* HAVE_UNAME */
+	{
 		strcpy(system_info.sysname, "unknown");
 		*system_info.machine = '\0';
 		*system_info.release = '\0';
 		*system_info.nodename = '\0';
 	}
-#	endif /* HAVE_SYS_UTSNAME_H && HAVE_UNAME */
-#endif /* !M_AMIGA */
-
-	if ((cptr = get_host_name()) != NULL)
-		strcpy(host_name, cptr);
+#endif /* HAVE_SYS_UTSNAME_H */
 
 #ifdef DOMAIN_NAME
 	if ((cptr = get_domain_name()) != NULL)
@@ -523,8 +567,8 @@ init_selfinfo(
 
 #ifdef HAVE_GETHOSTBYNAME
 	if (domain_name[0] == '\0') {
-		cptr = ((host_name[0] == '\0') ? get_fqdn((char *) 0) : get_fqdn(host_name));
-		if (cptr != (char *) 0)
+		cptr = get_fqdn(get_host_name());
+		if (cptr != NULL)
 			strcpy(domain_name, cptr);
 	}
 #endif /* HAVE_GETHOSTBYNAME */
@@ -534,46 +578,33 @@ init_selfinfo(
 	real_umask = umask(0);
 	(void) umask(real_umask);
 
-#ifndef M_AMIGA
-#	ifndef VMS
+#ifndef VMS
 	if ((myentry = getpwuid(getuid())) == NULL) {
 		error_message(_(txt_error_passwd_missing));
 		giveup();
 	}
-#	else
+#else
 	if (((ptr = getlogin()) != NULL) && strlen(ptr))
 		myentry = getpwnam(ptr);
 	else {
 		error_message(_(txt_error_passwd_missing));
 		giveup();
 	}
-#	endif /* !VMS */
+#endif /* !VMS */
 
 	strcpy(userid, myentry->pw_name);
 
-#	ifdef VMS
+#ifdef VMS
 	lower(userid);
-#	endif /* VMS */
-
-#else
-	if ((ptr = getenv("USERNAME")) == NULL) {
-		error_message(_(txt_env_var_not_found), "USERNAME");
-		giveup();
-	}
-	my_strncpy(userid, ptr, sizeof(userid) - 1);
-#endif /* !M_AMIGA */
+#endif /* VMS */
 
 	if (((ptr = getenv("TIN_HOMEDIR")) != NULL) && strlen(ptr)) {
 		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
 	} else if (((ptr = getenv("HOME")) != NULL) && strlen(ptr)) {
 		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
-	}
-#ifndef M_AMIGA
-	else if (strlen(myentry->pw_dir)) {
+	} else if (strlen(myentry->pw_dir)) {
 		strncpy(homedir, myentry->pw_dir, sizeof(homedir) - 1);
-	}
-#endif /* !M_AMIGA */
-	 else
+	} else
 		strncpy(homedir, TMPDIR, sizeof(homedir) - 1);
 
 	cmdline_nntpserver[0] = '\0';
@@ -604,7 +635,6 @@ init_selfinfo(
 	reread_active_for_posted_arts = TRUE;
 	batch_mode = FALSE;
 	check_for_new_newsgroups = TRUE;
-	wildcard_func = wildmat;
 
 #ifdef HAVE_COLOR
 	preinit_colors();
@@ -618,10 +648,10 @@ init_selfinfo(
 	index_savedir[0] = '\0';
 	newsrc[0] = '\0';
 
-	sprintf(page_header, "%s %s release %s (\"%s\") [%s%s]",
-		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME,
+	snprintf(page_header, sizeof(page_header), "%s %s release %s (\"%s\") [%s%s]",
+		PRODUCT, VERSION, RELEASEDATE, RELEASENAME, OSNAME,
 		(iso2asc_supported >= 0 ? " ISO2ASC" : ""));
-	sprintf(cvers, txt_copyright_notice, page_header);
+	snprintf(cvers, sizeof(cvers), txt_copyright_notice, page_header);
 
 	default_organization[0] = '\0';
 	news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
@@ -661,11 +691,6 @@ init_selfinfo(
 	newsgroups_file[0] = '\0';
 	subscriptions_file[0] = '\0';
 
-	/*
-	 * Setup default keymaps
-	 */
-	build_keymaps();
-
 	/*
 	 * read the global site config file to override some default
 	 * values given at compile time
@@ -682,14 +707,6 @@ init_selfinfo(
 		force_no_post = TRUE;
 	}
 
-	/*
-	 * Amiga uses assigns which end in a ':' and won't work with a '/'
-	 * tacked on after them: e.g. we want UULIB:active, and not
-	 * UULIB:/active. For this reason I have changed the sprintf calls
-	 * to joinpath. This is defined to sprintf(result,"%s/%s",dir,file)
-	 * on all UNIX systems.
-	 */
-
 	/*
 	 * only set the following variables if they weren't set from within
 	 * read_site_config()
@@ -768,15 +785,11 @@ init_selfinfo(
 	my_strncpy(mailer, get_val(ENV_VAR_MAILER, DEFAULT_MAILER), sizeof(mailer) - 1);
 #ifndef DISABLE_PRINTING
 	strcpy(tinrc.printer, DEFAULT_PRINTER);
-#	ifdef M_AMIGA
-	if (tin_bbs_mode)
-		strcpy(tinrc.printer, DEFAULT_BBS_PRINTER);
-#	endif /* M_AMIGA */
 #endif /* !DISABLE_PRINTING */
 	strcpy(tinrc.inews_prog, PATH_INEWS);
-	joinpath(article, homedir, TIN_ARTICLE_NAME);
+	joinpath(article_name, homedir, TIN_ARTICLE_NAME);
 #ifdef APPEND_PID
-	snprintf(article + strlen(article), sizeof(article) - strlen(article), ".%d", (int) process_id);
+	snprintf(article_name + strlen(article_name), sizeof(article_name) - strlen(article_name), ".%d", (int) process_id);
 #endif /* APPEND_PID */
 	joinpath(dead_article, homedir, "dead.article");
 	joinpath(dead_articles, homedir, "dead.articles");
@@ -798,8 +811,9 @@ init_selfinfo(
 	joinpath(filter_file, rcdir, FILTER_FILE);
 	joinpath(local_input_history_file, rcdir, INPUT_HISTORY_FILE);
 	joinpath(local_newsrctable_file, rcdir, NEWSRCTABLE_FILE);
-	joinpath(local_newsgroups_file, rcdir, NEWSGROUPS_FILE);
+#ifdef HAVE_MH_MAIL_HANDLING
 	joinpath(mail_active_file, rcdir, ACTIVE_MAIL_FILE);
+#endif /* HAVE_MH_MAIL_HANDLING */
 #ifdef VMS
 	joinpath(mailbox, DEFAULT_MAILBOX, "MAIL.TXT");
 #else
@@ -835,7 +849,7 @@ init_selfinfo(
 		write_attributes_file(local_attributes_file);
 
 	init_postinfo();
-	sprintf(txt_help_bug_report, _(txt_help_bug), bug_addr);
+	snprintf(txt_help_bug_report, sizeof(txt_help_bug_report), _(txt_help_bug), bug_addr);
 
 #ifdef HAVE_PGP_GPG
 	init_pgp();
@@ -935,8 +949,34 @@ postinit_regexp(
 	if (!strlen(tinrc.strip_re_regex))
 		STRCPY(tinrc.strip_re_regex, DEFAULT_STRIP_RE_REGEX);
 	compile_regex(tinrc.strip_re_regex, &strip_re_regex, PCRE_ANCHORED);
-	if (!strlen(tinrc.strip_was_regex))
-		STRCPY(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX);
+
+	if (strlen(tinrc.strip_was_regex)) {
+		/*
+		 * try to be clever, if we still use the initial default value
+		 * convert it to our needs
+		 *
+		 * TODO: a global soultion
+		 */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			if (IS_LOCAL_CHARSET("UTF-8") && utf8_pcre()) {
+				if (!strcmp(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX))
+					STRCPY(tinrc.strip_was_regex, DEFAULT_U8_STRIP_WAS_REGEX);
+			} else {
+				if (!strcmp(tinrc.strip_was_regex, DEFAULT_U8_STRIP_WAS_REGEX))
+					STRCPY(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX);
+			}
+#else
+			if (!strcmp(tinrc.strip_was_regex, DEFAULT_U8_STRIP_WAS_REGEX))
+				STRCPY(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	} else {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		if (IS_LOCAL_CHARSET("UTF-8") && utf8_pcre())
+			STRCPY(tinrc.strip_was_regex, DEFAULT_U8_STRIP_WAS_REGEX);
+		else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+			STRCPY(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX);
+	}
 	compile_regex(tinrc.strip_was_regex, &strip_was_regex, 0);
 
 #ifdef HAVE_COLOR
@@ -973,3 +1013,19 @@ postinit_regexp(
 
 	compile_regex(SHAR_REGEX, &shar_regex, PCRE_ANCHORED);
 }
+
+
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+static t_bool
+utf8_pcre(
+	void)
+{
+	int i = 0;
+
+#	if (defined(PCRE_MAJOR) && PCRE_MAJOR >= 4)
+	(void) pcre_config(PCRE_CONFIG_UTF8, &i);
+#	endif /* PCRE_MAJOR && PCRE_MAJOR >= $*/
+
+	return (i ? TRUE : FALSE);
+}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
