@@ -6,7 +6,7 @@
  *  Updated   : 2003-05-16
  *  Notes     :
  *
- * Copyright (c) 1998-2003 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1998-2005 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,7 @@
  *
  * try to lock a file descriptor with fcntl(), flock() or lockf()
  *
- * returncodes:
+ * return codes:
  *  0 = file locked successfully
  * -1 = some error occured
  */
@@ -105,7 +105,7 @@ fd_lock(
  * check for an existing lock on file descriptor with fcntl(), lockf()
  * or flock()
  *
- * returncodes:
+ * return codes:
  *  0 = file is not locked
  *  1 = file is locked
  * -1 = some error occured
@@ -167,7 +167,7 @@ test_fd_lock(
  *
  * try to unlock a file descriptor with fcntl(), lockf() or flock()
  *
- * returncodes:
+ * return codes:
  *  0 = file unlocked successfully
  * -1 = some error occured
  */
@@ -209,7 +209,7 @@ fd_unlock(
  *
  * try to lock filename via dotfile locking
  *
- * returncodes:
+ * return codes:
  *  TRUE  = file locked successfully
  *  FALSE = some error occured
  */
@@ -219,16 +219,15 @@ t_bool dot_lock(
 	char tempfile[PATH_LEN];
 	char lockfile[PATH_LEN];
 	char base_dir[PATH_LEN];
-	int dot_fd = -1;
+	int dot_fd;
 	struct stat statbuf;
 	t_bool rval = FALSE;
 
 	dir_name(filename, base_dir);
 	if (!strcmp(filename, base_dir)) /* no filename portion */
 		return rval;
-	if ((dot_fd = my_tmpfile(tempfile, sizeof(tempfile) - 1, TRUE, base_dir)) == -1) {
+	if ((dot_fd = my_tmpfile(tempfile, sizeof(tempfile) - 1, TRUE, base_dir)) == -1)
 		return rval;
-	}
 	snprintf(lockfile, sizeof(lockfile), "%s%s", filename, LOCK_SUFFIX);
 
 #ifdef HAVE_LINK
@@ -256,7 +255,7 @@ t_bool dot_lock(
 /*
  * try to remove a dotlock for filename
  *
- * returncodes:
+ * return codes:
  *  TRUE  = file unlocked successfully
  *  FALSE = some error occured
  */
