@@ -3,10 +3,10 @@
  *  Module    : mail.c
  *  Author    : I. Lea
  *  Created   : 1992-10-02
- *  Updated   : 2002-12-04
+ *  Updated   : 2005-08-14
  *  Notes     : Mail handling routines for creating pseudo newsgroups
  *
- * Copyright (c) 1992-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,17 +45,42 @@
 /*
  * local prototypes
  */
+static FILE *open_newsgroups_fp(void);
 static void read_groups_descriptions(FILE *fp, FILE *fp_save);
 static void read_newsgroups_file(t_bool verb);
 #ifdef HAVE_MH_MAIL_HANDLING
+	static FILE *open_mail_active_fp(const char *mode);
+	static FILE *open_mailgroups_fp(void);
 	static void read_mailgroups_file(t_bool verb);
 #endif /* HAVE_MH_MAIL_HANDLING */
 
 
+#ifdef HAVE_MH_MAIL_HANDLING
+/*
+ * Open the mail active file locally
+ */
+static FILE *
+open_mail_active_fp(
+	const char *mode)
+{
+	return fopen(mail_active_file, mode);
+}
+
+
+/*
+ * Open mail groups description file locally
+ */
+static FILE *
+open_mailgroups_fp(
+	void)
+{
+	return fopen(mailgroups_file, "r");
+}
+
+
 /*
  * Load the mail active file into active[]
  */
-#ifdef HAVE_MH_MAIL_HANDLING
 void
 read_mail_active_file(
 	void)
@@ -63,7 +88,7 @@ read_mail_active_file(
 	FILE *fp;
 	char buf[LEN];
 	char my_spooldir[PATH_LEN];
-	long count = -1L;
+	char buf2[PATH_LEN];
 	long min, max;
 	struct t_group *ptr;
 
@@ -77,9 +102,10 @@ read_mail_active_file(
 		if (cmd_line)
 			my_fputc('\n', stderr);
 
-		error_message(_(txt_cannot_open), mail_active_file);
+		if (!created_rcdir) /* no error on first start */
+			error_message(_(txt_cannot_open), mail_active_file);
 		/*
-		 * FIXME - maybe do an autoscan of maildir, create & do a reopen ?
+		 * TODO: do an autoscan of maildir, create & reopen?
 		 */
 		write_mail_active_file();
 		return;
@@ -95,7 +121,8 @@ read_mail_active_file(
 		if ((ptr = group_find(buf)) != NULL) {
 			if (strcmp(ptr->spooldir, my_spooldir) != 0) {
 				free(ptr->spooldir);
-				ptr->spooldir = my_strdup(my_spooldir);
+				strfpath(my_spooldir, buf2, sizeof(buf2) - 1, ptr);
+				ptr->spooldir = my_strdup(buf2);
 			}
 			ptr->xmax = max;
 			ptr->xmin = min;
@@ -109,15 +136,14 @@ read_mail_active_file(
 			continue;
 
 		/*
-		 * Load group info. TODO - integrate with active_add()
+		 * Load group info. TODO: integrate with active_add()
 		 */
-		ptr->aliasedto = (char *) 0;
-		ptr->description = (char *) 0;
-		ptr->spooldir = my_strdup(my_spooldir);
+		strfpath(my_spooldir, buf2, sizeof(buf2) - 1, ptr);
+		ptr->spooldir = my_strdup(buf2);
+		group_get_art_info(ptr->spooldir, buf, GROUP_TYPE_MAIL, &ptr->count, &ptr->xmax, &ptr->xmin);
+		ptr->aliasedto = NULL;
+		ptr->description = NULL;
 		ptr->moderated = 'y';
-		ptr->count = count;
-		ptr->xmax = max;
-		ptr->xmin = min;
 		ptr->type = GROUP_TYPE_MAIL;
 		ptr->inrange = FALSE;
 		ptr->read_during_session = FALSE;
@@ -148,7 +174,7 @@ write_mail_active_file(
 	FILE *fp;
 	char *file_tmp;
 	char group_path[PATH_LEN];
-	register int i;
+	int i;
 	struct t_group *group;
 
 	if (no_write && file_size(mail_active_file) != -1L)
@@ -213,16 +239,46 @@ read_mailgroups_file(
 
 
 /*
- * read group descriptions for news (and mailgroups)
+ * If reading via NNTP the newsgroups file will be saved to ~/.tin/newsgroups
+ * so that any subsequent rereads on the active file will not have to waste
+ * net bandwidth and the local copy of the newsgroups file can be accessed.
  */
-void
-read_descriptions(
-	t_bool verb)
+static FILE *
+open_newsgroups_fp(
+	void)
 {
-#ifdef HAVE_MH_MAIL_HANDLING
-	read_mailgroups_file(verb);
-#endif /* HAVE_MH_MAIL_HANDLING */
-	read_newsgroups_file(verb);
+#ifdef NNTP_ABLE
+	FILE *result;
+
+	if (read_news_via_nntp && !read_saved_news) {
+		if (read_local_newsgroups_file) {
+			struct stat buf;
+
+			if (!stat(local_newsgroups_file, &buf)) {
+				if (buf.st_size > 0) {
+					if ((result = fopen(local_newsgroups_file, "r")) != NULL) {
+#	ifdef DEBUG
+						debug_nntp("open_newsgroups_fp", "Using local copy of newsgroups file");
+#	endif /* DEBUG */
+						return result;
+					}
+				} else
+					unlink(local_newsgroups_file);
+			}
+		}
+#	if 0 /* TODO: */
+		if (list_newsgroups_wildmat_supported && newsrc_active
+		    && !list_active && num_active < some_useful_limit) {
+			for_each_group(i) {
+				snprintf(buff, sizeof(buff), "LIST NEWSGROUPS %s", active[i].name);
+				nntp_command(buff, OK_LIST, NULL, 0);
+			}
+		} else
+#	endif /* 0 */
+		return (nntp_command("LIST NEWSGROUPS", OK_GROUPS, NULL, 0));
+	} else
+#endif /* NNTP_ABLE */
+		return fopen(newsgroups_file, "r");
 }
 
 
@@ -241,9 +297,12 @@ read_newsgroups_file(
 		if (!batch_mode && verb)
 			wait_message(0, _(txt_reading_newsgroups_file));
 
+		if (read_news_via_nntp && !no_write) {
+			struct stat buf;
 
-		if (read_news_via_nntp && !read_local_newsgroups_file && !no_write)
-			fp_save = fopen(local_newsgroups_file, "w" FOPEN_OPTS);
+			if (stat(local_newsgroups_file, &buf) || !read_local_newsgroups_file)
+				fp_save = fopen(local_newsgroups_file, "w" FOPEN_OPTS);
+		}
 
 		read_groups_descriptions(fp, fp_save);
 
@@ -260,6 +319,20 @@ read_newsgroups_file(
 }
 
 
+/*
+ * read group descriptions for news (and mailgroups)
+ */
+void
+read_descriptions(
+	t_bool verb)
+{
+#ifdef HAVE_MH_MAIL_HANDLING
+	read_mailgroups_file(verb);
+#endif /* HAVE_MH_MAIL_HANDLING */
+	read_newsgroups_file(verb);
+}
+
+
 /*
  * Read groups descriptions from opened file & make local backup copy
  * of all groups that don't have a 'x' in the active file moderated
@@ -273,7 +346,7 @@ read_groups_descriptions(
 	FILE *fp_save)
 {
 	char *p, *q, *ptr;
-	char *groupname = (char *) 0;
+	char *groupname = NULL;
 	int count = 0;
 	size_t space = 0;
 	struct t_group *group;
@@ -289,7 +362,7 @@ read_groups_descriptions(
 		 * newsgroups file to only subscribed-to groups when tin is called
 		 * with the "-q" option.
 		 */
-		if ((fp_save != NULL) && read_news_via_nntp && !read_local_newsgroups_file)
+		if ((fp_save != NULL) && read_news_via_nntp)
 			fprintf(fp_save, "%s\n", ptr);
 
 		if (!space) { /* initial malloc */
@@ -313,30 +386,34 @@ read_groups_descriptions(
 		group = group_find(groupname);
 
 		if (group != NULL && group->description == NULL) {
+			char *r;
+			int r_len;
+
 			q = p;
 			while ((q = strchr(q, '\t')) != NULL)
 				*q = ' ';
 
-			group->description = convert_to_printable(my_strdup(p));
-
-#	if 0 /* not useful for cache_overview_files */
-			if (group->type == GROUP_TYPE_NEWS) {
-				if (fp_save != NULL && read_news_via_nntp && !read_local_newsgroups_file)
-					fprintf(fp_save, "%s\n", ptr);
-			}
-#	endif /* 0 */
+			r = my_strdup(p);
+			r_len = strlen(r);
+			/*
+			 * Protect against invalid character sequences.
+			 *
+			 * TODO: change US-ASCII to UTF-8 when NNTP draft becomes RFC
+			 */
+			process_charsets(&r, &r_len, "US-ASCII", tinrc.mm_local_charset, FALSE);
+			group->description = convert_to_printable(r);
 		}
 
 		if (++count % 100 == 0)
 			spin_cursor();
 	}
-	free(groupname);
+	FreeIfNeeded(groupname);
 }
 
 
 void
 print_active_head(
-	char *active_file)
+	const char *active_file)
 {
 	FILE *fp;
 
@@ -344,10 +421,7 @@ print_active_head(
 		return;
 
 	if ((fp = fopen(active_file, "w")) != NULL) {
-		/* FIXME: -> lang.c */
-		fprintf(fp, _("# [Mail/Save] active file. Format is like news active file:\n"));
-		fprintf(fp, _("#   groupname  max.artnum  min.artnum  /dir\n"));
-		fprintf(fp, _("# The 4th field is the basedir (ie. ~/Mail or ~/News)\n#\n"));
+		fprintf(fp, _(txt_mail_save_active_head));
 		fclose(fp);
 	}
 }
@@ -355,7 +429,7 @@ print_active_head(
 
 void
 find_art_max_min(
-	char *group_path,
+	const char *group_path,
 	long *art_max,
 	long *art_min)
 {
@@ -387,59 +461,37 @@ find_art_max_min(
 void
 print_group_line(
 	FILE *fp,
-	char *group_name,
+	const char *group_name,
 	long art_max,
 	long art_min,
-	char *base_dir)
+	const char *base_dir)
 {
 	fprintf(fp, "%s %05ld %05ld %s\n",
 		group_name, art_max, art_min, base_dir);
 }
 
 
-/*
- * Given a base pathname & a newsgroup name build an absolute pathname.
- * base = /usr/spool/news
- * newsgroup = alt.sources
- * absolute path = /usr/spool/news/alt/sources
- */
 void
-make_base_group_path(
-	char *base_dir,
-	char *group_name,
-	char *group_path)
-{
-	char *ptr;
-
-	joinpath(group_path, base_dir, group_name);
-
-	ptr = group_path + strlen(base_dir);
-	while ((ptr = strchr(ptr, '.')) != NULL)
-		*ptr = '/';
-}
-
-
-void
-vGrpDelMailArt(
+grp_del_mail_art(
 	struct t_article *article)
 {
 
-	if (article->delete_it) {
-		art_mark_undeleted(article);
+	if (article->delete_it)
 		info_message(_(txt_art_undeleted));
-	} else {
-		art_mark_deleted(article);
+	else
 		info_message(_(txt_art_deleted));
-	}
+
+	article->delete_it = bool_not(article->delete_it);
 }
 
 
 void
-vGrpDelMailArts(
+grp_del_mail_arts(
 	struct t_group *group)
 {
 	char article_filename[PATH_LEN];
 	char group_path[PATH_LEN];
+	char artnum[LEN];
 	int i;
 	struct t_article *article;
 #if 0 /* see comment below */
@@ -455,8 +507,8 @@ vGrpDelMailArts(
 		for_each_art(i) {
 			article = &arts[i];
 			if (article->delete_it) {
-				/* TODO: use joinpath() instead */
-				snprintf(article_filename, sizeof(article_filename) - 1, "%s/%ld", group_path, article->artnum);
+				snprintf(artnum, sizeof(artnum), "%ld", article->artnum);
+				joinpath(article_filename, group_path, artnum);
 				unlink(article_filename);
 				article->thread = ART_EXPIRED;
 #if 0 /* see comment below */
@@ -468,7 +520,7 @@ vGrpDelMailArts(
 #if 0
 /*
  * current tin's build_references() is changed to free msgid and refs,
- * therefore we cannot call write_nov_file after it. I simply commented
+ * therefore we cannot call write_overview after it. I simply commented
  * out this codes, NovFile will update at next time.
  */
 /*
@@ -476,7 +528,7 @@ vGrpDelMailArts(
  * the active[] entry for the group and rewrite the mail.active file
  */
 		if (update_index_file)
-			write_nov_file(group);
+			write_overview(group);
 #endif /* 0 */
 	}
 }
@@ -489,6 +541,7 @@ art_edit(
 {
 	char article_filename[PATH_LEN];
 	char temp_filename[PATH_LEN];
+	char buf[PATH_LEN];
 
 	/*
 	 * Check if news / mail group
@@ -497,9 +550,9 @@ art_edit(
 		return FALSE;
 
 	make_base_group_path(group->spooldir, group->name, temp_filename);
-	/* TODO: use joinpath() instead */
-	snprintf(article_filename, sizeof(article_filename) - 1, "%s/%ld", temp_filename, article->artnum);
-	snprintf(temp_filename, sizeof(temp_filename) - 1, "%s%d.art", TMPDIR, (int) process_id);
+	snprintf(buf, sizeof(buf), "%ld", article->artnum);
+	joinpath(article_filename, temp_filename, buf);
+	snprintf(temp_filename, sizeof(temp_filename), "%s%d.art", TMPDIR, (int) process_id);
 
 	if (!backup_file(article_filename, temp_filename))
 		return FALSE;
