@@ -3,10 +3,10 @@
  *  Module    : makecfg.c
  *  Author    : Thomas E. Dickey
  *  Created   : 1997-08-23
- *  Updated   : 2001-11-10
- *  Notes     : #defines and structs for config.c
+ *  Updated   : 2005-06-21
+ *  Notes     : #defines and structs for options_menu.c
  *
- * Copyright (c) 1997-2003 Thomas E. Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2005 Thomas E. Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,6 +36,7 @@
 
 
 #define __BUILD__
+#define PROTO_H 1	/* don't include proto.h */
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -204,17 +205,7 @@ generate_tbl(
 {
 	static const char *const table_1[] =
 	{
-		 ""
-		,"enum option_enum {"
-		,0
-	};
-	static const char *const table_2[] =
-	{
-		"\tVERY_LAST_OPT };"
-		,""
-		,"#define LAST_OPT VERY_LAST_OPT - 1"
-		,"#define OPT_ARG_COLUMN	9"
-		,""
+		""
 		,"#define OPT_TITLE     0"
 		,"#define OPT_ON_OFF    1"
 		,"#define OPT_LIST      2"
@@ -225,7 +216,7 @@ generate_tbl(
 		,"struct t_option option_table[]={"
 		,0
 	};
-	static const char *const table_3[] =
+	static const char *const table_2[] =
 	{
 		"};"
 		,0
@@ -235,28 +226,8 @@ generate_tbl(
 	MYDATA *p;
 	char temp[BUFSIZ];
 
-	/* generate enumerated type */
-	write_it(ofp, table_1);
-	for (p = all_data; p != 0; p = p->link) {
-		if (p->name[0] == '#')
-			fprintf(ofp, "%s\n", p->name);
-		else {
-			char *s = p->name;
-
-			fprintf(ofp, "\tOPT_");
-			while (*s != '\0') {
-				fprintf(ofp, "%c",
-					 isalpha ((unsigned char)*s) && islower((unsigned char)*s)
-					 ? toupper ((unsigned char)*s)
-					 : *s);
-				s++;
-			}
-			fprintf(ofp, ",\n");
-		}
-	}
-
 	/* generate the access table */
-	write_it(ofp, table_2);
+	write_it(ofp, table_1);
 	for (p = all_data; p != 0; p = p->link) {
 		if (p->name[0] == '#')
 			fprintf(ofp, "%s\n", p->name);
@@ -290,7 +261,46 @@ generate_tbl(
 		}
 	}
 
-	write_it(ofp, table_3);
+	write_it(ofp, table_2);
+}
+
+static void
+generate_enum(
+	FILE *ofp2)
+{
+	MYDATA *p;
+
+	static const char *const table_1[] = {
+		"enum option_enum {",
+		0
+	};
+	static const char *const table_2[] = {
+		"\tSIGNAL_HANDLER };",
+		"",
+		"#define LAST_OPT SIGNAL_HANDLER - 1",
+		0
+	};
+
+	/* generate enumerated type */
+	write_it(ofp2, table_1);
+	for (p = all_data; p != 0; p = p->link) {
+		if (p->name[0] == '#')
+			fprintf(ofp2, "%s\n", p->name);
+		else {
+			char *s = p->name;
+
+			fprintf(ofp2, "\tOPT_");
+			while (*s != '\0') {
+				fprintf(ofp2, "%c",
+					 isalpha ((unsigned char)*s) && islower((unsigned char)*s)
+					 ? toupper ((unsigned char)*s)
+					 : *s);
+				s++;
+			}
+			fprintf(ofp2, ",\n");
+		}
+	}
+	write_it(ofp2, table_2);
 }
 
 static void
@@ -377,7 +387,8 @@ generate_ptr(
 static void
 makecfg(
 	FILE *ifp,
-	FILE *ofp)
+	FILE *ofp,
+	FILE *ofp2)
 {
 	char buffer[BUFSIZ];
 	MYDATA *p, *q;
@@ -386,8 +397,8 @@ makecfg(
 	{
 		"/* This file is generated by MAKECFG */"
 		,""
-		,"#ifndef TINTBL_H"
-		,"#define TINTBL_H 1"
+		,"#ifndef TINCFG_H"
+		,"#define TINCFG_H 1"
 		,""
 		,"/* Macros for defining symbolic offsets that can be ifdef'd */"
 		,"#undef OINX"
@@ -426,9 +437,22 @@ makecfg(
 		,"#undef OEND"
 		,"#undef OTYP"
 		,""
-		,"#endif /* TINTBL_H */"
+		,"#endif /* TINCFG_H */"
 		,0
 	};
+	static const char *const table_3[] = {
+		"/* This file is generated by MAKECFG */",
+		"",
+		"#ifndef OPTIONS_MENU_H",
+		"#define OPTIONS_MENU_H 1",
+		"",
+		0
+	};
+	static const char *const table_4[] = {
+		"",
+		"#endif /* OPTIONS_MENU_H */",
+		0
+	};
 
 	/*
 	 * Process the input file.
@@ -473,6 +497,16 @@ makecfg(
 	write_it(ofp, table_2);
 
 	fclose(ofp);
+
+	if (ofp2) {
+		write_it(ofp2, table_3);
+
+		generate_enum(ofp2);
+
+		write_it(ofp2, table_4);
+
+		fclose(ofp2);
+	}
 }
 
 int
@@ -482,12 +516,13 @@ main(
 {
 	FILE *input = stdin;
 	FILE *output = stdout;
+	FILE *output2 = open_it("options_menu.h", "w");
 
 	if (argc > 1)
 		input = open_it(argv[1], "r");
 	if (argc > 2)
 		output = open_it(argv[2], "w");
-	makecfg(input, output);
+	makecfg(input, output, output2);
 
 	return (0);
 }
