@@ -3,10 +3,10 @@
  *  Module    : memory.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-05-10
+ *  Updated   : 2005-07-02
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,20 +38,18 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
-#ifndef RFC2046_H
-#	include "rfc2046.h"
-#endif /* !RFC2046_H */
+
 
 /*
- * Dynamic arrays maximum & current sizes
+ * Dynamic arrays maximum (initialized in init_alloc()) & current sizes
  * num_* values are one past top of used part of array
  */
-int max_active = 0;
+int max_active;
 int num_active = -1;
-int max_newnews = 0;
+int max_newnews;
 int num_newnews = 0;
-int max_art = 0;
-int max_save = 0;
+int max_art;
+int max_save;
 int num_save = 0;
 
 /*
@@ -87,7 +85,7 @@ init_alloc(
 	/*
 	 * active file arrays
 	 */
-	max_active = get_active_num();
+	max_active = DEFAULT_ACTIVE_NUM;
 	max_newnews = DEFAULT_NEWNEWS_NUM;
 
 	active = my_malloc(sizeof(*active) * max_active);
@@ -204,40 +202,40 @@ free_all_arrays(
 	free_active_arrays();
 	free_global_arrays();
 
-	if (!batch_mode) {
 #ifdef HAVE_COLOR
-		FreeIfNeeded(quote_regex.re);
-		FreeIfNeeded(quote_regex.extra);
-		FreeIfNeeded(quote_regex2.re);
-		FreeIfNeeded(quote_regex2.extra);
-		FreeIfNeeded(quote_regex3.re);
-		FreeIfNeeded(quote_regex3.extra);
+	FreeIfNeeded(quote_regex.re);
+	FreeIfNeeded(quote_regex.extra);
+	FreeIfNeeded(quote_regex2.re);
+	FreeIfNeeded(quote_regex2.extra);
+	FreeIfNeeded(quote_regex3.re);
+	FreeIfNeeded(quote_regex3.extra);
 #endif /* HAVE_COLOR */
-		FreeIfNeeded(slashes_regex.re);
-		FreeIfNeeded(slashes_regex.extra);
-		FreeIfNeeded(stars_regex.re);
-		FreeIfNeeded(stars_regex.extra);
-		FreeIfNeeded(strokes_regex.re);
-		FreeIfNeeded(strokes_regex.extra);
-		FreeIfNeeded(underscores_regex.re);
-		FreeIfNeeded(underscores_regex.extra);
-		FreeIfNeeded(strip_re_regex.re);
-		FreeIfNeeded(strip_re_regex.extra);
-		FreeIfNeeded(strip_was_regex.re);
-		FreeIfNeeded(strip_was_regex.extra);
-		FreeIfNeeded(uubegin_regex.re);
-		FreeIfNeeded(uubegin_regex.extra);
-		FreeIfNeeded(uubody_regex.re);
-		FreeIfNeeded(uubody_regex.extra);
-		FreeIfNeeded(url_regex.re);
-		FreeIfNeeded(url_regex.extra);
-		FreeIfNeeded(mail_regex.re);
-		FreeIfNeeded(mail_regex.extra);
-		FreeIfNeeded(news_regex.re);
-		FreeIfNeeded(news_regex.extra);
-		FreeIfNeeded(shar_regex.re);
-		FreeIfNeeded(shar_regex.extra);
+	FreeIfNeeded(slashes_regex.re);
+	FreeIfNeeded(slashes_regex.extra);
+	FreeIfNeeded(stars_regex.re);
+	FreeIfNeeded(stars_regex.extra);
+	FreeIfNeeded(strokes_regex.re);
+	FreeIfNeeded(strokes_regex.extra);
+	FreeIfNeeded(underscores_regex.re);
+	FreeIfNeeded(underscores_regex.extra);
+	FreeIfNeeded(strip_re_regex.re);
+	FreeIfNeeded(strip_re_regex.extra);
+	FreeIfNeeded(strip_was_regex.re);
+	FreeIfNeeded(strip_was_regex.extra);
+	FreeIfNeeded(uubegin_regex.re);
+	FreeIfNeeded(uubegin_regex.extra);
+	FreeIfNeeded(uubody_regex.re);
+	FreeIfNeeded(uubody_regex.extra);
+	FreeIfNeeded(url_regex.re);
+	FreeIfNeeded(url_regex.extra);
+	FreeIfNeeded(mail_regex.re);
+	FreeIfNeeded(mail_regex.extra);
+	FreeIfNeeded(news_regex.re);
+	FreeIfNeeded(news_regex.extra);
+	FreeIfNeeded(shar_regex.re);
+	FreeIfNeeded(shar_regex.extra);
 
+	if (!batch_mode) {
 		free_keymaps();
 		free_input_history();
 	}
@@ -254,6 +252,8 @@ free_all_arrays(
 		FreeAndNull(newnews);
 	}
 
+	FreeAndNull(nntp_caps.implementation);
+
 	tin_fgets(NULL, FALSE);
 }
 
@@ -325,6 +325,8 @@ free_attributes_array(
 
 			FreeAndNull(group->attribute->followup_to);
 
+			FreeAndNull(group->attribute->fcc);
+
 			FreeAndNull(group->attribute->mailing_list);
 			FreeAndNull(group->attribute->x_headers);
 			FreeAndNull(group->attribute->x_body);
@@ -362,6 +364,8 @@ free_attributes_array(
 
 	FreeAndNull(glob_attributes.followup_to);
 
+	FreeAndNull(glob_attributes.fcc);
+
 	FreeAndNull(glob_attributes.mailing_list);
 	FreeAndNull(glob_attributes.x_headers);
 	FreeAndNull(glob_attributes.x_body);
@@ -476,7 +480,7 @@ my_malloc1(
 	void *p;
 
 #ifdef DEBUG
-	vDbgPrintMalloc(TRUE, file, line, size);
+	debug_print_malloc(TRUE, file, line, size);
 #endif /* DEBUG */
 
 	if ((p = malloc(size)) == NULL) {
@@ -500,7 +504,7 @@ my_calloc1(
 	void *p;
 
 #ifdef DEBUG
-	vDbgPrintMalloc(TRUE, file, line, nmemb * size);
+	debug_print_malloc(TRUE, file, line, nmemb * size);
 #endif /* DEBUG */
 
 	if ((p = calloc(nmemb, size)) == NULL) {
@@ -519,7 +523,7 @@ my_realloc1(
 	size_t size)
 {
 #ifdef DEBUG
-	vDbgPrintMalloc(FALSE, file, line, size);
+	debug_print_malloc(FALSE, file, line, size);
 #endif /* DEBUG */
 
 	p = ((!p) ? (calloc(1, size)) : realloc(p, size));
