@@ -3,10 +3,10 @@
  *  Module    : misc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-06-17
+ *  Updated   : 2005-08-14
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,9 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
+#ifndef VERSION_H
+#	include "version.h"
+#endif /* !VERSION_H */
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
@@ -47,9 +50,14 @@
 #ifndef TIN_POLICY_H
 #	include "policy.h"
 #endif /* !TIN_POLICY_H */
-#ifndef RFC2046_H
-#	include "rfc2046.h"
-#endif /* !RFC2046_H */
+
+#if defined(HAVE_IDNA_H) && !defined(_IDNA_H)
+#	include <idna.h>
+#endif /* HAVE_IDNA_H && !_IDNA_H */
+#if defined(HAVE_STRINGPREP_H) && !defined(_STRINGPREP_H)
+#	include <stringprep.h>
+#endif /* HAVE_STRINGPREP_H & !_STRINGPREP_H */
+
 
 /*
  * defines to control GNKSA-checks behavior:
@@ -72,12 +80,12 @@ static int gnksa_dequote_plainphrase(char *realname, char *decoded, int addrtype
 static int strfeditor(char *editor, int linenum, const char *filename, char *s, size_t maxsize, char *format);
 static void write_input_history_file(void);
 #ifdef CHARSET_CONVERSION
-	static char *utf8_valid(char *line);
 	static t_bool buffer_to_local(char **line, int *max_line_len, const char *network_charset, const char *local_charset);
 #endif /* CHARSET_CONVERSION */
-#if (defined(MIME_STRICT_CHARSET) && !defined(NO_LOCALE)) || defined(CHARSET_CONVERSION)
-	static void buffer_to_ascii(char *c);
-#endif /* (MIME_STRICT_CHARSET && !NO_LOCALE) || CHARSET_CONVERSION */
+#if 0 /* currently unused */
+	static t_bool stat_article(long art, const char *group_path);
+#endif /* 0 */
+
 
 /*
  * generate tmp-filename
@@ -188,7 +196,7 @@ copy_fp(
  * try to backup filename as backupname. on success backupname has the same
  * permissions as filename.
  *
- * returncodes:
+ * return codes:
  * TRUE  = backup complete or source file was missing
  * FALSE = backup failed
  */
@@ -277,7 +285,7 @@ copy_body(
 	 */
 	if (tinrc.quote_style & QUOTE_COMPRESS) {
 		if (strstr(prefix, "%s"))
-			sprintf(prefixbuf, prefix, initl);
+			snprintf(prefixbuf, sizeof(prefixbuf), prefix, initl);
 		else {
 			/* strip tailing space from quote-char for quoting quoted lines */
 			strcpy(prefixbuf, prefix);
@@ -285,7 +293,7 @@ copy_body(
 				prefixbuf[strlen(prefixbuf) - 1] = '\0';
 		}
 	} else
-		sprintf(prefixbuf, prefix, initl);
+		snprintf(prefixbuf, sizeof(prefixbuf), prefix, initl);
 
 	/*
 	 * if raw_data is true, the signature is exceptionally quoted, even if
@@ -319,7 +327,7 @@ copy_body(
 					retcode = fprintf(fp_op, "%s\n", ((tinrc.quote_style & QUOTE_EMPTY) ? prefixbuf : ""));
 		} else {		/* no initials in quote_string, just copy */
 			if ((buf[0] != '\n') || (tinrc.quote_style & QUOTE_EMPTY))
-				retcode = fprintf(fp_op, "%s%s", ((buf[0] == '>' || buf[0] == ' ') ? prefixbuf : prefix), buf);  /* use blank-stripped quote string if line is already quoted or beginns with a space */
+				retcode = fprintf(fp_op, "%s%s", ((buf[0] == '>' || buf[0] == ' ') ? prefixbuf : prefix), buf);	/* use blank-stripped quote string if line is already quoted or beginns with a space */
 			else
 				retcode = fprintf(fp_op, "\n");
 		}
@@ -462,13 +470,16 @@ void
 shell_escape(
 	void)
 {
-	char *p;
+	char *p, *tmp;
 	char shell[LEN];
 
-	sprintf(mesg, _(txt_shell_escape), tinrc.default_shell_command);
+	tmp = fmt_string(_(txt_shell_escape), tinrc.default_shell_command);
 
-	if (!prompt_string(mesg, shell, HIST_SHELL_COMMAND))
+	if (!prompt_string(tmp, shell, HIST_SHELL_COMMAND)) {
+		free(tmp);
 		return;
+	}
+	free(tmp);
 
 	for (p = shell; *p && isspace((int) *p); p++)
 		continue;
@@ -481,8 +492,9 @@ shell_escape(
 	}
 
 	ClearScreen();
-	sprintf(mesg, _(txt_shell_command), p);
-	center_line(0, TRUE, mesg);
+	tmp = fmt_string(_(txt_shell_command), p);
+	center_line(0, TRUE, tmp);
+	free(tmp);
 	MoveCursor(INDEX_TOP, 0);
 
 	(void) invoke_cmd(p);
@@ -539,7 +551,7 @@ tin_done(
 			group = &active[my_group[i]];
 			if (group->read_during_session) {
 				if (ask) {
-					if (prompt_yn(cLINES, _(txt_catchup_all_read_groups), FALSE) == 1) {
+					if (prompt_yn(_(txt_catchup_all_read_groups), FALSE) == 1) {
 						ask = FALSE;
 						tinrc.thread_articles = THREAD_NONE;	/* speeds up index loading */
 					} else
@@ -556,7 +568,8 @@ tin_done(
 	 * chance to try again
 	 */
 	if (!no_write) {
-		forever {
+		i = 3; /* max retries */
+		while (i--) {
 			if (((wrote_newsrc_lines = write_newsrc()) >= 0L) && (wrote_newsrc_lines >= read_newsrc_lines)) {
 				if (!batch_mode || verbose)
 					my_fputs(_(txt_newsrc_saved), stdout);
@@ -575,7 +588,7 @@ tin_done(
 			}
 
 			if (!batch_mode) {
-				if (!prompt_yn(cLINES, _(txt_newsrc_again), TRUE))
+				if (prompt_yn(_(txt_newsrc_again), TRUE) <= 0)
 					break;
 			}
 		}
@@ -654,7 +667,7 @@ my_mkdir(
 	char buf[LEN];
 	struct stat sb;
 
-	sprintf(buf, "mkdir %s", path); /* redirect stderr to /dev/null? use invoke_cmd()? */
+	snprintf(buf, sizeof(buf), "mkdir %s", path); /* redirect stderr to /dev/null? use invoke_cmd()? */
 	if (stat(path, &sb) == -1) {
 		system(buf);
 		return chmod(path, mode);
@@ -757,26 +770,6 @@ rename_file(
 #endif /* VMS */
 
 
-#ifdef M_AMIGA
-/*
- * AmigaOS now has links. Better not to use them as not everybody has new ROMS
- */
-void
-rename_file(
-	char *old_filename,
-	char *new_filename)
-{
-	char buf[1024];
-
-	unlink(new_filename);
-	if (rename(old_filename, new_filename) == EOF)
-		perror_message(_(txt_rename_error), old_filename, new_filename);
-
-	return;
-}
-#endif /* M_AMIGA */
-
-
 /*
  * Note that we exit screen/curses mode when invoking
  * external commands
@@ -813,16 +806,21 @@ invoke_cmd(
 		need_resize = cYes;		/* Flag a redraw */
 	}
 
-#ifdef VMS
-	success = (ret != 0);
+#ifdef IGNORE_SYSTEM_STATUS
+	return TRUE;
 #else
-	success = (ret == 0);
-#endif /* VMS */
+
+#	ifdef VMS
+		success = (ret != 0);
+#	else
+		success = (ret == 0);
+#	endif /* VMS */
 
 	if (!success || system_status != 0)
 		error_message(_(txt_command_failed), nam);
 
 	return success;
+#endif /* IGNORE_SYSTEM_STATUS */
 }
 
 
@@ -832,6 +830,10 @@ draw_percent_mark(
 	long max_num)
 {
 	char buf[32]; /* should be big enough */
+	int len;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wbuf;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	if (NOTESLINES <= 0)
 		return;
@@ -841,7 +843,14 @@ draw_percent_mark(
 
 	clear_message();
 	snprintf(buf, sizeof(buf), "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num);
-	MoveCursor(cLINES, (cCOLS - (int) strlen(buf)) - (1 + BLANK_PAGE_COLS));
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	if ((wbuf = char2wchar_t(buf)) != NULL) {
+		len = wcswidth(wbuf, wcslen(wbuf) + 1);
+		free(wbuf);
+	} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		len = (int) strlen(buf);
+	MoveCursor(cLINES, cCOLS - len - (1 + BLANK_PAGE_COLS));
 	StartInverse();
 	my_fputs(buf, stdout);
 	my_flush();
@@ -900,60 +909,48 @@ dir_name(
 
 /*
  * Return TRUE if new mail has arrived
+ *
+ * TODO: why not cache the mailbox_name?
  */
+#define MAILDIR_NEW	"/new"
 t_bool
 mail_check(
 	void)
 {
 	const char *mailbox_name;
 	struct stat buf;
-#ifdef M_AMIGA
-	static long mbox_size = 0;
-#endif /* M_AMIGA */
 
 	mailbox_name = get_val("MAIL", mailbox);
 
-#ifdef M_AMIGA
-	/*
-	 * Since AmigaDOS does not distinguish between atime and mtime
-	 * we have to find some other way to figure out if the mailbox
-	 * was modified (to bad that Iain removed the mail_setup() and
-	 * mail_check() scheme used prior to 1.30 260694 which worked also
-	 * on AmigaDOS). (R. Luebke 10.7.1994)
-	 */
+	if (mailbox_name != 0 && stat(mailbox_name, &buf) >= 0) {
+		if ((int) (buf.st_mode & S_IFMT) == (int) S_IFDIR) { /* maildir setup */
+			DIR *dirp;
+			char *maildir_box;
+			struct dirent *dp;
 
-	/* this is only a first try, but it seems to work :) */
-
-	if (mailbox_name != 0) {
-		if (stat(mailbox_name, &buf) >= 0) {
-			if (buf.st_size > 0) {
-				if (buf.st_size > mbox_size) {
-					mbox_size = buf.st_size;
+			maildir_box = my_malloc(strlen(mailbox_name) + strlen(MAILDIR_NEW) + 1);
+#ifdef VMS
+			joindir(maildir_box, mailbox_name, MAILDIR_NEW);
+#else
+			joinpath(maildir_box, mailbox_name, MAILDIR_NEW);
+#endif /* VMS */
+			if (!(dirp = opendir(maildir_box))) {
+				free(maildir_box);
+				return FALSE;
+			}
+			free(maildir_box);
+			while ((dp = readdir(dirp)) != NULL) {
+				if ((strcmp(dp->d_name, ".")) && (strcmp(dp->d_name, ".."))) {
+					closedir(dirp);
 					return TRUE;
-				} else
-					/*
-					 * at this point we have to calculate how much the
-					 * mailbox has to grow until we say "new mail"
-					 * Unfortunately, some MUAs write status information
-					 * back to the users mailbox. This is a size increase
-					 * and would result in "new mail" if we only look for some
-					 * size increase. The mbox_size calculation below works
-					 * for me for some time now (I use AmigaELM).
-					 * Probably there is a better method, if you know one
-					 * you are welcome... :-)
-					 * I think a constant offset is more accurate today,
-					 * 1k is the average size of mail-headers alone in each
-					 * message I receive. (obw)
-					 */
-					mbox_size = buf.st_size + 1024;
-			} else
-				mbox_size = 0;
+				}
+			}
+			closedir(dirp);
+		} else {
+			if (buf.st_atime < buf.st_mtime && buf.st_size > 0)
+				return TRUE;
 		}
 	}
-#else
-	if (mailbox_name != 0 && stat(mailbox_name, &buf) >= 0 && buf.st_atime < buf.st_mtime && buf.st_size > 0)
-		return TRUE;
-#endif /* M_AMIGA */
 	return FALSE;
 }
 
@@ -972,7 +969,7 @@ eat_re(
 	char *s,
 	t_bool eat_was)
 {
-	int data, slen;
+	int data;
 	int offsets[6];
 	int size_offsets = ARRAY_SIZE(offsets);
 
@@ -980,15 +977,13 @@ eat_re(
 		return "<No subject>"; /* also used in art.c:parse_headers() */
 
 	do {
-		slen = strlen(s);
-		data = pcre_exec(strip_re_regex.re, strip_re_regex.extra, s, slen, 0, 0, offsets, size_offsets);
+		data = pcre_exec(strip_re_regex.re, strip_re_regex.extra, s, strlen(s), 0, 0, offsets, size_offsets);
 		if (offsets[0] == 0)
 			s += offsets[1];
 	} while (data > 0);
 
 	if (eat_was) do {
-		slen = strlen(s);
-		data = pcre_exec(strip_was_regex.re, strip_was_regex.extra, s, slen, 0, 0, offsets, size_offsets);
+		data = pcre_exec(strip_was_regex.re, strip_was_regex.extra, s, strlen(s), 0, 0, offsets, size_offsets);
 		if (offsets[0] > 0)
 			s[offsets[0]] = '\0';
 	} while (data > 0);
@@ -1036,24 +1031,25 @@ get_author(
 	char *str,
 	size_t len)
 {
+	char *p = idna_decode(art->from);
 	int author;
 
-	author = ((thread && !show_subject) ? SHOW_FROM_BOTH : CURR_GROUP.attribute->show_author);
+	author = ((thread && !show_subject) ? SHOW_FROM_BOTH : curr_group->attribute->show_author);
 
 	switch (author) {
 		case SHOW_FROM_ADDR:
-			strncpy(str, art->from, len);
+			strncpy(str, p, len);
 			break;
 
 		case SHOW_FROM_NAME:
-			strncpy(str, (art->name ? art->name : art->from), len);
+			strncpy(str, (art->name ? art->name : p), len);
 			break;
 
 		case SHOW_FROM_BOTH:
 			if (art->name)
-				snprintf(str, len, "%s <%s>", art->name, art->from);
+				snprintf(str, len, "%s <%s>", art->name, p);
 			else
-				strncpy(str, art->from, len);
+				strncpy(str, p, len);
 			break;
 
 		case SHOW_FROM_NONE:
@@ -1062,6 +1058,7 @@ get_author(
 			break;
 	}
 
+	free(p);
 	*(str + len) = '\0';				/* NULL terminate */
 }
 
@@ -1481,9 +1478,7 @@ _strfpath(
 	char tbuf[PATH_LEN];
 	char *envptr;
 	int i;
-#ifndef M_AMIGA
 	struct passwd *pwd;
-#endif /* !M_AMIGA */
 	t_bool is_mailbox = FALSE;
 
 	if (str == NULL || format == NULL || maxsize == 0)
@@ -1516,7 +1511,6 @@ _strfpath(
 						break;
 
 					default:	/* some other users homedir */
-#ifndef M_AMIGA
 						i = 0;
 						while (*format && *format != '/')
 							tbuf[i++] = *format++;
@@ -1529,11 +1523,6 @@ _strfpath(
 							return 0;
 						} else
 							sprintf(tbuf, "%s/", pwd->pw_dir);
-#else
-						/* Amiga has no other users */
-						return 0;
-						/* NOTREACHED */
-#endif /* !M_AMIGA */
 						break;
 				}
 				if ((str = strfpath_cp(str, tbuf, endp)) == NULL)
@@ -1697,13 +1686,6 @@ strfpath(
 }
 
 
-enum quote_enum {
-	no_quote = 0,
-	dbl_quote,
-	sgl_quote
-};
-
-
 /*
  * TODO: Properly explain this
  */
@@ -1714,37 +1696,46 @@ escape_shell_meta(
 {
 	static char buf[PATH_LEN];
 	char *dest = buf;
+	int space = sizeof(buf) - 2;
 
 	switch (quote_area) {
 		case no_quote:
-			while (*source) {
+			while (*source && (space > 0)) {
 				if (*source == '\'' || *source == '\\' || *source == '"' ||
 					*source == '$' || *source == '`' || *source == '*' ||
 					*source == '&' || *source == '|' || *source == '<' ||
 					*source == '>' || *source == ';' || *source == '(' ||
-					*source == ')')
+					*source == ')') {
 					*dest++ = '\\';
+					space--;
+				}
 				*dest++ = *source++;
+				space--;
 			}
 			break;
 
 		case dbl_quote:
-			while (*source) {
+			while (*source && (space > 0)) {
 				if (*source == '\\' || *source == '"' || *source == '$' ||
-					*source == '`')
+					*source == '`') {
 					*dest++ = '\\';
+					space--;
+				}
 				*dest++ = *source++;
+				space--;
 			}
 			break;
 
 		case sgl_quote:
-			while (*source) {
+			while (*source && (space > 4)) {
 				if (*source == '\'') {
 					*dest++ = '\'';
 					*dest++ = '\\';
 					*dest++ = '\'';
+					space -= 3;
 				}
 				*dest++ = *source++;
+				space--;
 			}
 			break;
 
@@ -1789,6 +1780,13 @@ strfmailer(
 	if (dest == NULL || format == NULL || maxsize == 0)
 		return 0;
 
+	/*
+	 * TODO: shouldn't we better check for no % OR format > maxsize?
+	 *       as no replacemnt doesn't make sense (hardcoded To, Subject
+	 *       and filename) and the resulting string usuly is longer after
+	 *       replacemnts were done (nobody uses enough %% to make the
+	 *       result shorter than the input).
+	 */
 	if (strchr(format, '%') == NULL && strlen(format) + 1 >= maxsize)
 		return 0;
 
@@ -1879,7 +1877,7 @@ strfmailer(
 
 				case 'S':	/* Subject */
 					/* don't MIME encode Subject if using external mail client */
-					if (tinrc.use_mailreader_i)
+					if (INTERACTIVE_NONE != tinrc.interactive_mailer)
 						strncpy(tbuf, escape_shell_meta(subject, quote_area), sizeof(tbuf) - 1);
 					else {
 #ifdef CHARSET_CONVERSION
@@ -1896,7 +1894,7 @@ strfmailer(
 
 				case 'T':	/* To */
 					/* don't MIME encode To if using external mail client */
-					if (tinrc.use_mailreader_i)
+					if (INTERACTIVE_NONE != tinrc.interactive_mailer)
 						strncpy(tbuf, escape_shell_meta(to, quote_area), sizeof(tbuf) - 1);
 					else {
 #ifdef CHARSET_CONVERSION
@@ -1913,7 +1911,7 @@ strfmailer(
 
 				case 'U':	/* User */
 					/* don't MIME encode User if using external mail client */
-					if (tinrc.use_mailreader_i)
+					if (INTERACTIVE_NONE != tinrc.interactive_mailer)
 						strncpy(tbuf, userid, sizeof(tbuf) - 1);
 					else {
 #ifdef CHARSET_CONVERSION
@@ -2055,15 +2053,12 @@ cleanup_tmp_files(
 #if 0
 	char acNovFile[PATH_LEN];
 
-	if (xover_cmd && !tinrc.cache_overview_files) {
-		sprintf(acNovFile, "%s%d.idx", TMPDIR, (int) process_id);
+	if (nntp_caps.over_cmd && !tinrc.cache_overview_files) {
+		snprintf(acNovFile, sizeof(acNovFile), "%s%d.idx", TMPDIR, (int) process_id);
 		unlink(acNovFile);
 	}
 #endif /* 0 */
 
-	if (!tinrc.cache_overview_files)
-		unlink(local_newsgroups_file);
-
 	if (batch_mode)
 		unlink(lock_file);
 }
@@ -2106,7 +2101,7 @@ file_size(
  * returns mtime
  * -1 in case of an error (file not found, or !S_IFREG)
  */
-long /* we use long (not time_t) here for FILE_CHANGED() macro */
+long /* we use long (not time_t) here */
 file_mtime(
 	const char *file)
 {
@@ -2389,15 +2384,13 @@ buffer_to_local(
 				char *tmpbuf, *tbuf;
 				ICONV_CONST char *cur_inbuf;
 				int used;
-				size_t inbytesleft;
-				size_t unknown_bytesleft;
+				size_t inbytesleft = 1;
+				size_t unknown_bytesleft = 4;
 				size_t tmpbytesleft, tsize;
 				size_t outbytesleft, osize;
 				size_t cur_obl, cur_ibl;
 				size_t result;
 
-				inbytesleft = 1;
-				unknown_bytesleft = 4;
 				unknown_ucs4 = unknown_buf;
 
 				/* convert '?' from ASCII to UCS-4 */
@@ -2471,7 +2464,7 @@ buffer_to_local(
 								 * outbuf was too small
 								 * As some input could be converted successfully
 								 * and we don`t know where the last complete char
-								 * ends, redo the last conversation completely.
+								 * ends, redo the last conversion completely.
 								 */
 								/* resize the output buffer */
 								obuf = my_realloc(obuf, osize * 2 + 1);
@@ -2573,18 +2566,20 @@ buffer_to_network(
 #endif /* CHARSET_CONVERSION */
 
 
-#if (defined(MIME_STRICT_CHARSET) && !defined(NO_LOCALE)) || defined(CHARSET_CONVERSION)
-static void
+char *
 buffer_to_ascii(
 	char *c)
 {
-	while (*c != '\0') { /* reduce to US-ASCII, other non-prints are filtered later */
+	char *a = c;
+
+	while (*c != '\0') {
+		/* reduce to US-ASCII, other non-prints are filtered later */
 		if ((unsigned char) *c >= 128)
 			*c = '?';
 		c++;
 	}
+	return a;
 }
-#endif /* (MIME_STRICT_CHARSET && !NO_LOCALE) || CHARSET_CONVERSION */
 
 
 /*
@@ -3423,7 +3418,15 @@ gnksa_split_from(
 		}
 	}
 
+	/*
+	 * if we allow <> as From: we must disallow <> as Mesage-ID,
+	 * see code in post.c:check_article_to_be_posted()
+	 */
+#if 0
+	if (!strchr(address, '@') && *address) /* check for From: without an @ but allow <> */
+#else
 	if (!strchr(address, '@')) /* check for From: without an @ */
+#endif /* 0 */
 		return GNKSA_ATSIGN_MISSING;
 
 	/* split successful */
@@ -3564,7 +3567,7 @@ strip_line(
 }
 
 
-#ifdef CHARSET_CONVERSION
+#if defined(CHARSET_CONVERSION) || (defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE))
 /*
  * 'check' a given UTF-8 strig and '?'-out illegal sequences
  * TODO: is this check complete?
@@ -3579,7 +3582,7 @@ strip_line(
  * UTF8-4          = %xF0 %x90-BF 2*UTF8-1 / %xF1-F3 3*UTF8-1 /
  *                   %xF4 %x80-8F 2*UTF8-1
  */
-static char *
+char *
 utf8_valid(
 	char *line)
 {
@@ -3604,88 +3607,95 @@ utf8_valid(
 			numc++;
 		} while ((d <<= 1) & 0x80);	/* get sequence length */
 
-		d = *c;
-		e = *(c + 1);
-
-		switch (numc) {
-			case 2:
-				/* out of range or sequences which would also fit into 1 byte */
-				if (d < 0xc2 || d > 0xdf)
-					illegal = TRUE;
-				break;
-
-			case 3:
-				f = *(c + 2);
-				/* out of range or sequences which would also fit into 2 bytes */
-				if (d < 0xe0 || d > 0xef || (d == 0xe0 && e < 0xa0))
-					illegal = TRUE;
-				/* U+D800 ... U+DFFF */
-				if (d == 0xed && e > 0x9f)
-					illegal = TRUE;
-				/* U+FDD0 ... U+FDEF */
-				if (d == 0xef && e == 0xb7 && (f >= 0x90 && f <= 0xaf))
-					illegal = TRUE;
-				/* U+FFFE, U+FFFF */
-				if (d == 0xef && e == 0xbf && (f == 0xbe || f == 0xbf))
-					illegal = TRUE;
-				break;
-
-			case 4:
-				f = *(c + 2);
-				g = *(c + 3);
-				/* out of range or sequences which would also fit into 3 bytes */
-				if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
-					illegal = TRUE;
-				/* largest current used sequence */
-				if ((d == 0xf4 && e > 0x8f) || d > 0xf4)
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
-				if (d == 0xf0 && (e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+4FFFE, U+4FFFF, U+5FFFE, U+5FFFF, U+6FFFE, U+6FFFF, U+7FFFE, U+7FFFF */
-				/* U+8FFFE, U+8FFFF, U+9FFFE, U+9FFFF, U+AFFFE, U+AFFFF, U+BFFFE, U+BFFFF */
-				/* U+CFFFE, U+CFFFF, U+DFFFE, U+DFFFF, U+EFFFE, U+EFFFF, U+FFFFE, U+FFFFF */
-				if ((d == 0xf1 || d == 0xf2 || d == 0xf3) && (e == 0x8f || e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+10FFFE, U+10FFFF */
-				if (d == 0xf4 && e == 0x8f && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				break;
-
-#	if 0	/* currently not used, see also check above */
-			case 5:
-				/* out of range or sequences which would also fit into 4 bytes */
-				if (d < 0xf8 || d > 0xfb || (d == 0xf8 && e < 0x88))
-					illegal = TRUE;
-				break;
-
-			case 6:
-				/* out of range or sequences which would also fit into 5 bytes */
-				if (d < 0xfc || d > 0xfd || (d == 0xfc && e < 0x84))
-					illegal = TRUE;
-				break;
-#	endif /* 0 */
-
-			default:
-				/*
-				 * with the check for plain US-ASCII above all other sequence
-				 * length are illegal.
-				 */
+		d = 1;
+		while (d < numc) {
+			if (*(c + d) == '\0' || *(c + d) == '\n')
 				illegal = TRUE;
-				break;
+			d--;
 		}
 
 		if (!illegal) {
-			for (d = 1; d < numc; d++) {
-				e = *(c + d);
-				if (e < 0x80 || e > 0xbf || *(c + d) == '\0' || *(c + d) == '\n')
+			d = *c;
+			e = *(c + 1);
+
+			switch (numc) {
+				case 2:
+					/* out of range or sequences which would also fit into 1 byte */
+					if (d < 0xc2 || d > 0xdf)
+						illegal = TRUE;
+					break;
+
+				case 3:
+					f = *(c + 2);
+					/* out of range or sequences which would also fit into 2 bytes */
+					if (d < 0xe0 || d > 0xef || (d == 0xe0 && e < 0xa0))
+						illegal = TRUE;
+					/* U+D800 ... U+DFFF */
+					if (d == 0xed && e > 0x9f)
+						illegal = TRUE;
+					/* U+FDD0 ... U+FDEF */
+					if (d == 0xef && e == 0xb7 && (f >= 0x90 && f <= 0xaf))
+						illegal = TRUE;
+					/* U+FFFE, U+FFFF */
+					if (d == 0xef && e == 0xbf && (f == 0xbe || f == 0xbf))
+						illegal = TRUE;
+					break;
+
+				case 4:
+					f = *(c + 2);
+					g = *(c + 3);
+					/* out of range or sequences which would also fit into 3 bytes */
+					if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
+						illegal = TRUE;
+					/* largest current used sequence */
+					if ((d == 0xf4 && e > 0x8f) || d > 0xf4)
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
+					if (d == 0xf0 && (e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+4FFFE, U+4FFFF, U+5FFFE, U+5FFFF, U+6FFFE, U+6FFFF, U+7FFFE, U+7FFFF */
+					/* U+8FFFE, U+8FFFF, U+9FFFE, U+9FFFF, U+AFFFE, U+AFFFF, U+BFFFE, U+BFFFF */
+					/* U+CFFFE, U+CFFFF, U+DFFFE, U+DFFFF, U+EFFFE, U+EFFFF, U+FFFFE, U+FFFFF */
+					if ((d == 0xf1 || d == 0xf2 || d == 0xf3) && (e == 0x8f || e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+10FFFE, U+10FFFF */
+					if (d == 0xf4 && e == 0x8f && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					break;
+
+#	if 0	/* currently not used, see also check above */
+				case 5:
+					/* out of range or sequences which would also fit into 4 bytes */
+					if (d < 0xf8 || d > 0xfb || (d == 0xf8 && e < 0x88))
+						illegal = TRUE;
+					break;
+
+				case 6:
+					/* out of range or sequences which would also fit into 5 bytes */
+					if (d < 0xfc || d > 0xfd || (d == 0xfc && e < 0x84))
+						illegal = TRUE;
+					break;
+#	endif /* 0 */
+
+				default:
+					/*
+					 * with the check for plain US-ASCII above all other sequence
+					 * length are illegal.
+					 */
 					illegal = TRUE;
+					break;
 			}
 		}
 
+		for (d = 1; d < numc && !illegal; d++) {
+			e = *(c + d);
+			if (e < 0x80 || e > 0xbf || e == '\0' || e == '\n')
+				illegal = TRUE;
+		}
+
 		if (!illegal)
 			c += numc; /* skip over valid sequence */
 		else {
@@ -3700,4 +3710,320 @@ utf8_valid(
 	}
 	return line;
 }
+#endif /* CHARSET_CONVERSION || (MULTIBYTE_ABLE && !NO_LOCALE) */
+
+
+char *idna_decode(
+	char *in)
+{
+	char *out = my_strdup(in);
+
+#if defined(HAVE_LIBIDN) && defined(HAVE_IDNA_TO_UNICODE_LZLZ)
+	if (stringprep_check_version("0.3.0")) {
+		char *q, *r = NULL;
+
+		if ((q = strrchr(out, '@')))
+			q++;
+		else
+			q = out;
+#	ifdef HAVE_IDNA_USE_STD3_ASCII_RULES
+		if (!idna_to_unicode_lzlz(q, &r, IDNA_USE_STD3_ASCII_RULES))
+#	else
+		if (!idna_to_unicode_lzlz(q, &r, 0))
+#	endif /* HAVE_IDNA_USE_STD3_ASCII_RULES */
+			strcpy(q, r);
+		FreeIfNeeded(r);
+	}
+#endif /* HAVE_LIBIDN && HAVE_IDNA_TO_UNICODE_LZLZ */
+
+	return out;
+}
+
+
+int
+tin_version_info(
+	FILE *fp)
+{
+	int wlines = 0;	/* written lines */
+
+#if defined(__DATE__) && defined(__TIME__)
+	fprintf(fp, _("Version: %s %s release %s (\"%s\") %s %s\n"),
+		PRODUCT, VERSION, RELEASEDATE, RELEASENAME, __DATE__, __TIME__);
+#else
+	fprintf(fp, _("Version: %s %s release %s (\"%s\")\n"),
+	       PRODUCT, VERSION, RELEASEDATE, RELEASENAME);
+#endif /* __DATE__ && __TIME__ */
+	wlines++;
+
+#ifdef SYSTEM_NAME
+	fprintf(fp, "Platform:\n");
+	fprintf(fp, "\tOS-Name  = \"%s\"\n", SYSTEM_NAME);
+	wlines += 2;
+#endif /* SYSTEM_NAME */
+
+#ifdef TIN_CC
+	fprintf(fp, "Compiler:\n");
+	fprintf(fp, "\tCC       = \"%s\"\n", TIN_CC);
+	wlines += 2;
+#	ifdef TIN_CFLAGS
+		fprintf(fp, "\tCFLAGS   = \"%s\"\n", TIN_CFLAGS);
+		wlines++;
+#	endif /* TIN_CFLAGS */
+#	ifdef TIN_CPP
+		fprintf(fp, "\tCPP      = \"%s\"\n", TIN_CPP);
+		wlines++;
+#	endif /* TIN_CPP */
+#	ifdef TIN_CPPFLAGS
+		fprintf(fp, "\tCPPFLAGS = \"%s\"\n", TIN_CPPFLAGS);
+		wlines++;
+#	endif /* TIN_CPPFLAGS */
+#endif /* TIN_CC */
+
+#ifdef TIN_LD
+	fprintf(fp, "Linker and Libraries:\n");
+	fprintf(fp, "\tLD       = \"%s\"\n", TIN_LD);
+	wlines += 2;
+#	ifdef TIN_LDFLAGS
+		fprintf(fp, "\tLDFLAGS  = \"%s\"\n", TIN_LDFLAGS);
+		wlines++;
+#	endif /* TIN_LDFLAGS */
+#	ifdef TIN_LIBS
+		fprintf(fp, "\tLIBS     = \"%s\"\n", TIN_LIBS);
+		wlines++;
+#	endif /* TIN_LIBS */
+#endif /* TIN_LD */
+	fprintf(fp, "\tPCRE     = \"%s\"\n", pcre_version());
+	wlines++;
+
+	fprintf(fp, "Characteristics:\n\t"
+/* TODO: complete list and do some useful grouping */
+#ifdef NNTP_ONLY
+			"+NNTP_ONLY "
+#else
+#	ifdef NNTP_ABLE
+			"+NNTP_ABLE "
+#	else
+			"-NNTP_ABLE "
+#	endif /* NNTP_ABLE */
+#endif /* NNTP_ONLY */
+#ifdef NO_POSTING
+			"+NO_POSTING "
+#else
+			"-NO_POSTING "
+#endif /* NO_POSTING */
+#ifdef BROKEN_LISTGROUP
+			"+BROKEN_LISTGROUP "
+#else
+			"-BROKEN_LISTGROUP "
+#endif /* BROKEN_LISTGROUP */
+#ifdef XHDR_XREF
+			"+XHDR_XREF"
+#else
+			"-XHDR_XREF"
+#endif /* XHDR_XREF */
+			"\n\t"
+#ifdef HAVE_FASCIST_NEWSADMIN
+			"+HAVE_FASCIST_NEWSADMIN "
+#else
+			"-HAVE_FASCIST_NEWSADMIN "
+#endif /* HAVE_FASCIST_NEWSADMIN */
+#ifdef ENABLE_IPV6
+			"+ENABLE_IPV6 "
+#else
+			"-ENABLE_IPV6 "
+#endif /* ENABLE_IPV6 */
+#ifdef HAVE_COREFILE
+			"+HAVE_COREFILE"
+#else
+			"-HAVE_COREFILE"
+#endif /* HAVE_COREFILE */
+			"\n\t"
+#ifdef NO_SHELL_ESCAPE
+			"+NO_SHELL_ESCAPE "
+#else
+			"-NO_SHELL_ESCAPE "
+#endif /* NO_SHELL_ESCAPE */
+#ifdef DISABLE_PRINTING
+			"+DISABLE_PRINTING "
+#else
+			"-DISABLE_PRINTING "
+#endif /* DISABLE_PRINTING */
+#ifdef DONT_HAVE_PIPING
+			"+DONT_HAVE_PIPING "
+#else
+			"-DONT_HAVE_PIPING "
+#endif /* DONT_HAVE_PIPING */
+#ifdef NO_ETIQUETTE
+			"+NO_ETIQUETTE"
+#else
+			"-NO_ETIQUETTE"
+#endif /* NO_ETIQUETTE */
+			"\n\t"
+#ifdef HAVE_LONG_FILE_NAMES
+			"+HAVE_LONG_FILE_NAMES "
+#else
+			"-HAVE_LONG_FILE_NAMES "
+#endif /* HAVE_LONG_FILE_NAMES */
+#ifdef APPEND_PID
+			"+APPEND_PID "
+#else
+			"-APPEND_PID "
+#endif /* APPEND_PID */
+#ifdef HAVE_MH_MAIL_HANDLING
+			"+HAVE_MH_MAIL_HANDLING"
+#else
+			"-HAVE_MH_MAIL_HANDLING"
+#endif /* HAVE_MH_MAIL_HANDLING */
+			"\n\t"
+#ifdef HAVE_ISPELL
+			"+HAVE_ISPELL "
+#else
+			"-HAVE_ISPELL "
+#endif /* HAVE_ISPELL */
+#ifdef HAVE_METAMAIL
+			"+HAVE_METAMAIL "
+#else
+			"-HAVE_METAMAIL "
+#endif /* HAVE_METAMAIL */
+#ifdef HAVE_SUM
+			"+HAVE_SUM"
+#else
+			"-HAVE_SUM"
+#endif /* HAVE_SUM */
+			"\n\t"
+#ifdef HAVE_COLOR
+			"+HAVE_COLOR "
+#else
+			"-HAVE_COLOR "
+#endif /* HAVE_COLOR */
+#ifdef HAVE_PGP
+			"+HAVE_PGP "
+#else
+			"-HAVE_PGP "
+#endif /* HAVE_PGP */
+#ifdef HAVE_PGPK
+			"+HAVE_PGPK "
+#else
+			"-HAVE_PGPK "
+#endif /* HAVE_PGPK */
+#ifdef HAVE_GPG
+			"+HAVE_GPG"
+#else
+			"-HAVE_GPG"
+#endif /* HAVE_GPG */
+			"\n\t"
+#ifdef MIME_BREAK_LONG_LINES
+			"+MIME_BREAK_LONG_LINES "
+#else
+			"-MIME_BREAK_LONG_LINES "
+#endif /* MIME_BREAK_LONG_LINES */
+#ifdef MIME_STRICT_CHARSET
+			"+MIME_STRICT_CHARSET "
+#else
+			"-MIME_STRICT_CHARSET "
+#endif /* MIME_STRICT_CHARSET */
+#ifdef CHARSET_CONVERSION
+			"+CHARSET_CONVERSION"
+#else
+			"-CHARSET_CONVERSION"
 #endif /* CHARSET_CONVERSION */
+			"\n\t"
+#ifdef MULTIBYTE_ABLE
+			"+MULTIBYTE_ABLE "
+#else
+			"-MULTIBYTE_ABLE "
+#endif /* MULTIBYTE_ABLE */
+#ifdef NO_LOCALE
+			"+NO_LOCALE"
+#else
+			"-NO_LOCALE"
+#endif /* NO_LOCALE */
+			"\n\t"
+#ifdef USE_CANLOCK
+			"+USE_CANLOCK "
+#else
+			"-USE_CANLOCK "
+#endif /* USE_CANLOCK */
+#ifdef EVIL_INSIDE
+			"+EVIL_INSIDE "
+#else
+			"-EVIL_INSIDE "
+#endif /* EVIL_INSIDE */
+#ifdef FORGERY
+			"+FORGERY "
+#else
+			"-FORGERY "
+#endif /* FORGERY */
+#ifdef TINC_DNS
+			"+TINC_DNS "
+#else
+			"-TINC_DNS "
+#endif /* TINC_DNS */
+#ifdef ENFORCE_RFC1034
+			"+ENFORCE_RFC1034"
+#else
+			"-ENFORCE_RFC1034"
+#endif /* ENFORCE_RFC1034 */
+			"\n\t"
+#ifdef REQUIRE_BRACKETS_IN_DOMAIN_LITERAL
+			"+REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
+#else
+			"-REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
+#endif /* REQUIRE_BRACKETS_IN_DOMAIN_LITERAL */
+#ifdef FOLLOW_USEFOR_DRAFT
+			"+FOLLOW_USEFOR_DRAFT"
+#else
+			"-FOLLOW_USEFOR_DRAFT"
+#endif /* FOLLOW_USEFOR_DRAFT */
+			"\n");
+	wlines += 11;
+
+	fflush(fp);
+	return wlines;
+}
+
+
+void
+draw_mark_selected(
+	int i)
+{
+	MoveCursor(INDEX2LNUM(i), MARK_OFFSET);
+	StartInverse();	/* ToggleInverse() doesn't work correct with ncurses4.x */
+	my_fputc(tinrc.art_marked_selected, stdout);
+	EndInverse();	/* ToggleInverse() doesn't work correct with ncurses4.x */
+	return;
+}
+
+
+#if 0
+/*
+ * Stat a mail/news article to see if it still exists
+ */
+static t_bool
+stat_article(
+	long art,
+	const char *group_path)
+{
+	char buf[NNTP_STRLEN];
+	struct t_group currgrp;
+
+	currgrp = CURR_GROUP;
+
+#	ifdef NNTP_ABLE
+	if (read_news_via_nntp && currgrp.type == GROUP_TYPE_NEWS) {
+		snprintf(buf, sizeof(buf), "STAT %ld", art);
+		return (nntp_command(buf, OK_NOTEXT, NULL, 0) != NULL);
+	} else
+#	endif /* NNTP_ABLE */
+	{
+		struct stat sb;
+
+		joinpath(buf, currgrp.spooldir, group_path);
+		snprintf(&buf[strlen(buf)], sizeof(buf), "/%ld", art);
+
+		return (stat(buf, &sb) != -1);
+	}
+}
+#endif /* 0 */
+
+
