@@ -3,10 +3,10 @@
  *  Module    : newsrc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-06-29
+ *  Updated   : 2005-05-04
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,21 +43,24 @@
 #endif /* !TCURSES_H */
 #ifndef TNNTP_H
 #	include "tnntp.h"
-#endif /* TNNTP_H */
-
-#define BITS_TO_BYTES(n)	((size_t) ((n + NBITS - 1) / NBITS))
+#endif /* !TNNTP_H */
+#ifndef NEWSRC_H
+#	include "newsrc.h"
+#endif /* !NEWSRC_H */
 
 static mode_t newsrc_mode = 0;
 
 /*
  * Local prototypes
  */
+static FILE *open_subscription_fp(void);
 static char *parse_newsrc_line(char *line, int *sub);
 static char *parse_subseq(struct t_group *group, char *seq, long *low, long *high, int *sum);
 static char *parse_get_seq(char *seq, long *low, long *high);
 static int write_newsrc_line(FILE *fp, char *line);
 static t_bool create_newsrc(char *newsrc_file);
 static void auto_subscribe_groups(char *newsrc_file);
+static void get_subscribe_info(struct t_group *grp);
 static void parse_bitmap_seq(struct t_group *group, char *seq);
 static void print_bitmap_seq(FILE *fp, struct t_group *group);
 
@@ -88,7 +91,7 @@ read_newsrc(
 		selmenu.max = skip_newgroups();
 
 	/*
-	 * make a .newsrc if one doesn't exist & auto subscribe to set groups
+	 * make a .newsrc if none exist & auto subscribe to set groups
 	 */
 	if (stat(newsrc_file, &statbuf) == -1) {
 		if (!create_newsrc(newsrc_file))
@@ -268,6 +271,28 @@ create_newsrc(
 }
 
 
+/*
+ * Get a list of default groups to subscribe to
+ */
+static FILE *
+open_subscription_fp(
+	void)
+{
+	if (!read_saved_news) {
+#ifdef NNTP_ABLE
+		if (read_news_via_nntp) {
+			/* if (nntp_caps.type = 2 && !nntp_caps.list_subscriptions)
+				return NULL;
+			else */
+				return (nntp_command("LIST SUBSCRIPTIONS", OK_GROUPS, NULL, 0));
+		} else
+#endif /* NNTP_ABLE */
+			return (fopen(subscriptions_file, "r"));
+	} else
+		return NULL;
+}
+
+
 /*
  * Automatically subscribe user to newsgroups specified in
  * NEWSLIBDIR/subscriptions (locally) or same file but from NNTP
@@ -331,7 +356,7 @@ backup_newsrc(
 	else
 #endif /* NNTP_ABLE */
 	{
-		STRCPY(filebuf, nntp_server);
+		STRCPY(filebuf, quote_space_to_dash(nntp_server));
 	}
 	JOINPATH(dirbuf, rcdir, filebuf);
 	joinpath(filebuf, dirbuf, OLDNEWSRC_FILE);
@@ -347,6 +372,123 @@ backup_newsrc(
 }
 
 
+/*
+ * Find the total, max & min articles number for specified group
+ * Use nntp GROUP command or read local spool
+ * Return 0, or -error
+ */
+int
+group_get_art_info(
+	char *tin_spooldir,
+	char *groupname,
+	int grouptype,
+	long *art_count,
+	long *art_max,
+	long *art_min)
+{
+	DIR *dir;
+	DIR_BUF *direntry;
+	char buf[NNTP_STRLEN];
+	long artnum;
+
+	if (read_news_via_nntp && grouptype == GROUP_TYPE_NEWS) {
+#ifdef NNTP_ABLE
+		char line[NNTP_STRLEN];
+
+		snprintf(buf, sizeof(buf), "GROUP %s", groupname);
+#	ifdef DEBUG
+		debug_nntp("group_get_art_info", buf);
+#	endif /* DEBUG */
+		put_server(buf);
+
+		switch (get_respcode(line, sizeof(line))) {
+			case OK_GROUP:
+				if (sscanf(line, "%ld %ld %ld", art_count, art_min, art_max) != 3)
+					error_message(_(txt_error_invalid_response_to_group), line);
+				break;
+
+			case ERR_NOGROUP:
+				*art_count = 0;
+				*art_min = 1;
+				*art_max = 0;
+				return -ERR_NOGROUP;
+
+			case ERR_ACCESS:
+				error_message("%s%s", cCRLF, line);
+				tin_done(NNTP_ERROR_EXIT);
+				/* keep lint quiet: */
+				/* NOTREACHED */
+				break;
+
+			default:
+#	ifdef DEBUG
+				debug_nntp("NOT_OK", line);
+#	endif /* DEBUG */
+				return -1;
+		}
+#else
+		my_fprintf(stderr, _("Unreachable?\n"));
+		return 0;
+#endif /* NNTP_ABLE */
+	} else {
+		*art_count = 0;
+		*art_min = 1;
+		*art_max = 0;
+
+		make_base_group_path(tin_spooldir, groupname, buf);
+
+		if ((dir = opendir(buf)) != NULL) {
+			while ((direntry = readdir(dir)) != NULL) {
+				artnum = atol(direntry->d_name); /* should be '\0' terminated... */
+				if (artnum >= 1) {
+					if (artnum > *art_max) {
+						*art_max = artnum;
+						if (*art_min == 0)
+							*art_min = artnum;
+					} else if (artnum < *art_min)
+						*art_min = artnum;
+					(*art_count)++;
+				}
+			}
+			CLOSEDIR(dir);
+		} else
+			return -1;
+	}
+
+	return 0;
+}
+
+
+/*
+ * Get and fixup (if needed) the counters for a newly subscribed group
+ */
+static void
+get_subscribe_info(
+	struct t_group *grp)
+{
+	long oldmin = grp->xmin;
+	long oldmax = grp->xmax;
+
+	group_get_art_info(grp->spooldir, grp->name, grp->type, &grp->count, &grp->xmax, &grp->xmin);
+
+	if (grp->newsrc.num_unread > grp->count) {
+#ifdef DEBUG
+		my_printf(cCRLF "Unread WRONG %s unread=[%ld] count=[%ld]", grp->name, grp->newsrc.num_unread, grp->count);
+		my_flush();
+#endif /* DEBUG */
+		grp->newsrc.num_unread = grp->count;
+	}
+
+	if (grp->xmin != oldmin || grp->xmax != oldmax) {
+		expand_bitmap(grp, 0);
+#ifdef DEBUG
+		my_printf(cCRLF "Min/Max DIFF %s old=[%ld-%ld] new=[%ld-%ld]", grp->name, oldmin, oldmax, grp->xmin, grp->xmax);
+		my_flush();
+#endif /* DEBUG */
+	}
+}
+
+
 /*
  * Subscribe/unsubscribe to a group in .newsrc.
  * This involves rewriting the .newsrc with the new info
@@ -398,7 +540,7 @@ subscribe(
 			if (sub_state == SUBSCRIBED) {
 				fprintf(newfp, "%s%c ", group->name, sub_state);
 				if (get_info) {
-					vGet1GrpArtInfo(group);
+					get_subscribe_info(group);
 					print_bitmap_seq(newfp, group);
 				} else /* we are not allowed to issue NNTP cmds during AUTOSUBSCRIBE loop */
 					fprintf(newfp, "1\n");
@@ -538,10 +680,7 @@ grp_mark_unread(
 	if (group->xmin > 0)
 		group->newsrc.xmin = group->xmin;
 
-	bitlength = (group->newsrc.xmax - group->newsrc.xmin) + 1;
-
-	if (bitlength < 0)
-		bitlength = 0;
+	bitlength = MAX(0, group->newsrc.xmax - group->newsrc.xmin + 1);
 
 	if (bitlength > 0)
 		newbitmap = my_malloc(BITS_TO_BYTES(bitlength));
@@ -616,7 +755,8 @@ parse_bitmap_seq(
 #ifdef DEBUG_NEWSRC
 	{
 		char buf[NEWSRC_LINE];
-		sprintf(buf, "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 20), ptr);
+
+		snprintf(buf, sizeof(buf), "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 20), ptr);
 		debug_print_comment(buf);
 		debug_print_bitmap(group, NULL);
 	}
@@ -858,7 +998,7 @@ parse_unread_arts(
 		 * check for wrong article numbers in the overview
 		 *
 		 * TODO: check disabled as we currently catch the artnum > high_mark
-		 *       case in read_nov_file() where we might be able to
+		 *       case in read_overview() where we might be able to
 		 *       fix the broken artnum (via xref:-parsing). currently
 		 *       we just skip the art there.
 		 */
@@ -993,16 +1133,16 @@ pos_group_in_newsrc(
 
 #ifdef VMS
 	joinpath(buf, TMPDIR, "subrc");
-	sprintf(sub, "%s.%d", buf, (int) process_id);
+	snprintf(sub, sizeof(sub), "%s.%d", buf, (int) process_id);
 
 	joinpath(buf, TMPDIR, "unsubrc");
-	sprintf(unsub, "%s.%d", buf, (int) process_id);
+	snprintf(unsub, sizeof(unsub), "%s.%d", buf, (int) process_id);
 #else
 	joinpath(buf, TMPDIR, ".subrc");
-	sprintf(sub, "%s.%d", buf, (int) process_id);
+	snprintf(sub, sizeof(sub), "%s.%d", buf, (int) process_id);
 
 	joinpath(buf, TMPDIR, ".unsubrc");
-	sprintf(unsub, "%s.%d", buf, (int) process_id);
+	snprintf(unsub, sizeof(unsub), "%s.%d", buf, (int) process_id);
 #endif /* !VMS */
 
 	if ((fp_sub = fopen(sub, "w" FOPEN_OPTS)) == NULL)
@@ -1248,16 +1388,15 @@ expand_bitmap(
 		debug_print_comment("expand_bitmap: group->newsrc.xbitmap == NULL");
 #endif /* DEBUG_NEWSRC */
 	} else if (need_full_copy) {
-		t_bitmap *newbitmap;
-		newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
+		t_bitmap *newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
 
 		/* Copy over old bitmap */
 		/* TODO: change to use shift */
 		for (tmp = group->newsrc.xmin; tmp <= group->newsrc.xmax; tmp++) {
-			if (NTEST(group->newsrc.xbitmap, tmp - group->newsrc.xmin))
-				NSET1(newbitmap, tmp - first);
-			else
+			if (NTEST(group->newsrc.xbitmap, tmp - group->newsrc.xmin) == ART_READ)
 				NSET0(newbitmap, tmp - first);
+			else
+				NSET1(newbitmap, tmp - first);
 		}
 
 		/* Mark earlier articles as read, updating num_unread */
@@ -1344,71 +1483,54 @@ art_mark(
 	struct t_article *art,
 	int flag)
 {
-	if (art != NULL) {
-		switch (flag) {
-			case ART_READ:
-				if (group != NULL) {
-					if (art->artnum >= group->newsrc.xmin && art->artnum <= group->newsrc.xmax)
-						NSET0(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
+	if (art == NULL)
+		return;
+
+	switch (flag) {
+		case ART_READ:
+			if (group != NULL) {
+				if (art->artnum >= group->newsrc.xmin && art->artnum <= group->newsrc.xmax)
+					NSET0(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
+#ifdef DEBUG_NEWSRC
+				debug_print_bitmap(group, art);
+#endif /* DEBUG_NEWSRC */
+			}
+			if ((art->status == ART_UNREAD) || (art->status == ART_WILL_RETURN)) {
+				art_mark_xref_read(art);
+
+				if (group != NULL && group->newsrc.num_unread)
+					group->newsrc.num_unread--;
+
+				art->status = ART_READ;
+			}
+			break;
+
+		case ART_UNREAD:
+		case ART_WILL_RETURN:
+			if (art->status == ART_READ) {
+				if (group != NULL)
+					group->newsrc.num_unread++;
+
+				art->status = flag;
+			}
+			if (group != NULL) {
+				if (art->artnum < group->newsrc.xmin)
+					expand_bitmap(group, art->artnum);
+				else {
+					NSET1(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
 #ifdef DEBUG_NEWSRC
 					debug_print_bitmap(group, art);
 #endif /* DEBUG_NEWSRC */
 				}
-				if ((art->status == ART_UNREAD) || (art->status == ART_WILL_RETURN)) {
-					art_mark_xref_read(art);
+			}
+			break;
 
-					if (group != NULL && group->newsrc.num_unread)
-						group->newsrc.num_unread--;
-
-					art->status = ART_READ;
-				}
-				break;
-
-			case ART_UNREAD:
-			case ART_WILL_RETURN:
-				if (art->status == ART_READ) {
-					if (group != NULL)
-						group->newsrc.num_unread++;
-
-					art->status = flag;
-				}
-				if (group != NULL) {
-					if (art->artnum < group->newsrc.xmin)
-						expand_bitmap(group, art->artnum);
-					else {
-						NSET1(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
-#ifdef DEBUG_NEWSRC
-						debug_print_bitmap(group, art);
-#endif /* DEBUG_NEWSRC */
-					}
-				}
-				break;
-
-			default:
-				break;
-		}
+		default:
+			break;
 	}
 }
 
 
-void
-art_mark_deleted(
-	struct t_article *art)
-{
-	if (art != NULL)
-		art->delete_it = TRUE;
-}
-
-
-void
-art_mark_undeleted(
-	struct t_article *art)
-{
-	if (art != NULL)
-		art->delete_it = FALSE;
-}
-
-
 void
 set_default_bitmap(
 	struct t_group *group)
@@ -1432,22 +1554,18 @@ set_default_bitmap(
 
 /* TEST harness */
 #ifdef DEBUG_NEWSRC
-
-#	ifdef DEBUG_NEWSRC_FIXME	/* something's broken here */
-		static void set_bitmap_range_read(struct t_newsrc *newsrc, long beg, long end);
-		static void set_bitmap_range_unread(struct t_newsrc *newsrc, long beg, long end);
-#	endif /* DEBUG_NEWSRC_FIXME */
+#	if 0
+static void set_bitmap_range_read(struct t_newsrc *my_newsrc, long beg, long end);
+static void set_bitmap_range_unread(struct t_newsrc *my_newsrc, long beg, long end);
 
 void
-vNewsrcTestHarness(
+newsrc_test_harness(
 	void)
 {
+	FILE *fp = NULL;
 	char seq[20000];
-	char *temp_file = NULL;
-	FILE *fp;
 	int i;
 	int retry = 10; /* max. retrys */
-	int fd;
 	long rng_min, rng_max;
 	struct t_group group;
 
@@ -1470,7 +1588,7 @@ vNewsrcTestHarness(
 
 	strcpy(seq, get_val("TIN_SEQ", ""));
 
-	my_printf("\nENV Min=[%ld] Max=[%ld] Rng=[%ld-%ld] Count=[%ld] Seq=[%s]\n",
+	fprintf(stderr, "\nENV Min=[%ld] Max=[%ld] Rng=[%ld-%ld] Count=[%ld] Seq=[%s]\n",
 		group.xmin, group.xmax, rng_min, rng_max, group.count, seq);
 
 	for (i = 0; i < 3; i++) {
@@ -1485,74 +1603,65 @@ vNewsrcTestHarness(
 		while (retry) {
 		/* FIXME - this is secure now, but doesn't write any debug output */
 		/* (it didn't before too) */
-			if ((temp_file = my_tempnam("", "NEWSRC")) != NULL) {
-				if ((fd = open (temp_file, (O_CREAT|O_EXCL), (S_IRUSR|S_IWUSR))) != -1) {
-					if ((fp = fopen(temp_file, "w")) != NULL) {
-						my_printf("\n%d. PARSE Seq=[%s]\n", i + 1, seq);
-						parse_bitmap_seq(&group, seq);
-						debug_print_newsrc(&group.newsrc, stdout);
-						print_bitmap_seq(fp, &group);
-						my_printf("   PRINT Seq=[");
-						print_bitmap_seq(stdout, &group);
-						fclose(fp);
-					} else
-						retry--;
-					close(fd);
-					break;
-				} else
-					retry--;
+			if ((fp = tmpfile()) != NULL) {
+				fprintf(stderr, "\n%d. PARSE Seq=[%s]\n", i + 1, seq);
+				parse_bitmap_seq(&group, seq);
+				debug_print_newsrc(&group.newsrc, stderr);
+				print_bitmap_seq(fp, &group);
+				fprintf(stderr, "   PRINT Seq=");
+				print_bitmap_seq(stderr, &group);
+				rewind(fp);
+				break;
 			} else
 				retry--;
 		}
 
-		debug_print_newsrc(&group.newsrc, stdout);
+		debug_print_newsrc(&group.newsrc, stderr);
 
-		if (!retry)
+		if (!retry || !fp)
 			error_message(_(txt_cannot_create_uniq_name));
 		else {
-			fp = fopen(temp_file, "r");
 			fgets(seq, (int) sizeof(seq), fp);
 			seq[strlen(seq) - 1] = '\0';
 			fclose(fp);
 		}
 	}
-#	ifdef DEBUG_NEWSRC_FIXME	/* something's broken here */
 	set_bitmap_range_read(&group.newsrc, rng_min, rng_max);
-	debug_print_newsrc(&group.newsrc, stdout);
+	debug_print_newsrc(&group.newsrc, stderr);
 
 	set_bitmap_range_unread(&group.newsrc, rng_min, rng_max);
-	debug_print_newsrc(&group.newsrc, stdout);
+	debug_print_newsrc(&group.newsrc, stderr);
 
-	NSETBLK0(group.newsrc.xbitmap, group.newsrc.xbitlen);
-	debug_print_newsrc(&group.newsrc, stdout);
+	if (group.newsrc.xbitmap != NULL)
+		NSETBLK0(group.newsrc.xbitmap, group.newsrc.xbitlen);
+	debug_print_newsrc(&group.newsrc, stderr);
 
-	NSETBLK1(group.newsrc.xbitmap, group.newsrc.xbitlen);
-	debug_print_newsrc(&group.newsrc, stdout);
-#	endif /* DEBUG_NEWSRC_FIXME */
+	if (group.newsrc.xbitmap != NULL)
+		NSETBLK1(group.newsrc.xbitmap, group.newsrc.xbitlen);
+	debug_print_newsrc(&group.newsrc, stderr);
 	my_printf("\n");
 }
 
 
-#	ifdef DEBUG_NEWSRC_FIXME	/* something's broken here */
 static void
 set_bitmap_range_read(
-	struct t_newsrc *newsrc,
+	struct t_newsrc *my_newsrc,
 	long beg,
 	long end)
 {
 	long length, offset;
 
-	if (beg >= newsrc->xmin && end <= newsrc->xmax) {
-		offset = beg - newsrc->xmin;
-		length = end - newsrc->xmin;
+	if (beg >= my_newsrc->xmin && end <= my_newsrc->xmax) {
+		offset = beg - my_newsrc->xmin;
+		length = end - my_newsrc->xmin;
 
-my_printf("\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
-newsrc->xmin, newsrc->xmax, beg, end, offset, length);
+fprintf(stderr, "\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
+my_newsrc->xmin, my_newsrc->xmax, beg, end, offset, length);
 
 		if (beg == end) {
-			NSET0(newsrc->xbitmap, offset);
+			NSET0(my_newsrc->xbitmap, offset);
 		} else {
-			NSETRNG0(newsrc->xbitmap, offset, length);
+			NSETRNG0(my_newsrc->xbitmap, offset, length);
 		}
 	}
 }
@@ -1560,24 +1669,25 @@ newsrc->xmin, newsrc->xmax, beg, end, offset, length);
 
 static void
 set_bitmap_range_unread(
-	struct t_newsrc *newsrc,
+	struct t_newsrc *my_newsrc,
 	long beg,
 	long end)
 {
 	long length, offset;
 
-	if (beg >= newsrc->xmin && end <= newsrc->xmax) {
-		offset = beg - newsrc->xmin;
-		length = end - newsrc->xmin;
+	if (beg >= my_newsrc->xmin && end <= my_newsrc->xmax) {
+		offset = beg - my_newsrc->xmin;
+		length = end - my_newsrc->xmin;
 
-my_printf("\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n", newsrc->xmin, newsrc->xmax, beg, end, offset, length);
+fprintf(stderr, "\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
+my_newsrc->xmin, my_newsrc->xmax, beg, end, offset, length);
 
 		if (beg == end) {
-			NSET1(newsrc->xbitmap, offset);
+			NSET1(my_newsrc->xbitmap, offset);
 		} else {
-			NSETRNG1(newsrc->xbitmap, offset, length);
+			NSETRNG1(my_newsrc->xbitmap, offset, length);
 		}
 	}
 }
-#	endif /* DEBUG_NEWSRC_FIXME */
+#	endif /* 0 */
 #endif /* DEBUG_NEWSRC */
