@@ -3,10 +3,10 @@
  *  Module    : prompt.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-02-08
+ *  Updated   : 2005-07-02
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,9 +41,15 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
+
+
+static char *prompt_slk_message;	/* prompt message for prompt_slk_redraw */
+
+/*
+ * Local prototypes
+ */
+static int prompt_list(int row, int col, int var, constext *help_text, constext *prompt_text, constext *list[], int size);
+
 
 /*
  *  prompt_num
@@ -56,20 +62,18 @@ prompt_num(
 	const char *prompt)
 {
 	char *p;
+	char tmp[LEN];
 	int num;
 
 	clear_message();
-
-	sprintf(mesg, "%c", ch);
-
-	if ((p = tin_getline(prompt, TRUE, mesg, 0, FALSE, HIST_OTHER)) != NULL) {
-		strcpy(mesg, p);
-		num = atoi(mesg);
+	snprintf(tmp, sizeof(tmp), "%c", ch);
+	if ((p = tin_getline(prompt, TRUE, tmp, 0, FALSE, HIST_OTHER)) != NULL) {
+		STRCPY(tmp, p);
+		num = atoi(tmp);
 	} else
 		num = -1;
 
 	clear_message();
-
 	return num;
 }
 
@@ -106,16 +110,13 @@ prompt_default_string(
 	char *p;
 
 	clear_message();
-
 	if ((p = tin_getline(prompt, FALSE, default_prompt, buf_len, FALSE, which_hist)) == NULL) {
 		buf[0] = '\0';
 		clear_message();
 		return FALSE;
 	}
 	strcpy(buf, p);
-
 	clear_message();
-
 	return TRUE;
 }
 
@@ -139,50 +140,49 @@ prompt_menu_string(
 	 * would lead to a 'n' answer to the reconnect prompt
 	 */
 	fflush(stdin);
-
 	MoveCursor(line, 0);
-
 	if ((p = tin_getline(prompt, FALSE, var, 0, FALSE, HIST_OTHER)) == NULL)
 		return FALSE;
 
 	strcpy(var, p);
-
 	return TRUE;
 }
 
 
 /*
  * prompt_yn
- * prompt user for 'y'es or 'n'o decision. "prompt" will be displayed in line
- * "line" giving the default answer "default_answer".
- * TODO: 'line' is constant - can we remove it ?
+ * prompt user for 'y'es or 'n'o decision. "prompt" will be displayed in the
+ * last line giving the default answer "default_answer".
  * The function returns 1 if the user decided "yes", -1 if the user wanted
- * to escape, or 0 for any other key or decision.
+ * to escape, or 0 for any other decision.
  */
 int
 prompt_yn(
-	int line,
 	const char *prompt,
 	t_bool default_answer)
 {
 	char *keyprompt;
 	char keyno[MAXKEYLEN], keyyes[MAXKEYLEN];
-	int ch = 'y', prompt_ch = 'y';
+	char yes, no, prompt_ch;
+	int ch;
 	size_t maxlen;
-	t_bool yn_loop = TRUE;
+	t_function func;
 
 /*	fflush(stdin); */		/* Prevent finger trouble from making important decisions */
 
-	(void) printascii(keyyes, (default_answer ? toupper(map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)));
-	(void) printascii(keyno, (!default_answer ? toupper(map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)));
+	yes = func_to_key(PROMPT_YES, prompt_keys);
+	no = func_to_key(PROMPT_NO, prompt_keys);
+
+	printascii(keyyes, (default_answer ? toupper(yes) : yes));
+	printascii(keyno, (!default_answer ? toupper(no) : no));
 	maxlen = MAX(strlen(keyyes), strlen(keyno));
 
-	while (yn_loop) {
-		prompt_ch = map_to_local((default_answer ? iKeyPromptYes : iKeyPromptNo), &menukeymap.prompt_yn);
+	do {
+		prompt_ch = (default_answer ? yes : no);
 		keyprompt = (default_answer ? keyyes : keyno);
 
 		if (!cmd_line) {
-			MoveCursor(line, 0);
+			MoveCursor(cLINES, 0);
 			CleartoEOLN();
 		}
 		my_printf("%s (%s/%s) %-*s", prompt, keyyes, keyno, (int) maxlen, keyprompt);
@@ -190,33 +190,31 @@ prompt_yn(
 			cursoron();
 		my_flush();
 		if (!cmd_line)
-			MoveCursor(line, (int) strlen(prompt) + strlen(keyyes) + strlen(keyno) + 5);
+			MoveCursor(cLINES, (int) strlen(prompt) + strlen(keyyes) + strlen(keyno) + 5);
 
 		if (((ch = (char) ReadCh()) == '\n') || (ch == '\r'))
 			ch = prompt_ch;
 
-		yn_loop = FALSE; /* normal case: leave loop */
-
 		switch (ch) {
 			case ESC:	/* (ESC) common arrow keys */
 #	ifdef HAVE_KEY_PREFIX
 			case KEY_PREFIX:
 #	endif /* HAVE_KEY_PREFIX */
 				switch (get_arrow_key(ch)) {
-
 					case KEYMAP_UP:
 					case KEYMAP_DOWN:
 						default_answer = bool_not(default_answer);
-						yn_loop = TRUE; /* don't leave loop */
+						ch = '\0';	/* set to a not bindable key to not leave the loop yet */
 						break;
 
 					case KEYMAP_LEFT:
-						ch = iKeyAbort;
+						ch = ESC;
 						break;
 
 					case KEYMAP_RIGHT:
 						ch = prompt_ch;
 						break;
+
 					default:
 						break;
 				}
@@ -225,20 +223,14 @@ prompt_yn(
 			default:
 				break;
 		}
-	}
+		func = key_to_func(ch, prompt_keys);
+	} while (func == NOT_ASSIGNED);
 
 	if (!cmd_line) {
-		if (line == cLINES)
-			clear_message();
-		else {
-			MoveCursor(line, (int) strlen(prompt));
-			my_fputc(((ch == iKeyAbort) ? prompt_ch : ch), stdout);
-		}
-		cursoroff();
+		clear_message();
 		my_flush();
 	}
-
-	return (tolower((unsigned char) map_to_default(ch, &menukeymap.prompt_yn)) == tolower((unsigned char)iKeyPromptYes)) ? 1 : (ch == iKeyAbort) ? -1 : 0;
+	return (func == PROMPT_YES) ? 1 : (func == GLOBAL_ABORT) ? -1 : 0;
 }
 
 
@@ -250,7 +242,7 @@ prompt_yn(
  * ESC is used to abort any changes, RET saves changes.
  * The new value is returned.
  */
-int
+static int
 prompt_list(
 	int row,
 	int col,
@@ -261,9 +253,13 @@ prompt_list(
 	int size)
 {
 	int ch, var_orig;
-	int i;
+	int i, offset;
+	int change;
 	int adjust = (strcasecmp(_(list[0]), _(txt_default)) == 0);
 	size_t width = 0;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wbuf;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	var += adjust;
 	size += adjust;
@@ -278,15 +274,68 @@ prompt_list(
 	show_menu_help(help_text);
 	cursoron();
 
-	do {
-		MoveCursor(row, col + (int) strlen(_(prompt_text)));
-		if ((ch = (char) ReadCh()) == ' ') {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	if ((wbuf = char2wchar_t(_(prompt_text))) != NULL) {
+		if ((offset = wcswidth(wbuf, wcslen(wbuf) + 1)) == -1) /* something went wrong, use wcslen as fallback */
+			offset = (int) wcslen(wbuf);
+		free(wbuf);
+	} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		offset = (int) strlen(_(prompt_text));
 
+	do {
+		MoveCursor(row, col + offset);
+		ch = (char) ReadCh();
+
+		/*
+		 * change:
+		 *   1 = move to the next list element
+		 *   0 = do nothing
+		 *  -1 = move to the previous list element
+		 *
+		 *  if an arrow key was pressed change ch to an other value
+		 *  otherwise we will exit the while loop
+		 */
+		switch (ch) {
+			case ' ':
+				change = 1;
+				break;
+
+			case ESC:	/* (ESC) common arrow keys */
+#	ifdef HAVE_KEY_PREFIX
+			case KEY_PREFIX:
+#	endif /* HAVE_KEY_PREFIX */
+				switch (get_arrow_key(ch)) {
+					case KEYMAP_UP:
+						change = -1;
+						ch = ' ';
+						break;
+
+					case KEYMAP_DOWN:
+						change = 1;
+						ch = ' ';
+						break;
+
+					default:
+						change = 0;
+						break;
+				}
+				break;
+
+			default:
+				change = 0;
+				break;
+		}
+
+		if (change) {
 			/*
-			 * Increment list, looping around at the max
+			 * increment or decrement list, loop around at the limits
 			 */
-			++var;
-			var %= size;
+			var += change;
+			if (var < 0)
+				var = size - 1;
+			else
+				var %= size;
 
 			my_printf("%-*s", (int) width, _(list[var]));
 			my_flush();
@@ -295,32 +344,47 @@ prompt_list(
 
 	if (ch == ESC) {
 		var = var_orig;
-
 		my_printf("%-*s", (int) width, _(list[var]));
 		my_flush();
 	}
 
 	cursoroff();
-
 	return (var - adjust);
 }
 
 
 /*
- * Special case of prompt_list() Toggle between ON and OFF
+ * Special case of prompt_option_list() Toggle between ON and OFF
+ * The function returns TRUE, if the value was changed, FALSE otherwise.
  */
-void
-prompt_on_off(
-	int row,
-	int col,
-	t_bool *var,
-	constext *help_text,
-	constext *prompt_text)
+t_bool
+prompt_option_on_off(
+	enum option_enum option)
 {
-	t_bool ret;
+	char prompt[LEN];
+	t_bool *variable = OPT_ON_OFF_list[option_table[option].var_index];
+	t_bool old_value = *variable;
 
-	ret = prompt_list(row, col, (int) *var, help_text, _(prompt_text), txt_onoff, 2) ? TRUE : FALSE;
-	*var = (ret != 0);
+	fmt_option_prompt(prompt, sizeof(prompt), TRUE, option);
+	*variable = prompt_list(option_row(option), 0, *variable, option_table[option].txt->help, prompt, txt_onoff, 2) ? TRUE: FALSE;
+	return *variable != old_value;
+}
+
+
+/*
+ * The function returns TRUE, if the value was changed, FALSE otherwise.
+ */
+t_bool
+prompt_option_list(
+	enum option_enum option)
+{
+	char prompt[LEN];
+	int *variable = option_table[option].variable;
+	int old_value = *variable;
+
+	fmt_option_prompt(prompt, sizeof(prompt), TRUE, option);
+	*variable = prompt_list(option_row(option), 0, *variable, option_table[option].txt->help, prompt, option_table[option].opt_list, option_table[option].opt_count);
+	return *variable != old_value;
 }
 
 
@@ -334,10 +398,10 @@ prompt_on_off(
  */
 t_bool
 prompt_option_string(
-	int option) /* return value is always ignored */
+	enum option_enum option) /* return value is always ignored */
 {
-	char prompt[LEN];
 	char *variable = OPT_STRING_list[option_table[option].var_index];
+	char prompt[LEN];
 
 	show_menu_help(option_table[option].txt->help);
 	fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
@@ -355,7 +419,7 @@ prompt_option_string(
  */
 t_bool
 prompt_option_num(
-	int option) /* return value is always ignored */
+	enum option_enum option) /* return value is always ignored */
 {
 	char prompt[LEN];
 	char number[LEN];
@@ -365,17 +429,15 @@ prompt_option_num(
 	show_menu_help(option_table[option].txt->help);
 	MoveCursor(option_row(option), 0);
 	fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
-	sprintf(&number[0], "%d", *(option_table[option].variable));
+	snprintf(&number[0], sizeof(number), "%d", *(option_table[option].variable));
 
 	if ((p = tin_getline(prompt, 2, number, 0, FALSE, HIST_OTHER)) == NULL)
 		return FALSE;
 
-	strcpy(number, p);
+	STRCPY(number, p);
 	num = atoi(number);
 	*(option_table[option].variable) = num;
-
 	clear_message();
-
 	return TRUE;
 }
 
@@ -389,7 +451,7 @@ prompt_option_num(
  */
 t_bool
 prompt_option_char(
-	int option) /* return value is always ignored */
+	enum option_enum option) /* return value is always ignored */
 {
 	char prompt[LEN];
 	char input[2];
@@ -413,7 +475,6 @@ prompt_option_char(
 	} while (!*p);
 
 	*variable = p[0];
-
 	clear_message();
 	return TRUE;
 }
@@ -509,24 +570,34 @@ prompt_msgid(
 /*
  * Format a message such that it'll fit within the screen width
  * Useful for fitting long Subjects and newsgroup names into prompts
- * TODO: maybe add a '...' to the string to show it was truncated.
- *       See center_line()
+ * result will contain a pointer to the malloced memory containing the
+ * sized message
  */
 char *
 sized_message(
+	char **result,
 	const char *format,
 	const char *subject)
 {
-	/* The formatting info (%.*s) wastes 4 chars, but our prompt needs 1 char */
-	int have = cCOLS - strlen(format) + 4 - 1;
-	int want = strlen(subject);
+	char *buf;
+	int max_len;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wformat;
 
-	if (want > 0 && subject[want - 1] == '\n')
-		want--;
-	if (have > want)
-		have = want;
-	sprintf(mesg, format, have, subject);
-	return mesg;
+	if ((wformat = char2wchar_t(format)) != NULL) {
+		/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
+		max_len = cCOLS - wcswidth(wformat, wcslen(wformat) + 1) + 2 - 1;
+		free(wformat);
+	} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		max_len = cCOLS - strlen(format) + 2 - 1;	/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
+
+	buf = strunc(subject, max_len);
+
+	*result = fmt_string(format, buf);
+	free(buf);
+
+	return *result;
 }
 
 
@@ -534,35 +605,90 @@ sized_message(
  * Implement the Single-Letter-Key mini menus at the bottom of the screen
  * eg, Press a)ppend, o)verwrite, q)uit :
  */
-int
+t_function
 prompt_slk_response(
-	int ch_default,
-	const t_menukeys /* char */ *responses,
+	t_function default_func,
+	const struct keylist keys,
 	const char *fmt,
 	...)
 {
 	va_list ap;
 	char ch;
 	char buf[LEN];
+	t_function func;
 
 	va_start(ap, fmt);
-	vsnprintf(buf, sizeof(buf), fmt, ap);	/* We need to do this, else wait_message() will clobber us */
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 	va_end(ap);
 
-	ch_default = map_to_local(ch_default, responses);
-	do {
-		wait_message(0, "%s%c", buf, ch_default);
+	prompt_slk_message = my_malloc(strlen(buf) + 2);
+	snprintf(prompt_slk_message, strlen(buf) + 2, "%s%c", buf, func_to_key(default_func, keys));
 
-		/* Get the cursor _just_ right */
-		MoveCursor(cLINES, (int) strlen(buf));
+	input_context = cPromptSLK;
+
+	do {
+		prompt_slk_redraw();		/* draw the prompt */
 
 		if ((ch = ReadCh()) == '\r' || ch == '\n')
-			ch = ch_default;
+			func = default_func;
+		else
+			func = key_to_func(ch, keys);
 
-	} while (!strchr(responses->localkeys, ch));
+		/*
+		 * TODO: ignore special-keys which are represented as a
+		 *       multibyte ESC-seq to avoid interpreting them as 'ESC' only
+		 *       like it's done in the ugly code below.
+		 */
+#if 0
+		if (ch == ESC) {
+			switch (get_arrow_key(ch)) {
+				case KEYMAP_UP:
+				case KEYMAP_DOWN:
+				case KEYMAP_LEFT:
+				case KEYMAP_RIGHT:
+				case KEYMAP_PAGE_DOWN:
+				case KEYMAP_PAGE_UP:
+				case KEYMAP_HOME:
+				case KEYMAP_END:
+					ch = '\0';
+					break;
+
+				default:
+					break;
+			}
+		}
+#endif /* 0 */
+	} while (func == NOT_ASSIGNED);
+
+	input_context = cNone;
+	FreeAndNull(prompt_slk_message);
 
 	clear_message();
-	return map_to_default(ch, responses);
+	return func;
+}
+
+
+/* (Re)draws the prompt message for prompt_slk_response() */
+void
+prompt_slk_redraw(
+	void)
+{
+	int column;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wtmp;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+	wait_message(0, "%s", prompt_slk_message);
+
+	/* get the cursor _just_ right */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	if ((wtmp = char2wchar_t(prompt_slk_message)) != NULL) {
+		column = wcswidth(wtmp, wcslen(wtmp) + 1) - 1;
+		free(wtmp);
+	} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		column = (int) strlen(prompt_slk_message) - 1;
+	MoveCursor(cLINES, column);
 }
 
 
@@ -590,6 +716,7 @@ prompt_continue(
 #	endif /* HAVE_KEY_PREFIX */
 			(void) get_arrow_key(ch);
 			/* FALLTHROUGH */
+
 		default:
 			break;
 	}
