@@ -3,12 +3,12 @@
  *  Module    : refs.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 2003-07-22
+ *  Updated   : 2005-07-20
  *  Notes     : Cacheing of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2003 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1996-2005 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,6 +61,7 @@ static char *_get_references(struct t_msgid *refptr, int depth);
 static struct t_msgid *add_msgid(int key, const char *msgid, struct t_msgid *newparent);
 static struct t_msgid *find_next(struct t_msgid *ptr);
 static struct t_msgid *parse_references(char *r);
+static t_bool valid_msgid(const char *msgid);
 static unsigned int hash_msgid(const char *key);
 static void add_to_parent(struct t_msgid *ptr);
 static void build_thread(struct t_msgid *ptr);
@@ -97,10 +98,10 @@ static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
  *
  *    When threading on Refs, a much better view of the original thread
  *    can be built up using this data, and threading is much faster
- *	   because all the article relationships are automatically available
- *	   to us.
+ *    because all the article relationships are automatically available
+ *    to us.
  *
- *	  NB: We don't cache msgids from the filter file.
+ * NB: We don't cache msgids from the filter file.
  */
 
 /*
@@ -176,6 +177,36 @@ add_to_parent(
 }
 
 
+/*
+ * Checks if Message-ID has valid format
+ * Returns TRUE if it does, FALSE if it does not
+ *
+ * TODO: combine with post.c:damaged_id()?
+ */
+static t_bool
+valid_msgid(
+	const char *msgid)
+{
+	size_t mlen = 0;
+	t_bool at_present = 0;
+
+	if (!msgid || *msgid != '<')
+		return FALSE;
+
+	while (isascii((unsigned char) *msgid) && isgraph((unsigned char) *msgid) && !iscntrl((unsigned char) *msgid) && *msgid != '>') {
+		if (*msgid == '@')
+			at_present = TRUE;
+		mlen++;
+		msgid++;
+	}
+
+	if (!at_present || (*msgid != '>') || mlen <= 2 || *(msgid + 1))
+		return FALSE;
+
+	return TRUE;
+}
+
+
 /*
  * Adds or updates a message id in the cache.
  * We return a ptr to the msgid, whether located or newly created.
@@ -207,7 +238,7 @@ add_msgid(
 	struct t_msgid *newparent)
 {
 	struct t_msgid *ptr;
-	struct t_msgid *i = NULL;
+	struct t_msgid *i;
 	unsigned int h;
 
 	if (!msgid) {
@@ -248,7 +279,6 @@ add_msgid(
 		 *         Change parent from null -> not-null & update ptrs
 		 */
 		if (i->parent == NULL) {
-
 			/*
 			 * Detect & ignore circular reference paths by looking for the
 			 * new parent in this thread
@@ -367,11 +397,17 @@ parse_references(
 	 * By definition, the head of the thread has no parent
 	 */
 	parent = NULL;
+
+	if (!valid_msgid(ptr))
+		return NULL;
+
 	current = add_msgid(REF_REF, ptr, parent);
 
 	while ((ptr = strtok(NULL, REF_SEP)) != NULL) {
-		parent = current;
-		current = add_msgid(REF_REF, ptr, parent);
+		if (valid_msgid(ptr)) {
+			parent = current;
+			current = add_msgid(REF_REF, ptr, parent);
+		}
 	}
 
 	return current;
@@ -646,7 +682,7 @@ dump_msgid_threads(
  */
 #define SKIP_ART(ptr)	\
 	(ptr && (ptr->article == ART_UNAVAILABLE || \
-		(arts[ptr->article].thread != ART_NORMAL || \
+		(arts[ptr->article].thread != ART_UNTHREADED || \
 			(tinrc.kill_level == KILL_NOTHREAD && arts[ptr->article].killed))))
 
 static struct t_msgid *
@@ -695,7 +731,6 @@ find_next(
 		 * a sibling if we can find one
 		 */
 		if (ptr->child == NULL && ptr->sibling == NULL) {
-
 			while (ptr != NULL && ptr->sibling == NULL)
 				ptr = ptr->parent;
 
@@ -734,13 +769,10 @@ build_thread(
 	 * Keep working through the thread, updating the ptrs as we go
 	 */
 	while ((newptr = find_next(ptr)) != NULL) {
-
 		arts[newptr->article].prev = ptr->article;
 		arts[ptr->article].thread = newptr->article;
-
 		ptr = newptr;
 	}
-
 }
 
 
@@ -874,7 +906,7 @@ void
 build_references(
 	struct t_group *group)
 {
-	char *s, *t;
+	char *s;
 	int i;
 	struct t_article *art;
 	struct t_msgid *refs;
@@ -914,33 +946,27 @@ build_references(
 			 *
 			 * TODO: do this in a single pass
 			 */
-			while (((s = strrchr(art->refs, ' ')) != NULL) && (!strcmp(art->msgid, s + 1))) {
-				/*
-				 * Remove circular reference to current article
-				 */
-				DEBUG_PRINT((dbgfd, "removing circular reference to%s\n", s));
-				*s = '\0';
+			if ((s = strrchr(art->refs, '<')) != NULL) {
+				if (!strcmp(art->msgid, s)) {
+					/*
+					 * Remove circular reference to current article
+					 */
+					DEBUG_PRINT((dbgfd, "removing circular reference to: %s\n", s));
+					*s = '\0';
+				}
 			}
-			while (((t = strrchr(art->refs, '\t')) != NULL) && (!strcmp(art->msgid, t + 1))) {
-				/*
-				 * Remove circular reference to current article
-				 */
-				DEBUG_PRINT((dbgfd, "removing circular reference to%s\n", t));
-				*t = '\0';
-			}
-			if (t > s)
-				s = t;
-
 			if (s != NULL) {
-				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s + 1, NULL));
+				if (valid_msgid(art->msgid))
+					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s, NULL));
 				*s = '\0';
 			} else {
-				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
+				if (valid_msgid(art->msgid))
+					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
 				FreeAndNull(art->refs);
 			}
 		} else
-			art->refptr = add_msgid(MSGID_REF, art->msgid, NULL);
-
+			if (valid_msgid(art->msgid))
+				art->refptr = add_msgid(MSGID_REF, art->msgid, NULL);
 		FreeAndNull(art->msgid);	/* Now cached - discard this */
 	}
 
@@ -961,7 +987,7 @@ build_references(
 
 		refs = parse_references(art->refs);
 
-		if (art->refptr->parent)
+		if (art->refptr && art->refptr->parent && valid_msgid(art->refptr->parent->txt))
 			add_msgid(REF_REF, art->refptr->parent->txt, refs);
 
 		FreeAndNull(art->refs);
