@@ -3,10 +3,10 @@
  *  Module    : rfc2046.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-02-18
- *  Updated   : 2003-08-26
+ *  Updated   : 2005-07-02
  *  Notes     : RFC 2046 MIME article parsing
  *
- * Copyright (c) 2000-2003 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2005 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,9 +38,6 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
-#ifndef RFC2046_H
-#	include "rfc2046.h"
-#endif /* !RFC2046_H */
 
 
 /*
@@ -69,6 +66,7 @@ static void progress(int line_count);
  * Local variables
  */
 static int art_lines = 0;		/* lines in art on spool */
+static char *progress_mesg = NULL;	/* message progress() should display */
 
 #define PARAM_SEP	"; \n"
 /* default parameters for Content-Type */
@@ -82,8 +80,8 @@ static void
 progress(
 	int line_count)
 {
-	if (line_count && line_count % MODULO_COUNT_NUM == 0)
-		show_progress(mesg, line_count, art_lines);
+	if (progress_mesg != NULL && line_count && line_count % MODULO_COUNT_NUM == 0)
+		show_progress(progress_mesg, line_count, art_lines);
 }
 
 
@@ -195,7 +193,7 @@ get_token(
 	char *dest = my_strdup(source);
 	char *ptr = dest;
 
-	while (isascii(*ptr) && isprint(*ptr) && *ptr != ' ' && !strchr(ATTRIBUTE_DELIMS, *ptr))
+	while (isascii((int) *ptr) && isprint((int) *ptr) && *ptr != ' ' && !strchr(ATTRIBUTE_DELIMS, *ptr))
 		ptr++;
 	*ptr = '\0';
 
@@ -360,6 +358,13 @@ parse_content_type(
 
 	/* Look up major type */
 
+	/*
+	 * TODO: remove/ignore comments in the CT-header, currently
+	 *       we do not recognize
+	 *          Content-Type: (foo) text/plain; charset=us-ascii
+	 *       as "text/plain"
+	 */
+
 	/*
 	 * Unrecognised type, treat according to RFC
 	 */
@@ -610,17 +615,19 @@ parse_header(
 			int type;
 
 			if (gnksa_split_from(ptr, addr, name, &type) == GNKSA_OK) {
+				buffer_to_ascii(addr);
+
 				if (*name) {
 					if (type == GNKSA_ADDRTYPE_OLDSTYLE)
-						sprintf(ptr, "%s (%s)", addr, rfc1522_decode(name));
+						sprintf(ptr, "%s (%s)", addr, convert_to_printable(rfc1522_decode(name)));
 					else
-						sprintf(ptr, "%s <%s>", rfc1522_decode(name), addr);
+						sprintf(ptr, "%s <%s>", convert_to_printable(rfc1522_decode(name)), addr);
 				} else
 					sprintf(ptr, "%s", addr);
 			} else
-				return ptr;
+				return convert_to_printable(ptr);
 		} else
-			return (rfc1522_decode(ptr));
+			return (convert_to_printable(rfc1522_decode(ptr)));
 	}
 
 	return ptr;
@@ -896,6 +903,13 @@ parse_multipart_article(
 				if (*line == '\0') {		/* End of MIME headers */
 					state = M_BODY;
 					curr_part->offset = ftell(artinfo->raw);
+
+					if (curr_part->type == TYPE_MULTIPART) {	/* Complex multipart article */
+						int ret;
+
+						if ((ret = parse_multipart_article(infile, artinfo, curr_part, depth + 1, show_progress_meter)) != 0)
+							return ret;							/* User abort or EOF reached */
+					}
 					break;
 				}
 
@@ -906,15 +920,7 @@ parse_multipart_article(
 				unfold_header(line);
 				if ((ptr = parse_header(line, "Content-Type", FALSE, FALSE))) {
 					parse_content_type(ptr, curr_part);
-
-					if (curr_part->type == TYPE_MULTIPART) {	/* Complex multipart article */
-						int ret;
-
-						if ((ret = parse_multipart_article(infile, artinfo, curr_part, depth + 1, show_progress_meter)) != 0)
-							return ret;							/* User abort or EOF reached */
-						else
-							break;
-					}
+					break;
 				}
 				if ((ptr = parse_header(line, "Content-Transfer-Encoding", FALSE, FALSE))) {
 					curr_part->encoding = parse_content_encoding(ptr);
@@ -999,6 +1005,7 @@ dump_uue(
 	}
 }
 
+
 static void
 dump_art(
 	t_openartinfo *art)
@@ -1092,6 +1099,45 @@ error:
 }
 
 
+/*
+ * Open a mail/news article using NNTP ARTICLE command
+ * or directly off local spool
+ * Return:
+ *		A pointer to the open postprocessed file
+ *		NULL pointer if article open fails in some way
+ */
+FILE *
+open_art_fp(
+	struct t_group *group,
+	long art)
+{
+	FILE *art_fp;
+	char buf[NNTP_STRLEN];
+
+#ifdef NNTP_ABLE
+	if (read_news_via_nntp && group->type == GROUP_TYPE_NEWS) {
+		snprintf(buf, sizeof(buf), "ARTICLE %ld", art);
+		art_fp = nntp_command(buf, OK_ARTICLE, NULL, 0);
+	} else {
+#endif /* NNTP_ABLE */
+		char pbuf[PATH_LEN];
+		char fbuf[NAME_LEN + 1];
+		char group_path[PATH_LEN];
+
+		make_group_path(group->name, group_path);
+		joinpath(buf, group->spooldir, group_path);
+		snprintf(fbuf, sizeof(fbuf), "%ld", art);
+		joinpath(pbuf, buf, fbuf);
+
+		art_fp = fopen(pbuf, "r");
+#ifdef NNTP_ABLE
+	}
+#endif /* NNTP_ABLE */
+
+	return art_fp;
+}
+
+
 /*----------- art_open() and art_close() are the only interface ---------*/
 /*------------------------for accessing articles -------------------*/
 
@@ -1110,7 +1156,8 @@ art_open(
 	struct t_article *art,
 	struct t_group *group,
 	t_openartinfo *artinfo,
-	t_bool show_progress_meter)
+	t_bool show_progress_meter,
+	char *pmesg)
 {
 	FILE *fp;
 
@@ -1123,8 +1170,12 @@ art_open(
 	fprintf(stderr, "art_open(%p)\n", (void *) artinfo);
 #endif /* DEBUG_ART */
 
-	if (parse_rfc2045_article(fp, art->line_count, artinfo, show_progress_meter) != 0)
+	progress_mesg = pmesg;
+	if (parse_rfc2045_article(fp, art->line_count, artinfo, show_progress_meter) != 0) {
+		progress_mesg = NULL;
 		return ART_ABORT;
+	}
+	progress_mesg = NULL;
 
 	/*
 	 * TODO: compare art->msgid and artinfo->hdr.messageid and issue a
