@@ -3,10 +3,10 @@
  *  Module    : rfc2047.c
  *  Author    : Chris Blum <chris@resolution.de>
  *  Created   : 1995-09-01
- *  Updated   : 2003-04-08
+ *  Updated   : 2005-07-02
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2003 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2005 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,9 +38,7 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
-#ifndef RFC2046_H
-#	include "rfc2046.h"
-#endif /* !RFC2046_H */
+
 
 #define isreturn(c) ((c) == '\r' || ((c) == '\n'))
 
@@ -71,16 +69,32 @@ static unsigned char base64_rank[256];
 static int base64_rank_table_built;
 static t_bool quoteflag;
 
+/* fixed prefix and default part for tin-generated MIME boundaries */
+static const char MIME_BOUNDARY_PREFIX[] = "=_tin=_";
+static const char MIME_BOUNDARY_DEFAULT_PART[] = "====____====____====____";
+/* required size of a buffer containing a MIME boundary, including the final '\0' */
+enum {
+	MIME_BOUNDARY_SIZE = sizeof(MIME_BOUNDARY_PREFIX) + sizeof(MIME_BOUNDARY_DEFAULT_PART) - 1
+};
+
 /*
  * local prototypes
  */
+static FILE *compose_message_rfc822(FILE *articlefp, t_bool *is_8bit);
+static FILE *compose_multipart_mixed(FILE *textfp, FILE *articlefp);
 static int do_b_encode(char *w, char *b, int max_ewsize, t_bool isstruct_head);
 static int sizeofnextword(char *w);
 static int which_encoding(char *w);
+static t_bool contains_8bit_characters(FILE *fp);
 static t_bool contains_nonprintables(char *w, t_bool isstruct_head);
+static t_bool contains_string(FILE *fp, const char *str);
 static t_bool rfc1522_do_encode(char *what, char **where, const char *charset, t_bool break_long_line);
+static t_bool split_mail(const char *filename, FILE **headerfp, FILE **textfp);
 static unsigned hex2bin(int x);
 static void build_base64_rank_table(void);
+static void do_rfc15211522_encode(FILE *f, constext * mime_encoding, struct t_group *group, t_bool allow_8bit_header, t_bool ismail, t_bool contains_headers);
+static void generate_mime_boundary(char *boundary, FILE *f, FILE *g);
+static void generate_random_mime_boundary(char *boundary, size_t len);
 static void str2b64(const char *from, char *to);
 
 
@@ -133,6 +147,9 @@ mmdecode(
 		int x;
 		unsigned hi, lo;
 
+		if (!what || !where) /* should not happen with 'q'-encoding */
+			return -1;
+
 		while (*what != delimiter) {
 			if (*what != '=') {
 				if (!delimiter || *what != '_')
@@ -165,12 +182,14 @@ mmdecode(
 		static int bits = 0;
 		unsigned char x;
 
-		build_base64_rank_table();
 		if (!what || !where) {		/* flush */
 			pattern = 0;
 			bits = 0;
 			return 0;
 		}
+
+		build_base64_rank_table();
+
 		while (*what != delimiter) {
 			x = base64_rank[(int) (*what++)];
 			/* ignore everything not in the alphabet, including '=' */
@@ -258,7 +277,7 @@ rfc1522_decode(
 				c++;
 				encoding = tolower((unsigned char)*c);
 				if (encoding == 'b')
-					(void) mmdecode((char *) 0, 'b', 0, (char *) 0);	/* flush */
+					(void) mmdecode(NULL, 'b', 0, NULL);	/* flush */
 				c++;
 				if (*c == '?') {
 					c++;
@@ -539,7 +558,7 @@ rfc1522_do_encode(
 		if (contains_nonprintables(what, isstruct_head) || isbroken_within) {
 			if (encoding == 'Q') {
 				if (!quoting) {
-					sprintf(buf2, "=?%s?%c?", charset, encoding);
+					snprintf(buf2, sizeof(buf2), "=?%s?%c?", charset, encoding);
 					ewsize = mystrcat(&t, buf2);
 					if (break_long_line) {
 						if (word_cnt == 2) {
@@ -563,7 +582,7 @@ rfc1522_do_encode(
 					if (is_EIGHT_BIT(what) || !isalnum((int)(unsigned char) *what))
 #endif /* 0 */
 					{
-						sprintf(buf2, "=%2.2X", *EIGHT_BIT(what));
+						snprintf(buf2, sizeof(buf2), "=%2.2X", *EIGHT_BIT(what));
 						*t++ = buf2[0];
 						*t++ = buf2[1];
 						*t++ = buf2[2];
@@ -608,7 +627,7 @@ rfc1522_do_encode(
 							*t++ = '_';
 							ewsize++;
 						} else {
-							sprintf(buf2, "=%2.2X", *EIGHT_BIT(what));
+							snprintf(buf2, sizeof(buf2), "=%2.2X", *EIGHT_BIT(what));
 							*t++ = buf2[0];
 							*t++ = buf2[1];
 							*t++ = buf2[2];
@@ -626,7 +645,7 @@ rfc1522_do_encode(
 				 * contain_nonprintables used in outer if-clause
 				 */
 				while (*what && (!isbetween(*what, isstruct_head) || rightafter_ew)) {
-					sprintf(buf2, "=?%s?%c?", charset, encoding);
+					snprintf(buf2, sizeof(buf2), "=?%s?%c?", charset, encoding);
 					ewsize = mystrcat(&t, buf2);
 
 					if (word_cnt == 2)
@@ -745,15 +764,20 @@ rfc1522_encode(
 }
 
 
-void
-rfc15211522_encode(
-	const char *filename,
+/*
+ * Helper function doing the hard work for rfc15211522_encode().
+ * Code moved from rfc15211522_encode(), with some adjustments to work on a
+ * file specified by a FILE* instead of a filename.
+ */
+static void
+do_rfc15211522_encode(
+	FILE *f,
 	constext * mime_encoding,
 	struct t_group *group,
 	t_bool allow_8bit_header,
-	t_bool ismail)
+	t_bool ismail,
+	t_bool contains_headers)
 {
-	FILE *f;
 	FILE *g;
 	char *c;
 	char *header;
@@ -763,7 +787,7 @@ rfc15211522_encode(
 	BodyPtr body_encode;
 	int i;
 #ifdef CHARSET_CONVERSION
-	int mmnwcharset = 0;
+	int mmnwcharset;
 
 	if (group) /* Posting */
 		mmnwcharset = group->attribute->mm_network_charset;
@@ -773,14 +797,10 @@ rfc15211522_encode(
 
 	if ((g = tmpfile()) == NULL)
 		return;
-	if ((f = fopen(filename, "r")) == NULL) {
-		fclose(g);
-		return;
-	}
 
 	quoteflag = FALSE;
 
-	while ((header = tin_fgets(f, TRUE))) {
+	while (contains_headers && (header = tin_fgets(f, TRUE))) {
 #ifdef CHARSET_CONVERSION
 		buffer_to_network(header, mmnwcharset);
 #endif /* CHARSET_CONVERSION */
@@ -821,13 +841,12 @@ rfc15211522_encode(
 			}
 		}
 	}
-	fclose(f);
-	rewind(g);
 
-	if ((f = fopen(filename, "w")) == NULL) {
-		fclose(g);
-		return;
-	}
+	rewind(g);
+	rewind(f);
+#ifdef HAVE_FTRUNCATE
+	ftruncate(fileno(f), 0);
+#endif /* HAVE_FTRUNCATE */
 
 	/* copy header */
 	while (fgets(buffer, 2048, g) && !isreturn(buffer[0]))
@@ -855,7 +874,8 @@ rfc15211522_encode(
 		 * now add MIME headers as necessary
 		 */
 		if (mime_headers_needed) {
-			fprintf(f, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
+			if (contains_headers)
+				fprintf(f, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
 #ifdef CHARSET_CONVERSION
 			fprintf(f, "Content-Type: text/plain; charset=%s\n", txt_mime_charsets[mmnwcharset]);
 #else
@@ -893,5 +913,354 @@ rfc15211522_encode(
 	}
 
 	fclose(g);
-	fclose(f);
+}
+
+
+void
+rfc15211522_encode(
+	const char *filename,
+	constext * mime_encoding,
+	struct t_group *group,
+	t_bool allow_8bit_header,
+	t_bool ismail)
+{
+	FILE *fp;
+
+	if ((fp = fopen(filename, "r+")) == NULL)
+		return;
+
+	do_rfc15211522_encode(fp, mime_encoding, group, allow_8bit_header, ismail, TRUE);
+
+	fclose(fp);
+}
+
+
+/*
+ * Generate a MIME boundary being unique with high probability, consisting
+ * of len - 1 random characters.
+ * This function is used as a last resort if anything else failed to
+ * generate a truly unique boundary.
+ */
+static void
+generate_random_mime_boundary(
+	char *boundary,
+	size_t len)
+{
+	size_t i;
+
+	srand((unsigned int) time(NULL));
+	for (i = 0; i < len - 1; i++)
+		boundary[i] = base64_alphabet[rand() % sizeof(base64_alphabet)];
+	boundary[len - 1] = '\0';
+}
+
+
+/*
+ * Generate a unique MIME boundary.
+ * boundary must have enough space for at least MIME_BOUNDARY_SIZE characters.
+ */
+static void
+generate_mime_boundary(
+	char *boundary,
+	FILE *f,
+	FILE *g)
+{
+	const char nice_chars[] = { '-', '_', '=' };
+	const size_t prefix_len = sizeof(MIME_BOUNDARY_PREFIX) - 1;
+	char *s;
+	size_t i = 0;
+	t_bool unique = FALSE;
+
+	/*
+	 * Choose MIME boundary as follows:
+	 *   - Always start with MIME_BOUNDARY_PREFIX.
+	 *   - Append MIME_BOUNDARY_DEFAULT_PART.
+	 *   - If necessary, change it from right to left, choosing from a set of
+	 *     `nice_chars' characters.
+	 *   - After that, if it is still not unique, replace MIME_BOUNDARY_DEFAULT_PART
+	 *     with random characters and hope the best.
+	 */
+
+	strcpy(boundary, MIME_BOUNDARY_PREFIX);
+	strcat(boundary, MIME_BOUNDARY_DEFAULT_PART);
+
+	s = boundary + MIME_BOUNDARY_SIZE - 2; /* set s to last character before '\0' */
+	do {
+		/*
+		 * Scan for entire boundary in both f and g.
+		 * When found: modify and redo.
+		 */
+		if (contains_string(f, boundary) || contains_string(g, boundary)) {
+			*s = nice_chars[i];
+			if ((i = (i + 1) % sizeof(nice_chars)) == 0)
+				--s;
+		} else
+			unique = TRUE;
+	} while (!unique && s >= boundary + prefix_len);
+
+	if (!unique)
+		generate_random_mime_boundary(boundary + prefix_len, sizeof(MIME_BOUNDARY_DEFAULT_PART));
+}
+
+
+/*
+ * Split mail into header and (optionally) body.
+ *
+ * If textfp is not NULL, everything behind the header is stored in it.
+ * Whenever an error is encountered, all files are closed and FALSE is returned.
+ */
+static t_bool
+split_mail(
+	const char *filename,
+	FILE **headerfp,
+	FILE **textfp)
+{
+	FILE *fp;
+	char *line;
+
+	if ((fp = fopen(filename, "r")) == NULL)
+		return FALSE;
+
+	/* Header */
+	if ((*headerfp = tmpfile()) == NULL) {
+		fclose(fp);
+		return FALSE;
+	}
+
+	while ((line = tin_fgets(fp, TRUE))) {
+		if (*line == '\0')
+			break;
+		else
+			fprintf(*headerfp, "%s\n", line);
+	}
+
+	/* Body */
+	if (textfp != NULL) {
+		if ((*textfp = tmpfile()) == NULL) {
+			fclose(fp);
+			fclose(*headerfp);
+			return FALSE;
+		}
+
+		while ((line = tin_fgets(fp, FALSE)))
+			fprintf(*textfp, "%s\n", line);
+	}
+
+	fclose(fp);
+	return TRUE;
+}
+
+
+/*
+ * Compose a mail consisting of a sole text/plain MIME part.
+ */
+void
+compose_mail_text_plain(
+	const char *filename,
+	struct t_group *group)
+{
+	rfc15211522_encode(filename, txt_mime_encodings[tinrc.mail_mime_encoding], group, tinrc.mail_8bit_header, TRUE);
+}
+
+
+/*
+ * Compose a mail consisting of an optional text/plain and a message/rfc822
+ * part.
+ *
+ * At this point, the file denoted by `filename' contains some common headers
+ * and any text the user entered. The file `articlefp' contains the forwarded
+ * article in raw form.
+ */
+void
+compose_mail_mime_forwarded(
+	const char *filename,
+	FILE *articlefp,
+	t_bool include_text,
+	struct t_group *group)
+{
+	FILE *fp;
+	FILE *headerfp;
+	FILE *textfp = NULL;
+	FILE *entityfp;
+	char *line;
+	constext* encoding = txt_mime_encodings[tinrc.mail_mime_encoding];
+	t_bool allow_8bit_header = tinrc.mail_8bit_header;
+	t_bool _8bit;
+
+	/* Split mail into headers and text */
+	if (!split_mail(filename, &headerfp, include_text ? &textfp : NULL))
+		return;
+
+	/* Encode header and text */
+	rewind(headerfp);
+	do_rfc15211522_encode(headerfp, encoding, group, allow_8bit_header, TRUE, TRUE);
+
+	if (textfp) {
+		rewind(textfp);
+		do_rfc15211522_encode(textfp, encoding, group, allow_8bit_header, TRUE, FALSE);
+	}
+
+	/* Compose top-level MIME entity */
+	if (include_text)
+		entityfp = compose_multipart_mixed(textfp, articlefp);
+	else
+		entityfp = compose_message_rfc822(articlefp, &_8bit);
+
+	if (entityfp == NULL) {
+		fclose(headerfp);
+		if (textfp)
+			fclose(textfp);
+		return;
+	}
+
+	/* Put it all together */
+	if ((fp = fopen(filename, "w")) == NULL)
+		return;
+
+	rewind(headerfp);
+	while ((line = tin_fgets(headerfp, TRUE))) {
+		if (*line != '\0')
+			fprintf(fp, "%s\n", line);
+	}
+	fprintf(fp, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
+	rewind(entityfp);
+	copy_fp(entityfp, fp);
+
+	/* Clean up */
+	fclose(fp);
+	fclose(headerfp);
+	fclose(entityfp);
+	if (textfp)
+		fclose(textfp);
+}
+
+
+/*
+ * Compose a message/rfc822 MIME entity containing articlefp.
+ */
+static FILE *
+compose_message_rfc822(
+	FILE *articlefp,
+	t_bool *is_8bit)
+{
+	FILE *fp;
+
+	if ((fp = tmpfile()) == NULL)
+		return NULL;
+
+	*is_8bit = contains_8bit_characters(articlefp);
+
+	/* Header: CT, CD, CTE */
+	fprintf(fp, "Content-Type: message/rfc822\n");
+	fprintf(fp, "Content-Disposition: inline\n");
+	fprintf(fp, "Content-Transfer-Encoding: %s\n", *is_8bit ? txt_8bit : txt_7bit);
+	fputc('\n', fp);
+
+	/* Body: articlefp */
+	rewind(articlefp);
+	copy_fp(articlefp, fp);
+
+	return fp;
+}
+
+
+/*
+ * Compose a multipart/mixed MIME entity consisting of a text/plain and a
+ * message/rfc822 part.
+ */
+static FILE *
+compose_multipart_mixed(
+	FILE *textfp,
+	FILE *articlefp)
+{
+	FILE *fp;
+	FILE *messagefp;
+	char boundary[MIME_BOUNDARY_SIZE];
+	t_bool requires_8bit;
+
+	if ((fp = tmpfile()) == NULL)
+		return NULL;
+
+	/* First compose message/rfc822 part (needed for choosing the appropriate CTE) */
+	if ((messagefp = compose_message_rfc822(articlefp, &requires_8bit)) == NULL) {
+		fclose(fp);
+		return NULL;
+	}
+
+	requires_8bit = (requires_8bit || contains_8bit_characters(textfp));
+
+	/* Header: CT with multipart boundary, CTE */
+	generate_mime_boundary(boundary, textfp, articlefp);
+	fprintf(fp, "Content-Type: multipart/mixed; boundary=\"%s\"\n", boundary);
+	fprintf(fp, "Content-Transfer-Encoding: %s\n\n", requires_8bit ? txt_8bit : txt_7bit);
+
+	/*
+	 * preamble
+	 * TODO: -> lang.c
+	 */
+	fprintf(fp, _("This message has been composed in the 'multipart/mixed' MIME-format. If you\n\
+are reading this prefix, your mail reader probably has not yet been modified\n\
+to understand the new format, and some of what follows may look strange.\n\n"));
+
+	/*
+	 * Body: boundary+text, message/rfc822 part, closing boundary
+	 */
+	/* text */
+	fprintf(fp, "--%s\n", boundary);
+	rewind(textfp);
+	copy_fp(textfp, fp);
+	fputc('\n', fp);
+
+	/* message/rfc822 part */
+	fprintf(fp, "--%s\n", boundary);
+	rewind(messagefp);
+	copy_fp(messagefp, fp);
+	fputc('\n', fp);
+
+	/* closing boundary */
+	fprintf(fp, "--%s--\n", boundary);
+	/* TODO: insert an epilogue here? */
+
+	return fp;
+}
+
+
+/*
+ * Determines whether the file denoted by fp contains 8bit characters.
+ */
+static t_bool
+contains_8bit_characters(
+	FILE *fp)
+{
+	char *line;
+
+	rewind(fp);
+	while ((line = tin_fgets(fp, FALSE))) {
+		for ( ; *line != '\0'; line++) {
+			if (is_EIGHT_BIT(line))
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
+/*
+ * Determines whether any line of the file denoted by fp contains str.
+ */
+static t_bool
+contains_string(
+	FILE *fp,
+	const char *str)
+{
+	char *line;
+
+	rewind(fp);
+	while ((line = tin_fgets(fp, FALSE))) {
+		if (strstr(line, str))
+			return TRUE;
+	}
+
+	return FALSE;
 }
