@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-08-26
+ *  Updated   : 2005-07-02
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,12 +41,6 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
-#ifndef RFC2046_H
-/* #	include "rfc2046.h" */
-#endif /* !RFC2046_H */
 
 #ifdef HAVE_UUDEVIEW_H
 #	ifndef __UUDEVIEW_H__
@@ -100,12 +94,12 @@ check_start_save_any_news(
 	FILE *artfp, *savefp;
 	FILE *fp_log = (FILE *) 0;
 	char *line;
-	char buf[LEN], logfile[LEN];
+	char buf[LEN];
 	char group_path[PATH_LEN];
 	char path[PATH_LEN];
-	char savefile[PATH_LEN];
+	char logfile[PATH_LEN], savefile[PATH_LEN];
 	char subject[HEADER_LEN];
-	int group_count;
+	int group_count = 0;
 	int i, j;
 	int art_count, hot_count;
 	int saved_arts = 0;					/* Total # saved arts */
@@ -148,14 +142,19 @@ check_start_save_any_news(
 			break;
 	}
 
-	group_count = 0;
-
 	/*
 	 * For each group we subscribe to...
 	 */
 	for (i = 0; i < selmenu.max; i++) {
 		art_count = hot_count = 0;
 		group = &active[my_group[i]];
+		/*
+		 * FIXME: workaround to get a valid CURR_GROUP
+		 * it also points to the currently processed group so that
+		 * the correct attributes are used
+		 * The correct fix is to get rid of CURR_GROUP
+		 */
+		selmenu.curr = i;
 
 		if (group->bogus || !group->subscribed)
 			continue;
@@ -203,10 +202,8 @@ check_start_save_any_news(
 					art_count++;
 					if (arts[j].score >= tinrc.score_select)
 						hot_count++;
-#if 0 /* is "-cZ" usefull? */
 					if (catchup)
 						art_mark(group, &arts[j], ART_READ);
-#endif /* 0 */
 					break;
 
 				case START_ANY_NEWS:
@@ -231,7 +228,7 @@ check_start_save_any_news(
 						continue;
 					}
 
-					if (function == MAIL_ANY_NEWS) {
+					if ((function == MAIL_ANY_NEWS) && ((INTERACTIVE_NONE == tinrc.interactive_mailer) || (INTERACTIVE_WITH_HEADERS == tinrc.interactive_mailer))) {
 						fprintf(savefp, "To: %s\n", mail_news_user);
 						fprintf(savefp, "Subject: %s\n", arts[j].subject);
 						/*
@@ -288,7 +285,10 @@ check_start_save_any_news(
 
 	switch (function) {
 		case CHECK_ANY_NEWS:
-			if (unread_news)
+			/*
+			 * TODO: shall we return 2 or 0 in the -cZ case?
+			 */
+			if (unread_news && !catchup)
 				return 2;
 			else {
 				if (verbose)
@@ -340,8 +340,8 @@ open_save_filename(
 	FILE *fp;
 	char keyappend[MAXKEYLEN], keyoverwrite[MAXKEYLEN], keyquit[MAXKEYLEN];
 	char mode[3];
-	int ch;
 	struct stat st;
+	t_function func;
 
 	strcpy(mode, "a+");
 
@@ -358,27 +358,31 @@ open_save_filename(
 		}
 /* TODO: will this get called every art? Should only be done once/batch */
 /* TODO: or add an option for defaulting on all future queries */
-		ch = prompt_slk_response(tinrc.default_save_mode,
-				&menukeymap.save_append_overwrite_quit,
+/* TODO: 'truncate' path if query exceeds screen-width */
+		func = prompt_slk_response((tinrc.default_save_mode == 'a' ? SAVE_APPEND_FILE : SAVE_OVERWRITE_FILE),
+				save_append_overwrite_keys,
 				_(txt_append_overwrite_quit), path,
-				printascii(keyappend, map_to_local(iKeySaveAppendFile, &menukeymap.save_append_overwrite_quit)),
-				printascii(keyoverwrite, map_to_local(iKeySaveOverwriteFile, &menukeymap.save_append_overwrite_quit)),
-				printascii(keyquit, map_to_local(iKeyQuit, &menukeymap.save_append_overwrite_quit)));
+				printascii(keyappend, func_to_key(SAVE_APPEND_FILE, save_append_overwrite_keys)),
+				printascii(keyoverwrite, func_to_key(SAVE_OVERWRITE_FILE, save_append_overwrite_keys)),
+				printascii(keyquit, func_to_key(GLOBAL_QUIT, save_append_overwrite_keys)));
 
-		switch (ch) {
-			case iKeySaveOverwriteFile:
+		switch (func) {
+			case SAVE_OVERWRITE_FILE:
 				strcpy(mode, "w");
 				break;
 
-			case iKeyAbort:
-			case iKeyQuit:
+			case GLOBAL_ABORT:
+			case GLOBAL_QUIT:
 				wait_message(1, _(txt_art_not_saved));
 				return NULL;
 
-			default:	/* iKeySaveAppendFile */
+			default:	/* SAVE_APPEND_FILE */
 				break;
 		}
-		tinrc.default_save_mode = ch;
+		if (func == SAVE_OVERWRITE_FILE)
+			tinrc.default_save_mode = 'o';
+		else
+			tinrc.default_save_mode = 'a';
 	}
 
 	if ((fp = fopen(path, mode)) == NULL) {
@@ -438,7 +442,7 @@ save_and_process_art(
 	 * If using the auto-save feature on an article with Archive-Name,
 	 * the path will be: <original-path>/<archive-name>/<part|patch><part#>
 	 */
-	if (!is_mailbox && CURR_GROUP.attribute->auto_save && artptr->archive) {
+	if (!is_mailbox && curr_group->attribute->auto_save && artptr->archive) {
 		const char *partprefix;
 		char *ptr;
 		char archpath[PATH_LEN];
@@ -503,8 +507,7 @@ save_and_process_art(
 		 */
 	}
 
-	if (copy_fp(artinfo->raw, fp))
-		/* Write tailing newline or MMDF-mailbox seperator */
+	if (copy_fp(artinfo->raw, fp)) /* Write tailing newline or MMDF-mailbox seperator */
 		print_art_seperator_line(fp, is_mailbox);
 	else {
 		fclose(fp);
@@ -616,10 +619,7 @@ expand_save_filename(
 	char *outpath,
 	const char *path)
 {
-	int ret;
-	struct t_group *group = &CURR_GROUP;
-
-	ret = strfpath(path, outpath, PATH_LEN, group);
+	int ret = strfpath(path, outpath, PATH_LEN, curr_group);
 
 	/*
 	 * If no path exists or the above failed in some way, use sensible defaults
@@ -628,7 +628,7 @@ expand_save_filename(
 	if ((ret == 0) || !(strrchr(outpath, DIRSEP))) {
 		char buf[PATH_LEN];
 
-		if (!strfpath(group->attribute->savedir, buf, sizeof(buf), group))
+		if (!strfpath(curr_group->attribute->savedir, buf, sizeof(buf), curr_group))
 			joinpath(buf, homedir, DEFAULT_SAVEDIR);
 		joinpath(outpath, buf, path);
 		return FALSE;
@@ -646,7 +646,7 @@ expand_save_filename(
  */
 t_bool
 post_process_files(
-	int proc_type_ch,
+	t_function proc_type_func,
 	t_bool auto_delete)
 {
 	if (num_save < 1)
@@ -658,13 +658,13 @@ post_process_files(
 #endif /* USE_CURSES */
 	my_printf("%s%s", _(txt_post_processing), cCRLF);
 
-	switch (proc_type_ch) {
-		case iKeyPProcShar:
+	switch (proc_type_func) {
+		case POSTPROCESS_SHAR:
 			post_process_sh();
 			break;
 
 		/* This is the default, eg, with AUTOSAVE */
-		case iKeyPProcYes:
+		case POSTPROCESS_YES:
 		default:
 			post_process_uud();
 			break;
@@ -713,8 +713,8 @@ post_process_uud(
 {
 	FILE *fp_in;
 	char file_out_dir[PATH_LEN];
-	int i;
 	const char *eptr;
+	int i;
 	int count;
 	int errors = 0;
 	uulist *item;
@@ -734,13 +734,13 @@ post_process_uud(
 		}
 	}
 
+#	if 0
 	/*
 	 * uudeview's "intelligent" multi-part detection
 	 * From the uudeview docs: This function is a bunch of heuristics, and I
 	 * don't really trust them... should only be called as a last resort on
 	 * explicit user request
 	 */
-#	if 0
 	UUSmerge(0);
 	UUSmerge(1);
 	UUSmerge(99);
@@ -805,13 +805,13 @@ post_process_uud(
 	void)
 {
 	FILE *fp_in;
-	char file_out_dir[PATH_LEN];
-	int i;
 	FILE *fp_out = NULL;
 	char *filename = NULL;
+	char file_out_dir[PATH_LEN];
 	char path[PATH_LEN];
 	char s[LEN], t[LEN], u[LEN];
-	int state;
+	int state = INITIAL;
+	int i;
 	mode_t mode = 0;
 
 	/*
@@ -822,10 +822,7 @@ post_process_uud(
 	t[0] = '\0';
 	u[0] = '\0';
 
-	state = INITIAL;
-
 	for (i = 0; i < num_save; i++) {
-
 		if ((fp_in = fopen(save[i].path, "r")) == NULL)
 			continue;
 
@@ -856,7 +853,7 @@ post_process_uud(
 
 						filename = name;
 						expand_save_filename(path, filename);
-						filename = strrchr(path, DIRSEP) + 1;  /* ptr to filename portion */
+						filename = strrchr(path, DIRSEP) + 1;	/* ptr to filename portion */
 						if ((fp_out = fopen(path, "w")) == NULL) {
 							perror_message(_(txt_cannot_open), path);
 							return;
@@ -1202,7 +1199,8 @@ decode_save_one(
 	/*
 	 * Decode this message part if appropriate
 	 */
-	if (!(check_save_mime_type(part, CURR_GROUP.attribute->mime_types_to_save))) {
+	if (!(check_save_mime_type(part, curr_group->attribute->mime_types_to_save))) {
+		/* TODO: skip message if saving multiple files (e.g. save 't'agged) */
 		wait_message(1, "Skipped %s/%s", content_types[part->type], part->subtype);	/* TODO: better msg */
 		return TRUE;
 	}
@@ -1287,7 +1285,7 @@ decode_save_one(
 		}
 	} else {
 		snprintf(buf, sizeof(buf), _(txt_view_attachment), savepath, content_types[part->type], part->subtype);
-		if ((i = prompt_yn(cLINES, buf, TRUE)) == 1)
+		if ((i = prompt_yn(buf, TRUE)) == 1)
 			start_viewer(part, savepath);
 		else if (i == -1) {	/* Skip rest of attachments */
 			unlink(savepath);
@@ -1304,7 +1302,7 @@ decode_save_one(
 	}
 	if (!postproc) {
 		snprintf(buf, sizeof(buf), _(txt_save_attachment), savepath, content_types[part->type], part->subtype);
-		if ((i = prompt_yn(cLINES, buf, FALSE)) != 1) {
+		if ((i = prompt_yn(buf, FALSE)) != 1) {
 			unlink(savepath);
 			if (i == -1)	/* Skip rest of attachments */
 				return FALSE;
@@ -1453,7 +1451,7 @@ decode_save_mime(
 		 * the role of a multipart part. Check to see if we want to
 		 * save text and if not, skip this part.
 		 */
-		if ((ptr->type == TYPE_MULTIPART || ((NULL != ptr->uue) && (!check_save_mime_type(ptr, CURR_GROUP.attribute->mime_types_to_save)))))
+		if ((ptr->type == TYPE_MULTIPART || ((NULL != ptr->uue) && (!check_save_mime_type(ptr, curr_group->attribute->mime_types_to_save)))))
 			continue;
 
 		if (!(decode_save_one(ptr, art->raw, postproc)))
