@@ -3,10 +3,10 @@
  *  Module    : screen.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-08-03
+ *  Updated   : 2005-02-11
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,12 +42,11 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-char mesg[LEN];
-
 #ifndef USE_CURSES
 	struct t_screen *screen;
 #endif /* !USE_CURSES */
 
+
 /*
  * Move the cursor to the lower-left of the screen, where it won't be annoying
  */
@@ -60,6 +59,33 @@ stow_cursor(
 }
 
 
+/*
+ * helper for the varius *_message() functions
+ * returns a pointer to an allocated buffer with the formated message
+ * must be freed if not needed anymore
+ */
+char *
+fmt_message(
+	const char *fmt,
+	va_list ap)
+{
+	char *msg;
+#ifdef HAVE_VASPRINTF
+
+	if (vasprintf(&msg, fmt, ap) == -1)	/* something went wrong */
+#endif /* HAVE_VASPRINTF */
+	{
+		size_t size = LEN;
+
+		msg = my_malloc(size);
+		/* TODO: realloc msg if necessary */
+		vsnprintf(msg, size, fmt, ap);
+	}
+
+	return msg;
+}
+
+
 /*
  * Centre a formatted colour message at the bottom of the screen
  */
@@ -68,6 +94,7 @@ info_message(
 	const char *fmt,
 	...)
 {
+	char *buf;
 	va_list ap;
 
 	va_start(ap, fmt);
@@ -77,9 +104,9 @@ info_message(
 	fcol(tinrc.col_message);
 #endif /* HAVE_COLOR */
 
-	vsnprintf(mesg, sizeof(mesg), fmt, ap);
-
-	center_line(cLINES, FALSE, mesg);	/* center the message at screen bottom */
+	buf = fmt_message(fmt, ap);
+	center_line(cLINES, FALSE, buf);	/* center the message at screen bottom */
+	free(buf);
 
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_normal);
@@ -99,6 +126,7 @@ wait_message(
 	const char *fmt,
 	...)
 {
+	char *buf;
 	va_list ap;
 
 	va_start(ap, fmt);
@@ -108,8 +136,9 @@ wait_message(
 	fcol(tinrc.col_message);
 #endif /* HAVE_COLOR */
 
-	vsnprintf(mesg, sizeof(mesg), fmt, ap);
-	my_fputs(mesg, stdout);
+	buf = fmt_message(fmt, ap);
+	my_fputs(buf, stdout);
+	free(buf);
 
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_normal);
@@ -132,16 +161,18 @@ error_message(
 	const char *fmt,
 	...)
 {
+	char *buf;
 	va_list ap;
 
 	va_start(ap, fmt);
 
 	errno = 0;
 	clear_message();
-	vsnprintf(mesg, sizeof(mesg), fmt, ap);
 
-	my_fputs(mesg, stderr);	/* don't use my_fprintf() here due to %format chars */
+	buf = fmt_message(fmt, ap);
+	my_fputs(buf, stderr);	/* don't use my_fprintf() here due to %format chars */
 	my_fflush(stderr);
+	free(buf);
 
 	if (cmd_line) {
 		my_fputc('\n', stderr);
@@ -165,7 +196,7 @@ perror_message(
 	const char *fmt,
 	...)
 {
-	char buf[LEN];
+	char *buf;
 	int err;
 	va_list ap;
 
@@ -174,12 +205,13 @@ perror_message(
 
 	clear_message();
 
-	vsnprintf(buf, sizeof(buf), fmt, ap);
+	if ((buf = fmt_message(fmt, ap)) != NULL) {
+		error_message("%s: Error: %s", buf, strerror(err));
+		free(buf);
+	}
 
 	va_end(ap);
 
-	error_message("%s: Error: %s", buf, strerror(err));
-
 	return;
 }
 
@@ -199,47 +231,31 @@ clear_message(
 }
 
 
-#define TRUNC_TAIL	" ..."
 void
 center_line(
 	int line,
 	t_bool inverse,
 	const char *str)
 {
-	char buffer[256];
 	int pos;
 	int len;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	int width;
-	wchar_t wbuffer[256];	/* needs same number of elements as buffer */
-	wchar_t wbuffer2[256];
-	wchar_t suffix_buf[6];	/* space for TRUNC_TAIL */
+	wchar_t *wbuffer;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-	STRCPY(buffer, str);
-
-	/* protect terminal... */
-	convert_to_printable(buffer);
-
+	len = strlen(str);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	if ((len = mbstowcs(wbuffer, buffer, ARRAY_SIZE(wbuffer) - 1)) <= 0)
-		len = strlen(buffer);
-	else
-		wbuffer[ARRAY_SIZE(wbuffer) - 1] = (wchar_t) '\0';
-	if ((width = wcswidth(wbuffer, ARRAY_SIZE(wbuffer) - 1)) <= 0)
-		width = len;
-#else
-	len = strlen(buffer);
+	if ((wbuffer = char2wchar_t(str)) != NULL) {
+		if ((width = wcswidth(wbuffer, wcslen(wbuffer) + 1)) > 0)
+			len = width;
+		free(wbuffer);
+	}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	if (!cmd_line) {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		if (cCOLS >= width)
-			pos = (cCOLS - width) / 2;
-#else
 		if (cCOLS >= len)
 			pos = (cCOLS - len) / 2;
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		else
 			pos = 1;
 
@@ -250,23 +266,14 @@ center_line(
 		}
 	}
 
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	if (width >= cCOLS) {
-		wcspart(wbuffer2, wbuffer, cCOLS - 6, ARRAY_SIZE(wbuffer2) - 5, TRUE);
-		mbstowcs(suffix_buf, TRUNC_TAIL, ARRAY_SIZE(suffix_buf) - 1);
-		wcsncat(wbuffer2, suffix_buf, 4);
-		wcstombs(buffer, wbuffer2, sizeof(buffer) - 1);
-	}
-#else
 	if (len >= cCOLS) {
-		char *buf;
+		char *buffer;
 
-		buf = my_strdup(buffer);
-		snprintf(buffer, sizeof(buffer), "%-.*s%s", cCOLS - 6, buf, TRUNC_TAIL);
-		free(buf);
-	}
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-	my_fputs(buffer, stdout);
+		buffer = strunc(str, cCOLS - 2);
+		my_fputs(buffer, stdout);
+		free(buffer);
+	} else
+		my_fputs(str, stdout);
 
 	if (cmd_line)
 		my_flush();
@@ -406,6 +413,7 @@ spin_cursor(
 }
 
 
+#define DISPLAY_FMT "%s %3d%% "
 /*
  * progressmeter in %
  */
@@ -420,7 +428,6 @@ show_progress(
 	time_t curr_time;
 	static char last_display[LEN];
 	static const char *last_txt;
-	static int last_length;
 	static int last_ratio;
 	static long last_total;
 	static time_t last_update;
@@ -431,6 +438,7 @@ show_progress(
 	static int sum;
 	static struct timeval last_time;
 	static struct timeval this_time;
+	char *display_format;
 	int time_diff;
 	int secs_left;
 	long count_diff;
@@ -441,7 +449,6 @@ show_progress(
 
 	/* If this is a new progress meter, start recalculating */
 	if ((last_txt != txt) || (last_total != total)) {
-		last_length = 0;
 		last_ratio = -1;
 		last_display[0] = '\0';
 		last_update = time(NULL) - 2;
@@ -459,11 +466,12 @@ show_progress(
 	last_update = curr_time;
 
 #ifdef HAVE_GETTIMEOFDAY
-	if (last_length == 0) {
+	display_format = my_malloc(strlen(DISPLAY_FMT) + strlen(_(txt_remaining)) + 1);
+	strcpy(display_format, DISPLAY_FMT);
+
+	if (last_ratio == -1) {
 		/* Don't print a "time remaining" this time */
-		snprintf(display, sizeof(display), "%s %3d%%", txt, ratio);
-		display[sizeof(display) - 1] = '\0';
-		last_length = strlen(txt) + 5;
+		snprintf(display, sizeof(display), display_format, txt, ratio);
 
 		/* Reset the variables */
 		sum = average = samples = 0;
@@ -505,20 +513,22 @@ show_progress(
 		if (secs_left < 0)
 			secs_left = 0;
 
-		/* TODO: -> lang.c, difficult with hardcoded last_length */
-		snprintf(display, sizeof(display), "%s %3d%% (%d:%02d remaining)", txt, ratio, secs_left / 60, secs_left % 60);
-		last_length = strlen(txt) + 21 + secs_left / 600;
+		strcat(display_format, _(txt_remaining));
+		snprintf(display, sizeof(display), display_format, txt, ratio, secs_left / 60, secs_left % 60);
 	}
+	free(display_format);
 
 	last_count = count;
 	gettimeofday(&last_time, NULL);
 
-#else /* HAVE_GETTIMEOFDAY */
+#else
 	snprintf(display, sizeof(display), "%s %3d%%", txt, ratio);
 #endif /* HAVE_GETTIMEOFDAY */
 
 	/* Only display text if it changed from last time */
 	if (strcmp(display, last_display)) {
+		char *tmp;
+
 		clear_message();
 		MoveCursor(cLINES, 0);
 
@@ -526,7 +536,13 @@ show_progress(
 		fcol(tinrc.col_message);
 #	endif /* HAVE_COLOR */
 
-		my_printf("%s", display);
+		/*
+		 * TODO: depending on the length of the newsgroup name
+		 * it's possible to cut away a great part of the progress meter
+		 * perhaps we should shorten the newsgroup name instead?
+		 */
+		my_printf("%s", sized_message(&tmp, "%s", display));
+		free(tmp);
 
 #	ifdef HAVE_COLOR
 		fcol(tinrc.col_normal);
