@@ -3,10 +3,10 @@
  *  Module    : search.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-05-15
+ *  Updated   : 2005-07-02
  *  Notes     :
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,9 +38,7 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
+
 
 /*
  * local prototypes
@@ -52,16 +50,11 @@ static int subject_search(int i, char *searchbuf);
 static int search_group(t_bool forward, int current_art, char *searchbuff, int (*search_func) (int i, char *searchbuff));
 
 
-/*
- * The search function may place error text into mesg
- */
-#define MATCH_MSG	(mesg[0] ? mesg : _(txt_no_match))
-
 /*
  * Kludge to maintain some internal state for body search
  */
-static int total_cnt = 0, curr_cnt = 0;
 int srch_lineno = -1;
+static int total_cnt = 0, curr_cnt = 0;
 
 /*
  * Used by article and body search - this saves passing around large numbers
@@ -69,7 +62,7 @@ int srch_lineno = -1;
  */
 static int srch_offsets[6];
 static int srch_offsets_size = ARRAY_SIZE(srch_offsets);
-static struct regex_cache srch_regex;
+static struct regex_cache search_regex = { NULL, NULL };
 
 
 /*
@@ -92,8 +85,7 @@ get_search_pattern(
 	if (repeat) {
 		*forward = last_forward;
 		my_strncpy(def, last_pattern, LEN);
-	}
-	else {
+	} else {
 		sprintf(tmpbuf, (*forward ? fwd_msg : bwd_msg), def);
 
 		if (!prompt_string_default(tmpbuf, def, _(txt_no_search_string), which_hist))
@@ -104,15 +96,26 @@ get_search_pattern(
 
 		/* HIST_BODY_SEARCH doesn't exist, hence i_key_search_last is set directly in search_body() */
 		if (which_hist == HIST_AUTHOR_SEARCH)
-			i_key_search_last = *forward ? iKeySearchAuthF : iKeySearchAuthB;
+			last_search = *forward ? GLOBAL_SEARCH_AUTHOR_FORWARD : GLOBAL_SEARCH_AUTHOR_BACKWARD;
 		else
-			i_key_search_last = *forward ? iKeySearchSubjF : iKeySearchSubjB;
+			last_search = *forward ? GLOBAL_SEARCH_SUBJECT_FORWARD : GLOBAL_SEARCH_SUBJECT_BACKWARD;
 	}
 
 	wait_message(0, _(txt_searching));
 
 	stow_cursor();
 
+#ifdef HAVE_UNICODE_NORMALIZATION
+	/* normalize search pattern */
+	if (IS_LOCAL_CHARSET("UTF-8")) {
+		char *tmp;
+
+		tmp = normalize(def);
+		my_strncpy(def, tmp, LEN);
+		free(tmp);
+	}
+#endif /* HAVE_UNICODE_NORMALIZATION */
+
 	if (tinrc.wildcard) {			/* ie, not wildmat() */
 		strcpy(def, quote_wild_whitespace(def));
 		return def;
@@ -130,39 +133,55 @@ get_search_pattern(
 /*
  * called by config.c
  */
-int
+enum option_enum
 search_config(
 	t_bool forward,
 	t_bool repeat,
-	int current,
-	int last)
+	enum option_enum current,
+	enum option_enum last)
 {
-	char *buf;
-	int n;
-	int incr;
-	int result = current;
+	char *pattern, *buf;
+	enum option_enum n = current;
+	enum option_enum result = current;
 
-	if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_config, HIST_CONFIG_SEARCH)))
+	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_config, HIST_CONFIG_SEARCH)))
 		return result;
 
-	incr = forward ? 1 : -1;
+	if (tinrc.wildcard && !(compile_regex(pattern, &search_regex, PCRE_CASELESS)))
+		return result;
 
-	current += incr;
-	n = current;
 	do {
-		if (n < 0)
+		if (n == 0 && !forward)
 			n = last;
 		else {
-			if (n > last)
+			if (n == last && forward)
 				n = 0;
+			else
+				n += forward ? 1 : -1;
 		}
-		if (REGEX_MATCH(_(option_table[n].txt->opt), buf, TRUE)) {
-			result = n;
-			break;
+		/* search only visible options */
+		if (option_is_visible(n)) {
+#ifdef HAVE_UNICODE_NORMALIZATION
+			if (IS_LOCAL_CHARSET("UTF-8"))
+				buf = normalize(_(option_table[n].txt->opt));
+			else
+#endif /* HAVE_UNICODE_NORMALIZATION */
+				buf = my_strdup(_(option_table[n].txt->opt));
+
+			if (match_regex(buf, pattern, &search_regex, TRUE)) {
+				result = n;
+				free(buf);
+				break;
+			}
+			free(buf);
 		}
-		n += incr;
 	} while (n != current);
+
 	clear_message();
+	if (tinrc.wildcard) {
+		FreeAndNull(search_regex.re);
+		FreeAndNull(search_regex.extra);
+	}
 	return result;
 }
 
@@ -178,8 +197,8 @@ search_active(
 	t_bool repeat)
 {
 	char *buf;
+	char *ptr;
 	char buf2[LEN];
-	char *ptr = buf2;
 	int i;
 
 	if (!selmenu.max) {
@@ -190,6 +209,9 @@ search_active(
 	if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_group, HIST_GROUP_SEARCH)))
 		return -1;
 
+	if (tinrc.wildcard && !(compile_regex(buf, &search_regex, PCRE_CASELESS)))
+		return -1;
+
 	i = selmenu.curr;
 
 	do {
@@ -210,12 +232,20 @@ search_active(
 		} else
 			ptr = active[my_group[i]].name;
 
-		if (REGEX_MATCH(ptr, buf, TRUE)) {
+		if (match_regex(ptr, buf, &search_regex, TRUE)) {
+			if (tinrc.wildcard) {
+				FreeAndNull(search_regex.re);
+				FreeAndNull(search_regex.extra);
+			}
 			return i;
 		}
 	} while (i != selmenu.curr);
 
-	info_message(MATCH_MSG);
+	if (tinrc.wildcard) {
+		FreeAndNull(search_regex.re);
+		FreeAndNull(search_regex.extra);
+	}
+	info_message(_(txt_no_match));
 	return -1;
 }
 
@@ -232,10 +262,11 @@ body_search(
 	int i,
 	char *searchbuf)
 {
-	char *line;
+	static char msg[LEN];	/* show_progress needs a constant message buffer */
+	char *line, *tmp;
 	t_openartinfo artinfo;
 
-	switch (art_open(TRUE, &arts[i], curr_group, &artinfo, FALSE)) {
+	switch (art_open(TRUE, &arts[i], curr_group, &artinfo, FALSE, NULL)) {
 		case ART_ABORT:					/* User 'q'uit */
 			art_close(&artinfo);
 			return -1;
@@ -256,11 +287,18 @@ body_search(
 	/*
 	 * Now search the body
 	 */
-	sprintf(mesg, _(txt_searching_body), ++curr_cnt, total_cnt);
-	show_progress(mesg, curr_cnt, total_cnt);
-	while ((line = tin_fgets(artinfo.cooked, FALSE)) != NULL) {
+	snprintf(msg, sizeof(msg), _(txt_searching_body), ++curr_cnt, total_cnt);
+	show_progress(msg, curr_cnt, total_cnt);
+	while ((tmp = tin_fgets(artinfo.cooked, FALSE)) != NULL) {
+#ifdef HAVE_UNICODE_NORMALIZATION
+		if (IS_LOCAL_CHARSET("UTF-8"))
+			line = normalize(tmp);
+		else
+#endif /* HAVE_UNICODE_NORMALIZATION */
+			line = my_strdup(tmp);
+
 		if (tinrc.wildcard) {
-			if (pcre_exec(srch_regex.re, srch_regex.extra, line, strlen(line), 0, 0, srch_offsets, srch_offsets_size) != PCRE_ERROR_NOMATCH) {
+			if (pcre_exec(search_regex.re, search_regex.extra, line, strlen(line), 0, 0, srch_offsets, srch_offsets_size) != PCRE_ERROR_NOMATCH) {
 				srch_lineno = i;
 				art_close(&pgart);		/* Switch the pager over to matched art */
 				pgart = artinfo;
@@ -268,6 +306,7 @@ body_search(
 				if (debug == 2)
 					fprintf(stderr, "art_switch(%p = %p)\n", (void *) &pgart, (void *) &artinfo);
 #endif /* DEBUG */
+				free(line);
 
 				return 1;
 			}
@@ -276,10 +315,12 @@ body_search(
 				srch_lineno = i;
 				art_close(&pgart);		/* Switch the pager over to matched art */
 				pgart = artinfo;
+				free(line);
 				return 1;
 			}
 		}
 		i++;
+		free(line);
 	}
 
 	if (tin_errno != 0) {			/* User abort */
@@ -288,7 +329,7 @@ body_search(
 	}
 
 	art_close(&artinfo);
-/*	info_message(MATCH_MSG); */
+/*	info_message(_(txt_no_match)); */
 	return 0;
 }
 
@@ -302,15 +343,32 @@ author_search(
 	int i,
 	char *searchbuf)
 {
-	char buf[LEN];
-	char *ptr = buf;
+	char *buf, *tmp;
 
 	if (arts[i].name == NULL)
-		ptr = arts[i].from;
-	else
-		snprintf(buf, sizeof(buf), "%s <%s>", arts[i].name, arts[i].from);
+		tmp = my_strdup(arts[i].from);
+	else {
+		size_t len = strlen(arts[i].from) + strlen(arts[i].name) + 4;
 
-	return (REGEX_MATCH(ptr, searchbuf, TRUE)) ? 1 : 0;
+		tmp = my_malloc(len);
+		snprintf(tmp, len, "%s <%s>", arts[i].name, arts[i].from);
+	}
+
+#ifdef HAVE_UNICODE_NORMALIZATION
+	if (IS_LOCAL_CHARSET("UTF-8")) {
+		buf = normalize(tmp);
+		free(tmp);
+	} else
+#endif /* HAVE_UNICODE_NORMALIZATION */
+		buf = tmp;
+
+	if (match_regex(buf, searchbuf, &search_regex, TRUE)) {
+		free(buf);
+		return 1;
+	}
+
+	free(buf);
+	return 0;
 }
 
 
@@ -323,7 +381,22 @@ subject_search(
 	int i,
 	char *searchbuf)
 {
-	return (REGEX_MATCH(arts[i].subject, searchbuf, TRUE)) ? 1 : 0;
+	char *buf;
+
+#ifdef HAVE_UNICODE_NORMALIZATION
+	if (IS_LOCAL_CHARSET("UTF-8"))
+		buf = normalize(arts[i].subject);
+	else
+#endif /* HAVE_UNICODE_NORMALIZATION */
+		buf = my_strdup(arts[i].subject);
+
+	if (match_regex(buf, searchbuf, &search_regex, TRUE)) {
+		free(buf);
+		return 1;
+	}
+
+	free(buf);
+	return 0;
 }
 
 
@@ -337,13 +410,19 @@ search_group(
 	char *searchbuff,
 	int (*search_func) (int i, char *searchbuff))
 {
-	int i;
+	int i, ret;
 
 	if (grpmenu.curr < 0) {
 		info_message(_(txt_no_arts));
 		return -1;
 	}
 
+	/*
+	 * precompile if we're using full regex
+	 */
+	if (tinrc.wildcard && !(compile_regex(searchbuff, &search_regex, PCRE_CASELESS)))
+		return -1;
+
 	i = current_art;
 
 	do {
@@ -356,10 +435,16 @@ search_group(
 		}
 
 		/* Only search displayed articles */
-		if (CURR_GROUP.attribute->show_only_unread && arts[i].status != ART_UNREAD)
+		if (curr_group->attribute->show_only_unread && arts[i].status != ART_UNREAD)
 			continue;
 
-		switch (search_func(i, searchbuff)) {
+		ret = search_func(i, searchbuff);
+		if (tinrc.wildcard && (ret == 1 || ret == -1)) {
+			/* we will exit soon, clean up */
+			FreeAndNull(search_regex.re);
+			FreeAndNull(search_regex.extra);
+		}
+		switch (ret) {
 			case 1:								/* Found */
 				clear_message();
 				return i;
@@ -369,6 +454,10 @@ search_group(
 		}
 	} while (i != current_art);
 
+	if (tinrc.wildcard) {
+		FreeAndNull(search_regex.re);
+		FreeAndNull(search_regex.extra);
+	}
 	info_message(_(txt_no_match));
 	return -1;
 }
@@ -380,22 +469,29 @@ search_group(
  */
 int
 search(
-	int key,
+	t_function func,
 	int current_art,
-	t_bool forward,
 	t_bool repeat)
 {
 	char *buf = NULL;
 	int (*search_func) (int i, char *searchbuff) = author_search;
+	t_bool forward;
 
-	switch (key) {
-		case SEARCH_SUBJ:
+	if (func == GLOBAL_SEARCH_SUBJECT_FORWARD || func == GLOBAL_SEARCH_AUTHOR_FORWARD)
+		forward = TRUE;
+	else
+		forward = FALSE;
+
+	switch (func) {
+		case GLOBAL_SEARCH_SUBJECT_FORWARD:
+		case GLOBAL_SEARCH_SUBJECT_BACKWARD:
 			if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_subject, HIST_SUBJECT_SEARCH)))
 				return -1;
 			search_func = subject_search;
 			break;
 
-		case SEARCH_AUTH:
+		case GLOBAL_SEARCH_AUTHOR_FORWARD:
+		case GLOBAL_SEARCH_AUTHOR_BACKWARD:
 		default:
 			if (!(buf = get_search_pattern(&forward, repeat, _(txt_author_search_forwards), _(txt_author_search_backwards), tinrc.default_search_author, HIST_AUTHOR_SEARCH)))
 				return -1;
@@ -421,27 +517,23 @@ search_article(
 	int reveal_ctrl_l_lines,
 	FILE *fp)
 {
-	char *pattern, *ptr;
-	int i;
-	struct regex_cache srch;
+	char *pattern, *ptr, *tmp;
+	int i = start_line;
+	int tmp_srch_offsets[2];
+	t_bool wrap = FALSE;
+	t_bool match = FALSE;
 
 	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_art, HIST_ART_SEARCH)))
-		return FALSE;
+		return 0;
 
-	if (tinrc.wildcard && !(compile_regex(pattern, &srch, PCRE_CASELESS)))
+	if (tinrc.wildcard && !(compile_regex(pattern, &search_regex, PCRE_CASELESS)))
 		return -1;
 
 	srch_lineno = -1;
-	i = start_line;
 
 	forever {
-		if (forward) {
-			if (++i == lines)
-				break;
-		} else {
-			if (--i < 0)
-				break;
-		}
+		if (i == start_line && wrap)
+			break;
 
 		/*
 		 * TODO: consider not searching some line types?
@@ -453,23 +545,72 @@ search_article(
 		if ((line[i].flags&C_CTRLL) && i > reveal_ctrl_l_lines)
 			break;
 
-		ptr = tin_fgets(fp, FALSE);
+		tmp = tin_fgets(fp, FALSE);
+		if (!forward && srch_offsets[0] >= 0) {
+			tmp[srch_offsets[0]] = '\0';	/* ignore anything on this line after the last match */
+			srch_offsets[1] = 0;	/* start backwards search at the beginning of the line */
+		}
+
+#ifdef HAVE_UNICODE_NORMALIZATION
+		if (IS_LOCAL_CHARSET("UTF-8"))
+			ptr = normalize(tmp);
+		else
+#endif /* HAVE_UNICODE_NORMALIZATION */
+			ptr = my_strdup(tmp);
 
 		if (tinrc.wildcard) {
-			if (pcre_exec(srch.re, srch.extra, ptr, strlen(ptr), 0, 0,
-								srch_offsets, srch_offsets_size) != PCRE_ERROR_NOMATCH) {
+			while (pcre_exec(search_regex.re, search_regex.extra, ptr, strlen(ptr), srch_offsets[1], 0, srch_offsets, srch_offsets_size) != PCRE_ERROR_NOMATCH) {
+				match = TRUE;
+				if (forward)
+					break;
+				else {
+					tmp_srch_offsets[0] = srch_offsets[0];
+					tmp_srch_offsets[1] = srch_offsets[1];
+				}
+			}
+			if (match) {
+				if (!forward) {
+					srch_offsets[0] = tmp_srch_offsets[0];
+					srch_offsets[1] = tmp_srch_offsets[1];
+				}
 				srch_lineno = i;
+				FreeAndNull(search_regex.re);
+				FreeAndNull(search_regex.extra);
+				free(ptr);
 				return i;
 			}
 		} else {
 			if (wildmatpos(ptr, pattern, TRUE, srch_offsets, srch_offsets_size)) {
 				srch_lineno = i;
+				free(ptr);
 				return i;
 			}
 		}
+		free(ptr);
+
+		if (forward) {
+			if (i >= lines - 1) {
+				i = 0;
+				wrap = TRUE;
+			} else
+				i++;
+		} else {
+			if (i <= 0) {
+				i = lines - 1;
+				wrap = TRUE;
+			} else
+				i--;
+		}
+
+		/* search at the beginning of the line */
+		srch_offsets[1] = 0;
 	}
 
 	info_message(_(txt_no_match));
+	if (tinrc.wildcard) {
+		FreeAndNull(search_regex.re);
+		FreeAndNull(search_regex.extra);
+	}
 	return -1;
 }
 
@@ -490,7 +631,7 @@ search_body(
 {
 	char *buf;
 	int i;
-	t_bool forward_fake;
+	t_bool forward_fake = TRUE;
 
 	if (!(buf = get_search_pattern(
 			&forward_fake,				/* we pass a dummy var since body search has no `forward' */
@@ -501,7 +642,7 @@ search_body(
 			HIST_ART_SEARCH
 	))) return -1;
 
-	i_key_search_last = iKeySearchBody;	/* store last search type for repeated search */
+	last_search = GLOBAL_SEARCH_BODY;	/* store last search type for repeated search */
 	total_cnt = curr_cnt = 0;			/* Reset global counter of articles done */
 
 	/*
@@ -517,12 +658,6 @@ search_body(
 		}
 	}
 
-	/*
-	 * Pre-compile if we're using full regex
-	 */
-	if (tinrc.wildcard && !(compile_regex(buf, &srch_regex, PCRE_CASELESS)))
-		return -1;
-
 	srch_lineno = -1;
 	return search_group(1, current_art, buf, body_search);
 }
@@ -544,3 +679,15 @@ get_search_vectors(
 	srch_lineno = -1;			/* We can only retrieve this info once */
 	return i;
 }
+
+
+/*
+ * Reset offsets so that the next search starts at the beginning of the line.
+ * This function is needed to access srch_offsets from within other modules.
+ */
+void
+reset_srch_offsets(
+	void)
+{
+	srch_offsets[0] = srch_offsets[1] = 0;
+}
