@@ -3,10 +3,10 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-05-15
+ *  Updated   : 2004-06-21
  *  Notes     : signal handlers for different modes and window resizing
  *
- * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -106,6 +106,7 @@ static void _CDECL signal_handler(SIG_ARGS);
 #endif /* SIGTSTP */
 
 int signal_context = cMain;
+int input_context = cNone;
 int need_resize = cNo;
 /*
  * # lines of non-static data available for display
@@ -157,9 +158,9 @@ static const struct {
 #	ifdef SIGTERM
 	{ SIGTERM,	"SIGTERM" },	/* termination */
 #	endif /* SIGTERM */
-#	ifdef SIGWINCH
+#	if defined(SIGWINCH) && !(defined(USE_CURSES) && defined(KEY_RESIZE))
 	{ SIGWINCH,	"SIGWINCH" },	/* window-size change */
-#	endif /* SIGWINCH */
+#	endif /* SIGWINCH && !(USE_CURSES && KEY_RESIZE) */
 };
 #endif /* !__LCLINT__ */
 
@@ -209,9 +210,9 @@ allow_resize(
 	if (!allow)
 		sa.sa_flags |= SA_RESTART;
 #	endif /* SA_RESTART */
-#	ifdef SIGWINCH
+#	if defined(SIGWINCH) && !(defined(USE_CURSES) && defined(KEY_RESIZE))
 	sigaction(SIGWINCH, &sa, &osa);
-#	endif /* SIGWINCH */
+#	endif /* SIGWINCH && !(USE_CURSES && KEY_RESIZE) */
 #	ifdef SIGTSTP
 	sigaction(SIGTSTP, &sa, &osa);
 #	endif /* SIGTSTP */
@@ -275,28 +276,46 @@ handle_resize(
 			ClearScreen();
 			show_art_msg(curr_group->name);
 			break;
+
 		case cConfig:
-			refresh_config_page(-1);
+			refresh_config_page(SIGNAL_HANDLER);
 			break;
+
 		case cFilter:
 			refresh_filter_menu();
 			break;
+
 		case cInfopager:
 			display_info_page(0);
 			break;
+
 		case cGroup:
 		case cSelect:
 		case cThread:
 			ClearScreen();
 			currmenu->redraw();
 			break;
+
 		case cPage:
 			resize_article(TRUE, &pgart);
 			draw_page(curr_group->name, 0);
 			break;
+
 		case cMain:
 			break;
 	}
+	switch (input_context) {
+		case cGetline:
+			gl_redraw();
+			break;
+
+		case cPromptSLK:
+			prompt_slk_redraw();
+			break;
+
+		default:
+			break;
+	}
 	my_fflush(stdout);
 #endif /* SIGWINCH || SIGTSTP */
 }
@@ -347,12 +366,10 @@ signal_handler(
 	switch (sig) {
 #ifdef SIGINT
 		case SIGINT:
-#	if !defined(M_AMIGA) && !defined(__SASC)
 			if (!batch_mode) {
 				RESTORE_HANDLER(sig, signal_handler);
 				return;
 			}
-#	endif /* !M_AMIGA && !__SASC */
 			break;
 #endif /* SIGINT */
 
@@ -400,20 +417,24 @@ signal_handler(
 #ifdef SIGTERM
 		case SIGTERM:
 #endif /* SIGTERM */
+#if defined(SIGHUP) || defined(SIGUSR1) || defined(SIGTERM)
 			dangerous_signal_exit = TRUE;
 			tin_done(-sig);
 			/* NOTREACHED */
 			break;
+#endif /* SIGHUP || SIGUSR1 || SIGTERM */
 
 #ifdef SIGBUS
 		case SIGBUS:
-#endif /*S IGBUS */
+#endif /* SIGBUS */
 #ifdef SIGSEGV
 		case SIGSEGV:
 #endif /* SIGSEGV */
+#if defined(SIGBUS) || defined(SIGSEGV)
 			my_fprintf(stderr, _(txt_send_bugreport), tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME, bug_addr);
 			my_fflush(stderr);
 			break;
+#endif /* SIGBUS || SIGSEGV */
 
 		default:
 			break;
@@ -444,9 +465,9 @@ set_signal_catcher(
 		sigdisp(SIGTSTP, flag ? signal_handler : SIG_DFL);
 #endif /* SIGTSTP */
 
-#ifdef SIGWINCH
+#if defined(SIGWINCH) && !(defined(USE_CURSES) && defined(KEY_RESIZE))
 	sigdisp(SIGWINCH, flag ? signal_handler : SIG_DFL);
-#endif /* SIGWINCH */
+#endif /* SIGWINCH && !(USE_CURSES && KEY_RESIZE) */
 }
 
 
@@ -529,10 +550,6 @@ set_win_size(
 			*num_cols = win.ws_col;
 	}
 #		else
-#			ifdef M_AMIGA
-	AmiGetWinSize(num_lines, num_cols);
-	(*num_lines)--;
-#			endif /* M_AMIGA */
 #		endif /* TIOCGWINSZ */
 #	endif /* TIOCGSIZE */
 
@@ -542,11 +559,6 @@ set_win_size(
 
 #endif /* HAVE_XCURSES */
 
-	set_subj_from_size(*num_cols);
-
-	/* FIXME: values do differ for different languages */
-	RIGHT_POS = *num_cols - 20;
-	MORE_POS = *num_cols - 15;
 	set_noteslines(*num_lines);
 	return (*num_lines != old_lines || *num_cols != old_cols);
 }
