@@ -3,10 +3,10 @@
  *  Module    : tags.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1999-12-06
- *  Updated   : 2003-07-19
+ *  Updated   : 2005-05-10
  *  Notes     : Split out from other modules
  *
- * Copyright (c) 1999-2003 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2005 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -85,8 +85,8 @@ look_for_multipart_info(
 	int *offset)
 {
 	MultiPartInfo tmp;
-	char *subj = (char *) 0;
-	char *pch = (char *) 0;
+	char *subj;
+	char *pch;
 
 	*offset = 0;
 
@@ -137,7 +137,7 @@ get_multiparts(
 {
 	MultiPartInfo tmp, tmp2;
 	MultiPartInfo *info = 0;
-	int i = 0;
+	int i;
 	int part_index;
 
 	/* entry assertions */
@@ -208,7 +208,7 @@ tag_multipart(
 	int base_index)
 {
 	MultiPartInfo *info = 0;
-	int i = 0;
+	int i;
 	const int qty = get_multiparts(base_index, &info);
 
 	/* check for failure... */
@@ -253,7 +253,7 @@ line_is_tagged(
 {
 	int code = 0;
 
-	if (CURR_GROUP.attribute->thread_arts) {
+	if (curr_group->attribute->thread_arts) {
 		int i;
 		for (i = n; i >= 0; i = arts[i].thread) {
 			if (arts[i].tagged > code)
@@ -354,6 +354,7 @@ set_range(
 	int curr)
 {
 	char *range;
+	char *prompt;
 	int artnum;
 	int i;
 	int depth;
@@ -380,10 +381,13 @@ set_range(
 #if 0
 	error_message("Min=[%d] Max=[%d] Cur=[%d] DefRng=[%s]", min, max, curr, range);
 #endif /* 0 */
-	sprintf(mesg, _(txt_enter_range), range);
+	prompt = fmt_string(_(txt_enter_range), range);
 
-	if (!(prompt_string_default(mesg, range, _(txt_range_invalid), HIST_OTHER)))
+	if (!(prompt_string_default(prompt, range, _(txt_range_invalid), HIST_OTHER))) {
+		free(prompt);
 		return FALSE;
+	}
+	free(prompt);
 
 	/*
 	 * Parse range string
@@ -465,19 +469,20 @@ parse_range(
 	*range_end = -1;
 
 	while (*ptr && state != DONE) {
-		if (isdigit(*ptr)) {
+		if (isdigit((int) *ptr)) {
 			if (state == FINDMAX) {
 				*range_end = atoi(ptr);
 				state = DONE;
 			} else
 				*range_start = atoi(ptr);
-			while (isdigit(*ptr))
+			while (isdigit((int) *ptr))
 				ptr++;
 		} else {
 			switch (*ptr) {
 				case '-':
 					state = FINDMAX;
 					break;
+
 				case '.':
 					if (state == FINDMAX) {
 						*range_end = curr;
@@ -485,12 +490,14 @@ parse_range(
 					} else
 						*range_start = curr;
 					break;
+
 				case '$':
 					if (state == FINDMAX) {
 						*range_end = max;
 						state = DONE;
 					}
 					break;
+
 				default:
 					break;
 			}
@@ -519,12 +526,12 @@ do_auto_select_arts(
 #	ifdef DEBUG_NEWSRC
 			debug_print_comment("group.c: X command");
 #	endif /* DEBUG_NEWSRC */
-			art_mark(&CURR_GROUP, &arts[i], ART_READ);
+			art_mark(curr_group, &arts[i], ART_READ);
 			arts[i].zombie = TRUE;
 		}
 	}
-	if (CURR_GROUP.attribute->show_only_unread)
-		find_base(&CURR_GROUP);
+	if (curr_group->attribute->show_only_unread)
+		find_base(curr_group);
 
 	grpmenu.curr = 0;
 	show_group_page();
@@ -543,12 +550,12 @@ undo_auto_select_arts(
 #	ifdef DEBUG_NEWSRC
 			debug_print_comment("group.c: + command");
 #	endif /* DEBUG_NEWSRC */
-			art_mark(&CURR_GROUP, &arts[i], ART_UNREAD);
+			art_mark(curr_group, &arts[i], ART_UNREAD);
 			arts[i].zombie = FALSE;
 		}
 	}
-	if (CURR_GROUP.attribute->show_only_unread)
-		find_base(&CURR_GROUP);
+	if (curr_group->attribute->show_only_unread)
+		find_base(curr_group);
 
 	grpmenu.curr = 0;	/* do we want this? */
 	show_group_page();
@@ -578,7 +585,7 @@ arts_selected(
 	int i;
 
 	for_each_art(i) {
-		 if (arts[i].selected)
+		if (arts[i].selected)
 			return TRUE;
 	}
 
